/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<stdexcept>
#include"../common/numerals.h"
#include"../common/bitboard.h"
using std::cout;
using std::cerr;
using std::endl;
using std::flush;
using std::dec;
using std::hex;
using std::logic_error;


//-------------------------------------------------------
//
//           unittest bitboard:
//

void test_bitboard()
{
  bool r=true;

  {
    BitBoard x,y;
    x.set_square(5);
    y.set_square(5);
    if(x!=y){r=false;}
  }


  {
    BitBoard x;
    x.set_square(a1);
    x.set_square(e5);
    x.set_square(h8);
    vector<Square> vs;
    vs=x.get_squares();
    if(vs.size()!=3){r=false;}
    x.clear();
    vs=x.get_squares();
    if(vs.size()!=0){r=false;}
  }


  {
    BitBoard x;
    x.set_square(a1);
    x.set_square(e5);
    x.set_square(h8);
    if(x.popcount()!=3){r=false;std::cerr << "error popcount: " << x.popcount() << endl;}
  }

  {
    BitBoard x;
    x.set_square(b1);
    x.set_square(d5);
    x.set_square(f8);
    x.set_square(g8);
    if(x.popcount()!=4){r=false;}
  }

  {
    BitBoard x;
    x.set_square(a1);
    x.set_square(b2);
    x.set_square(c3);
    x.set_square(d4);
    if(BitBoard::to_ull(x)!=0x08040201){r=false;}
    x.flip();
    if(BitBoard::to_ull(x)!=0x0102040800000000ull){r=false;}
  }

  {
    BitBoard x;
    x.set_square(a1);
    x.set_square(b2);
    x.set_square(c3);
    x.set_square(d4);
    if(BitBoard::to_ull(x)!=0x08040201){r=false;}
    x.flop();
    if(BitBoard::to_ull(x)!=0x0000000010204080ull){r=false;}
  }

  {
    BitBoard x,y;
    x=BitBoard(0x0000FFFF00000000ull);
    if(!x){r=false;cerr << "x=false" << endl;}
    y=x;y&=static_cast<BitBoard>(0x00000000FFFF0000ull);
    if(y){r=false;cerr << "error 1: y=" << y << endl;}
    y=x;y|=static_cast<BitBoard>(0x00000000FFFF0000ull);
    if(BitBoard::to_ull(y)!=0x0000FFFFFFFF0000ull){r=false;cerr << "error 2: y=" << hex << y << dec << endl;}
  }

  
  {
    BitBoard x;
    for(int i=0;i<64;i++){
      uint64_t s=1ull<<i;
      x=BitBoard(s);
      if(x.get_first_square().id()!=i){r=false;}
    }
  }
  
  {
    BitBoard x;
    for(int i=0;i<64;i++){
      uint64_t s=7ull<<i;
      x=BitBoard(s);
      if(x.get_first_square().id()!=i){r=false;}
    }
  }

  cout << "Test BitBoard ........................................." << flush;
  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






