/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../hashtable/bucket.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest Bucket:
//

void test_bucket()
{
  cout << "Test Bucket ..........................................." << flush;
  bool r=true;

  {
    TTrecord A;
    uint64_t k=0xaaaaull;
    A.set_key(k);
    A.set_score(-200,0);
    A.set_bound(exact_value);
    A.set_draft(20);

    TTrecord B;
    k=0xaaabull;
    B.set_key(k);
    B.set_score(-50,0);
    B.set_bound(lowerbound);
    B.set_draft(19);

    TTrecord C;
    k=0xaaacull;
    C.set_key(k);
    C.set_score(50,0);
    C.set_bound(upperbound);
    C.set_draft(18);

    TTrecord D;
    k=0xaaadull;
    D.set_key(k);
    D.set_score(100,0);
    D.set_bound(exact_value);
    D.set_draft(17);

    TTrecord E;
    k=0xaaaeull;
    E.set_key(k);
    E.set_score(200,0);
    E.set_bound(exact_value);
    E.set_draft(16);

    TTrecord F;
    k=0xaaaaull;
    F.set_key(k);
    F.set_score(200,0);
    F.set_bound(exact_value);
    F.set_draft(16);

    TTrecord G;
    k=0xaaaaull;
    G.set_key(k);
    G.set_score(200,0);
    G.set_bound(exact_value);
    G.set_draft(21);

    Bucket buk;
    buk.write(A);
    buk.write(B);
    buk.write(C);
    buk.write(D);
    buk.write(E);
    buk.write(F);
    buk.write(G);
  
    if(buk.read(C) && C.get_score(0)!=50)r=false;
    if(!r){std::cerr << "error1" << endl;}
    if(buk.read(A) && A.get_draft()!=21)r=false;
    if(!r){std::cerr << "error2" << endl;}
    if(buk.read(E) && E.get_key()==0)r=false;
    if(buk.read(E) && E.get_score(7)!=200)r=false;
    if(!r){std::cerr << "error3" << endl;}
    if(buk.read(G) && G.get_draft()!=21)r=false;
  }

  {
    TTrecord A1;
    uint64_t k=0xaaaaull;
    A1.set_key(k);
    A1.set_score(100,0);
    A1.set_bound(exact_value);
    A1.set_draft(1);
    
    TTrecord A2;
    k=0xaaaaull;
    A2.set_key(k);
    A2.set_score(200,0);
    A2.set_bound(exact_value);
    A2.set_draft(2);
    
    TTrecord A3;
    k=0xaaaaull;
    A3.set_key(k);
    A3.set_score(300,0);
    A3.set_bound(exact_value);
    A3.set_draft(3);
    
    TTrecord A4;
    k=0xaaaaull;
    A4.set_key(k);
    A4.set_score(400,0);
    A4.set_bound(exact_value);
    A4.set_draft(1);
    
    Bucket buk;
    buk.write(A1);
    buk.write(A2);
    buk.write(A3);
    buk.write(A4);

    if(buk.read(A1) && A1.get_draft()!=1)r=false;
    if(buk.read(A1) && A1.get_score(0)!=400)r=false;

  }
  

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






