/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../eval/materialindex.h"
#include"../movegen/movegenerator.h"
#include"../eval/evaluation.h"
#include"../hashtable/transposition.h"
#include"../search/analyze.h"
#include"../book/bkmove.h"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest chess position:
//

void test_cposition()
{
  bool r=true;

  cout << "Test ChessPosition ...................................." << flush;

  Tables tbs;
  ChessPosition cp(tbs);


  //--------------------------------------------------------
  {
    constexpr short castle_q=0x0100;
    cp.newgame();
    cp.make_move(c2,c4);
    cp.make_move(b8,c6);
    cp.make_move(d1,a4);
    cp.make_move(d7,d6);
    cp.make_move(a4,a7);
    cp.make_move(c8,e6);
    cp.make_move(d2,d4);
    cp.make_move(d8,d7);
    cp.make_move(a7,a8);
    if(cp.castlingrights()&castle_q){r=false;}
    if(cp.initial_position()!="rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"){r=false;}

    ChessPosition cp2(tbs);
    cp2.fen(cp.fen());
    ChessPosition cp3(cp2);
    if(cp3.initial_position()!=cp2.fen()){r=false;}
    ChessPosition cp4(tbs);cp4=cp2;
    if(cp4.initial_position()!=cp2.fen()){r=false;}
    ChessPosition cp5{cp};
    if(cp5.initial_position()!=cp.initial_position()){r=false;}
    if(cp5.moves_history()!=cp.moves_history()){r=false;}

    cp4=cp;
    if(cp.initial_position()!=cp4.initial_position()){r=false;}
    if(cp.moves_history()!=cp4.moves_history()){r=false;}
    
  }
  if(!r){cout << "error 1" << endl;}


  {
    string fen1,fen2,epd;
    cp.newgame();
    fen1=cp.fen();
    epd=cp.epd();
    cp.fen(epd);
    fen2=cp.fen();
    if(fen1!=fen2){r=false;}
  }

  if(!r){cout << "error 2" << endl;throw logic_error("error 2");}  

  {
    cp.newgame();
    int dfz=cp.get_dfz();
    if(dfz!=0){r=false;}
    cp.make_move(e2,e4);
    cp.make_move(e7,e5);
    dfz=cp.get_dfz();
    if(dfz!=0){r=false;}
    cp.make_move(g1,f3);
    cp.make_move(b8,c6);
    dfz=cp.get_dfz();
    if(dfz!=2){r=false;}
    cp.make_move(f3,e5);
    dfz=cp.get_dfz();
    if(dfz!=0){r=false;}
  }

     
  //------------------------ chess960() --------------------------------------
  cp.set_chess960(true);
  {
    cp.newgame960();
    //std::cerr << cp.fen() << endl;    
    cp.newgame();
    if(cp.fen()!="rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w AHah - 0 1"){r=false;}
  }

  {
    cp.fen("qbbrkrnn/pppppppp/8/8/8/8/PPPPPPPP/QBBRKRNN w DFdf - 0 1");
    Move m(e1,f1,castling,WhiteKing(),NoPiece());
    if(m.code()!=castling){r=false;}
    if(m.capture().rook()){r=false;}
    cp.make_move(g1,f3);
    cp.make_move(g8,f6);
    cp.make_move(m);
    if(cp.fen()!="qbbrkr1n/pppppppp/5n2/8/8/5N2/PPPPPPPP/QBBR1RKN b df - 3 2"){
      cout << endl << "chess960 error 1  " << cp.fen() << endl;
      r=false;
    }
  }
  
  {
    Move m(c1,b1,castling,WhiteKing(),NoPiece());
    cp.fen("qrk1r2n/pppppppp/8/8/8/8/PPPPPPPP/QRK1R2N w BEbe - 0 1");
    cp.make_move(m);
    Move m2(c8,b8,castling,BlackKing(),NoPiece());
    cp.make_move(m2);
    if(cp.fen()!="q1krr2n/pppppppp/8/8/8/8/PPPPPPPP/Q1KRR2N w - - 2 2"){
      cout << endl << "chess960 error 2  " << cp.fen() << endl;
      r=false;
    }
  }
  

  {
    Move m(c1,e1,castling,WhiteKing(),NoPiece());
    cp.fen("qrk1r2n/pppppppp/8/8/8/8/PPPPPPPP/QRK1R2N w BEbe - 0 1");
    cp.make_move(m);
    Move m2(c8,e8,castling,BlackKing(),NoPiece());
    cp.make_move(m2);
    if(cp.fen()!="qr3rkn/pppppppp/8/8/8/8/PPPPPPPP/QR3RKN w - - 2 2"){
      cout << endl << "chess960 error 3  " << cp.fen() << endl;
      r=false;
    }
  }

  {
    Move m(e1,g1,castling,WhiteKing(),NoPiece());
    cp.fen("bbrqknrn/pppppppp/8/8/8/8/PPPPPPPP/BBRQKNRN w CGcg - 0 1");
    cp.make_move(f1,g3);
    cp.make_move(f8,g6);
    cp.make_move(m);
    Move m2(e8,g8,castling,BlackKing(),NoPiece());
    cp.make_move(m2);
    if(cp.fen()!="bbrq1rkn/pppppppp/6n1/8/8/6N1/PPPPPPPP/BBRQ1RKN w - - 4 3"){
      cout << endl << "chess960 error 4  " << cp.fen() << endl;
      r=false;
    }
  }
  
  {
    cp.fen("nnrkqbbr/pppppppp/8/8/8/8/PPPPPPPP/NNRKQBBR w KQkq - 0 1");
    string m{"d1c1"};
    cp.make_move(cp.convert_to_move(m));
    if(cp.fen()!="nnrkqbbr/pppppppp/8/8/8/8/PPPPPPPP/NNKRQBBR b ch - 1 1"){
      cout << endl << "chess960 error 5  " << cp.fen() << endl;
      r=false;
    }
  }
  
  {
    cp.fen("nnrkqbbr/pppppppp/8/8/8/8/PPPPPPPP/NNRKQBBR w CHch - 0 1");
    string m{"d1c1"};
    cp.make_move(cp.convert_to_move(m));
    if(cp.fen()!="nnrkqbbr/pppppppp/8/8/8/8/PPPPPPPP/NNKRQBBR b ch - 1 1"){
      cout << endl << "chess960 error 6  " << cp.fen() << endl;
      r=false;
    }
  }
  cp.set_chess960(false);

  /*
  {
    cout << endl;
    for(int i=0;i<960;i++){
      cp.newgame960(i);
      cout << cp.epd() << endl;
    }
  }
  */

  if(!r){goto exit_test;}
  //-------------------------------------------------------------------------------------


  {
    cp.newgame();
    cp.make_move(e2,e3);
    cp.make_move(e7,e6);
    cp.make_move(d1,f3);
    if(!cp.white_attacks(Square(e3).bitboard())){r=false;}
    if(!cp.white_attacks(Square(c6).bitboard())){r=false;}
    if(!cp.white_attacks(Square(b7).bitboard())){r=false;}
    if(cp.white_attacks(Square(a8).bitboard())){r=false;}
    if(!cp.white_attacks(Square(a2).bitboard())){r=false;}
    cp.make_move(h7,h6);
    cp.make_move(b1,c3);
    if(!cp.white_attacks(Square(b5).bitboard())){r=false;}
    if(!cp.white_attacks(Square(e2).bitboard())){r=false;}
    cp.make_move(a7,a6);
    cp.make_move(b2,b3);
    cp.make_move(g8,f6);
    cp.make_move(c1,b2);
    if(!cp.white_attacks(Square(a1).bitboard())){r=false;}
  }


  {
    cp.newgame();
    cp.make_move(b2,b3);
    cp.make_move(b7,b6);
    cp.make_move(c1,b2);
    cp.make_move(c8,b7);
    cp.make_move(a2,a4);
    cp.make_move(a7,a5);
    cp.make_move(a1,a3);
    cp.make_move(a8,a5);
    cp.make_move(b2,a1);
    if(!cp.white_attacks(Square(b2).bitboard())){r=false;}
  }

  {
    cp.clear();
    string s="4k3/8/8/8/8/8/4P3/4R2K w - - 0 40";
    cp.fen(s);
    if(cp.check()){r=false;}
    cp.make_move(e2,e4);
    if(cp.check()){r=false;}
    if(cp.initial_position()!="4k3/8/8/8/8/8/4P3/4R2K w - - 0 40"){r=false;}
  }
  
  {
    cp.clear();
    string s="rnb1kbnr/ppp1pppp/8/6q1/8/2N2N2/PPPPBPPP/R1BQK2R b KQkq - 5 5";
    cp.fen(s);
    ChessPosition cp2(cp);
    if(cp!=cp2){
      r=false;
      cout << "reason 4.4" << endl;
      throw logic_error("reason 4.5");
    }
  }

  
  {
    cp.clear();
    string s="r3k2r/pppqbppp/2np1n2/4p1B1/4P1b1/2NP1N2/PPPQBPPP/R3K2R w KQkq - 8 8";
    cp.fen(s);
    cp.make_move(e1,g1);
    cp.make_move(e8,g8);
    cp.unmake_move();
    cp.unmake_move();
    cp.make_move(e1,c1);
    cp.make_move(e8,c8);
    cp.unmake_move();
    cp.unmake_move();
    if(cp.fen()!=s){r=false;}
  }


  {
    cp.clear();
    string s="R2nk3/1r1nb3/8/1BR5/1KN4r/8/8/4Q3 w - - 0 1";
    cp.fen(s);
    cp.find_pinned_pieces();
    BitBoard pp{cp.pinned_pieces()};
    BitBoard x;
    x.set_square(d8);
    x.set_square(d7);
    x.set_square(e7);
    x.set_square(b5);
    x.set_square(c5);
    x.set_square(c4);
    if(x!=pp){r=false;}
    if(pp.popcount()!=6){r=false;}
  }


  {
    cp.clear();
    cp.newgame();
    Square bkf=cp.black_kingsfield();
    if(bkf.name()!="e8"){r=false;}
    Square wkf=cp.white_kingsfield();
    if(wkf.name()!="e1"){r=false;}
    cp.make_move(e2,e4);
    cp.make_move(e7,e5);
    cp.make_move(e1,e2);
    cp.make_move(e8,e7);
    bkf=cp.black_kingsfield();
    if(bkf.name()!="e7"){r=false;}
    wkf=cp.white_kingsfield();
    if(wkf.name()!="e2"){r=false;}    
  }


  {
    string s="rn1qkbnr/p3p1p1/2p5/3P1p2/P1NP4/1Q6/1P3pPP/R1B1KB1R w KQkq - 0 14";
    cp.fen(s);
    BitBoard ba=cp.black_attackers(Square(e1).bitboard());
    if(ba.popcount()!=1){r=false;}
  }

  
  {
    string s="k3r3/2b2n2/5p2/1q2n2R/3P4/3N4/4Q2B/K7 w - - 0 1";
    cp.fen(s);
    Square wsa=cp.white_weakest_attacker(e5);
    if(wsa!=d4){r=false;}
    
    if(!r){cerr << "test1 white weakest attacker " << endl;}

    cp.make_move(d4,e5);
    cp.make_move(f6,e5);
    wsa=cp.white_weakest_attacker(e5);
    if(wsa!=d3){r=false;}

    if(!r){cerr << "test2 white weakest attacker " << endl;}

    cp.make_move(d3,e5);
    cp.make_move(f7,e5);
    wsa=cp.white_weakest_attacker(e5);
    if(wsa!=h2){r=false;}

    if(!r){cerr << "test3 white weakest attacker " << endl;}

    cp.make_move(h2,e5);
    cp.make_move(c7,e5);
    wsa=cp.white_weakest_attacker(e5);
    if(wsa!=h5){r=false;}

    cp.make_move(h5,e5);
    cp.make_move(e8,e5);
    wsa=cp.white_weakest_attacker(e5);
    if(wsa!=e2){r=false;}
    
  }

  
  //--------------- test moves history -------------------
  {
    cp.newgame();
    cp.make_move(b2,b3);
    cp.make_move(b7,b6);
    cp.make_move(c1,b2);
    cp.make_move(c8,b7);
    cp.make_move(a2,a4);
    cp.make_move(a7,a5);
    cp.make_move(a1,a3);
    cp.make_move(a8,a5);
    cp.make_move(b2,a1);
    if(cp.moves_history().size()!=9){r=false;}
    if(cp.moves_history()[0].algebraic()!="b2b3"){r=false;}
    if(cp.moves_history()[8].algebraic()!="b2a1"){r=false;}

    ChessPosition cp2{cp};
    if(cp2.moves_history().size()!=9){r=false;}

    ChessPosition cp3(tbs);
    cp3=cp;
    if(cp3.moves_history().size()!=9){r=false;}

    cp3=cp3;
    if(cp2.history_str() != cp3.history_str()){r=false;}
  }

  
  //--------------- test MaterialIndex --------------------
  {
    cp.newgame();
    MaterialIndex mi=cp.materialindex();
    if(mi.str()!="KQRRBBNNPPPPPPPPkqrrbbnnpppppppp"){r=false;}
  }

  //--------------- test discovering pieces ---------------
  {
    string f="r1b1kbnr/pppp1ppp/8/q7/1n1P4/4B3/PPP1QPPP/RN2KBNR w KQkq - 0 4";
    cp.fen(f);
    cp.find_pinned_pieces();
    BitBoard bb=cp.discovering_pieces();
    BitBoard cc;
    cc.set_square(b4);
    cc.set_square(e3);
    if(bb!=cc){r=false;cout << "error discovering pieces" << endl;}
  }
  
  //--------------- test pinned pieces ---------------
  {
    string f="4k3/4p3/8/8/8/8/4R3/4K3 w - - 0 1";
    cp.fen(f);
    cp.find_pinned_pieces();
    BitBoard cc;
    BitBoard bb=cp.pinned_pieces();
    cc.set_square(e7);
    if(bb!=cc){r=false;cout << "error pinned pieces(1)" << endl;}

    f="4k3/4p3/4P3/8/8/8/4R3/4K3 w - - 0 1";
    cp.fen(f);
    cp.find_pinned_pieces();
    cc.clear();
    bb=cp.pinned_pieces();
    cc.set_square(e7);
    if(bb==cc){r=false;cout << "error pinned pieces(2)" << endl;}

    f="8/6pp/3q3k/4N3/1n2pP2/7P/6PK/8 b - f3 0 1";
    cp.fen(f);
    cp.make_move(e4,f3,en_passant);
    bb=cp.pinned_pieces();
    if(!bb){r=false;cout << "error pinned pieces(3)" << endl;}

    f="4k3/4q3/8/8/8/4N3/8/4K1N1 w - - 0 1";
    cp.fen(f);
    bb=cp.pinned_pieces();
    cp.make_move(g1,e2);
    bb=cp.pinned_pieces();
    if(bb){r=false;cout << "error pinned pieces(4)" << endl;}
  }


  //------------------- weakest attacker: -------------------------
  //
  {
    string s="k3r3/2b2n2/5p2/1q2n2R/3P4/3N4/4Q2B/K7 w - - 0 1";
    cp.fen(s);
    cp.make_move(d4,e5);
    Square bsa=cp.black_weakest_attacker(e5);
    if(bsa!=f6){r=false;}

    cp.make_move(f6,e5);
    cp.make_move(d3,e5);
    bsa=cp.black_weakest_attacker(e5);
    if(bsa!=f7){r=false;}

    cp.make_move(f7,e5);
    cp.make_move(h2,e5);
    bsa=cp.black_weakest_attacker(e5);
    if(bsa!=c7){r=false;}

    cp.make_move(c7,e5);
    cp.make_move(h5,e5);
    bsa=cp.black_weakest_attacker(e5);
    if(bsa!=e8){r=false;}

    cp.make_move(e8,e5);
    cp.make_move(e2,e5);
    bsa=cp.black_weakest_attacker(e5);
    if(bsa!=b5){r=false;}

    s="6k1/2r2p1Q/8/6N1/8/8/8/6K1 b - - 0 1";
    cp.fen(s);
    bsa=cp.black_weakest_attacker(h7);
    if(bsa!=g8){
      cout << "\nbsa=" << bsa.name() << endl;
      cout << "error1 weakest attacker" << endl;r=false;
    }

    s="6k1/8/8/8/6n1/8/2R2P1q/6K1 w - - 0 1";
    cp.fen(s);
    bsa=cp.black_weakest_attacker(h2);
    if(bsa!=g4){cout << "error2 weakest attacker" << endl;r=false;}
    bsa=cp.white_weakest_attacker(h2);
    if(bsa!=g1){cout << "error3 weakest attacker" << endl;r=false;}

    s="6k1/4q3/1Q6/8/1r6/1r6/4R3/4R1K1 w - - 0 1";
    cp.fen(s);
    bsa=cp.black_weakest_attacker(b6);
    if(bsa!=b4){cout << "error4 weakest attacker" << endl;r=false;}
    bsa=cp.white_weakest_attacker(e7);
    if(bsa!=e2){cout << "error5 weakest attacker" << endl;r=false;}

  }
  //
  //------------------- weakest attacker -------------------------

  
  //---------------------- test flip / flop : --------------------
  {
    cp.fen("r3k3/5ppp/8/8/3pP3/8/5PPP/4K2R b Kq e3 0 1");
    cp.flip();
    if(cp.fen()!="4k2r/5ppp/8/3Pp3/8/8/5PPP/R3K3 w Qk e6 0 1"){
      cout << "error flip" << endl
	   << "cp=" << cp.fen() << endl;
      r=false;
    }

    cp.fen("r3k3/5ppp/8/8/3pP3/8/5PPP/4K2R b Kq e3 0 1");
    cp.flop();
    if(cp.fen()!="3k3r/ppp5/8/8/3Pp3/8/PPP5/R2K4 b Qk d3 0 1"){cout << "error flop" << endl;r=false;}
  }

  
  //---------------------- test ep square: -----------------------
  {
    cp.fen("4k3/8/8/8/8/8/4P3/4K3 w - - 0 1");
    cp.make_move(e2,e4);
    if(cp.fen()!="4k3/8/8/8/4P3/8/8/4K3 b - - 0 1"){cout << "error(1) ep square" << endl;r=false;}

    cp.fen("4k3/8/8/8/3p4/8/4P3/4K3 w - - 0 1");
    cp.make_move(e2,e4);
    if(cp.fen()!="4k3/8/8/8/3pP3/8/8/4K3 b - e3 0 1"){cout << "error(2) ep square" << endl;r=false;}

    cp.fen("4k3/8/8/8/5p2/8/4P3/4K3 w - - 0 1");
    cp.make_move(e2,e4);
    if(cp.fen()!="4k3/8/8/8/4Pp2/8/8/4K3 b - e3 0 1"){cout << "error(3) ep square" << endl;r=false;}

    cp.fen("4k3/4p3/8/8/8/8/8/4K3 b - - 0 1");
    cp.make_move(e7,e5);
    if(cp.fen()!="4k3/8/8/4p3/8/8/8/4K3 w - - 0 2"){cout << "error(4) ep square" << endl;r=false;}
    
    cp.fen("4k3/4p3/8/3P4/8/8/8/4K3 b - - 0 1");
    cp.make_move(e7,e5);
    if(cp.fen()!="4k3/8/8/3Pp3/8/8/8/4K3 w - e6 0 2"){cout << "error(5) ep square" << endl;r=false;}

    cp.fen("4k3/4p3/8/5P2/8/8/8/4K3 b - - 0 1");
    cp.make_move(e7,e5);
    if(cp.fen()!="4k3/8/8/4pP2/8/8/8/4K3 w - e6 0 2"){cout << "error(6) ep square" << endl;r=false;}

  }


  //------------------- test nullmove: --------------------
  //
  {
    cp.newgame();
    short cr1=cp.castlingrights();
    cp.make_move(e2,e4);
    NullMove nm;
    cp.make_move(nm);
    short cr2=cp.castlingrights();
    if(cr1!=cr2){r=false;cout << "error nullmove" << endl;}
  }



  //--------------- test make illegal moves ---------------
  {
    cp.newgame();
    string s=cp.fen();

    cp.make_move(e2,e4);
    cp.make_move(g8,f6);
    cp.make_move(e4,e5);
    cp.make_move(f6,g8);
    cp.make_move(e5,d6);
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    if(cp.fen()!=s){r=false;cerr << "error in test make illegal EP move" << endl;}

    cp.make_move(b1,c3);
    cp.make_move(e7,e5);
    cp.make_move(g1,f3);
    cp.make_move(e5,e4);
    cp.make_move(f3,g1);
    cp.make_move(e4,d3);
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    if(cp.fen()!=s){r=false;cerr << "error in test make illegal EP move" << endl;}

  }

  
  //------------------- get hash: -------------------------
  //
  {
    cp.newgame();
    cp.make_move(e2,e4);
    cp.make_move(g8,f6);
    cp.make_move(e4,e5);
    cp.make_move(d7,d5);
    cp.make_move(e5,d6);
    cp.make_move(e7,d6);
    cp.make_move(g1,f3);
    cp.make_move(f8,e7);
    cp.make_move(f1,c4);
    cp.make_move(e8,h8);
    cp.make_move(a2,a4);
    NullMove nm;
    cp.make_move(nm);
    cp.make_move(a4,a5);
    cp.make_move(b7,b5);
    cp.make_move(a5,b6);
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();
    cp.unmake_move();

    uint64_t h1=cp.get_hash();
    string s="rnbq1rk1/ppp1bppp/3p1n2/8/2B5/5N2/PPPP1PPP/RNBQK2R w KQ - 4 5";
    cp.fen(s);
    uint64_t h2=cp.get_hash();
    if(h1!=h2){r=false;}
  }



  //----------------- convert BKmove to move: ----------------
  {
    {
      string s="k7/8/8/8/8/8/5N2/K7 w - - 0 1";
      cp.fen(s);
      Move m1(f2,e4,normal_move,WhiteKnight(),NoPiece());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(1)" << endl;}
    }

    {
      string s="r3k2r/8/8/8/8/8/8/R3K2R w - - 0 1";
      cp.fen(s);
      Move m1(e1,h1,castling,WhiteKing(),NoPiece());
      BKmove bkm1{m1};
      Move m2=bkm1(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(2)" << endl;}
      Move m3(e1,a1,castling,WhiteKing(),NoPiece());
      BKmove bkm2{m3};
      Move m4=bkm2(cp);
      if(m3!=m4){r=false;cout << "convert BKmove to move: error(3)" << endl;}
      Move m5(e8,h8,castling,BlackKing(),NoPiece());
      BKmove bkm3{m5};
      Move m6=bkm3(cp);
      if(m5!=m6){r=false;cout << "convert BKmove to move: error(4)" << endl;}
      Move m7(e8,a8,castling,BlackKing(),NoPiece());
      BKmove bkm4{m7};
      Move m8=bkm4(cp);
      if(m7!=m8){r=false;cout << "convert BKmove to move: error(5)" << endl;}
    }

    
    {
      string s="4k3/8/8/2pP4/8/8/8/4K3 w - c6 0 1";
      cp.fen(s);
      Move m1(d5,c6,en_passant,WhitePawn(),BlackPawn());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(6)" << endl;}
    }

    {
      string s="4k3/8/8/8/2pP4/8/8/4K3 b - d3 0 1";
      cp.fen(s);
      Move m1(c4,d3,en_passant,BlackPawn(),WhitePawn());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(7)" << endl;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_queen,BlackPawn(),WhiteRook());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(8)" << endl;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_rook,BlackPawn(),WhiteRook());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(9)" << endl;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_bishop,BlackPawn(),WhiteRook());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(10)" << endl;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_knight,BlackPawn(),WhiteRook());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(11)" << endl;}
    }

    {
      string s="rnbqk2r/pp2bpp1/4p2p/1B1n4/3N2PP/2N5/PPP2P2/R1BQK1R1 b Qkq - 1 10";
      cp.fen(s);
      Move m1(e8,f8,normal_move,BlackKing(),NoPiece());
      BKmove bkm{m1};
      Move m2=bkm(cp);
      if(m1!=m2){r=false;cout << "convert BKmove to move: error(12)" << endl;}
    }

  }
  //----------------------------------------------------------


  //----------------- convert string to move: ----------------
  {
    {
      string s="k7/8/8/8/8/8/5N2/K7 w - - 0 1";
      cp.fen(s);
      s="f2e4";
      Move m1(f2,e4,normal_move,WhiteKnight(),NoPiece());
      Move m2=cp.convert_to_move(s);
      if(m1!=m2){r=false;}
    }

    {
      string s="k7/5P2/8/8/8/8/8/K7 w - - 0 1";
      cp.fen(s);
      s="f7f8q";
      Move m1(f7,f8,promo_queen,WhitePawn(),NoPiece());
      m1.check(true);
      Move m2=cp.convert_to_move(s);
      if(m1!=m2){
	cerr << "error1" << endl;
	cerr << "m1: " << m1.dump() << endl;
	cerr << "m2: " << m2.dump() << endl;
	r=false;
      }
    }

    {
      string s="k7/5P2/8/8/8/8/8/K7 w - - 0 1";
      cp.fen(s);
      s="f7f8r";
      Move m1(f7,f8,promo_rook,WhitePawn(),NoPiece());
      m1.check(true);
      Move m2=cp.convert_to_move(s);
      if(m1!=m2){r=false;}
    }

    {
      string s="k7/5P2/8/8/8/8/8/K7 w - - 0 1";
      cp.fen(s);
      s="f7f8n";
      Move m1(f7,f8,promo_knight,WhitePawn(),NoPiece());
      Move m2=cp.convert_to_move(s);
      if(m1!=m2){r=false;}
    }

    {
      string s="8/5P2/4k3/8/8/8/8/K7 w - - 0 1";
      cp.fen(s);
      s="f7f8n";
      Move m1(f7,f8,promo_knight,WhitePawn(),NoPiece());
      m1.check(true);
      Move m2=cp.convert_to_move(s);
      if(m1!=m2){r=false;}
    }
  }


  //----------------- convert TTmove to move: ----------------
  {
    {
      string s="k7/8/8/8/8/8/5N2/K7 w - - 0 1";
      cp.fen(s);
      Move m1(f2,e4,normal_move,WhiteKnight(),NoPiece());
      TTmove ttm{m1};
      Move m2=ttm(cp);
      if(m1!=m2){r=false;}
    }

    {
      string s="r3k2r/8/8/8/8/8/8/R3K2R w - - 0 1";
      cp.fen(s);
      Move m1(e1,h1,castling,WhiteKing(),NoPiece());
      TTmove ttm1{m1};
      Move m2=ttm1(cp);
      if(m1!=m2){r=false;}
      Move m3(e1,a1,castling,WhiteKing(),NoPiece());
      TTmove ttm2{m3};
      Move m4=ttm2(cp);
      if(m3!=m4){r=false;}
      Move m5(e8,h8,castling,BlackKing(),NoPiece());
      TTmove ttm3{m5};
      Move m6=ttm3(cp);
      if(m5!=m6){r=false;}
      Move m7(e8,a8,castling,BlackKing(),NoPiece());
      TTmove ttm4{m7};
      Move m8=ttm4(cp);
      if(m7!=m8){r=false;}
    }

    {
      string s="4k3/8/8/2pP4/8/8/8/4K3 w - c6 0 1";
      cp.fen(s);
      Move m1(d5,c6,en_passant,WhitePawn(),BlackPawn());
      TTmove ttm{m1};
      Move m2=ttm(cp);
      if(m1!=m2){r=false;}
    }

    {
      string s="4k3/8/8/8/2pP4/8/8/4K3 b - d3 0 1";
      cp.fen(s);
      Move m1(c4,d3,en_passant,BlackPawn(),WhitePawn());
      TTmove ttm{m1};
      Move m2=ttm(cp);
      if(m1!=m2){r=false;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_queen,BlackPawn(),WhiteRook());
      TTmove ttm{m1};
      Move m2=ttm(cp);
      if(m1!=m2){r=false;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_rook,BlackPawn(),WhiteRook());
      TTmove ttm{m1};
      Move m2=ttm(cp);
      if(m1!=m2){r=false;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_bishop,BlackPawn(),WhiteRook());
      TTmove ttm{m1};
      Move m2=ttm(cp);
      if(m1!=m2){r=false;}
    }

    {
      string s="4k3/8/8/8/8/8/6p1/4K2R b - d3 0 1";
      cp.fen(s);
      Move m1(g2,h1,promo_knight,BlackPawn(),WhiteRook());
      TTmove ttm{m1};
      Move m2=ttm(cp);
      if(m1!=m2){r=false;}
    }

  }
  //----------------------------------------------------------


  //----------------- force move: ----------------------------
  {
    string s1="k7/8/8/3pP3/8/8/8/K7 w - d6 0 1";
    cp.fen(s1);
    Move m1(e5,d6,en_passant,WhitePawn(),BlackPawn());
    cp.force_move(m1);
    cp.undo_forced_move(m1);
    string s2=cp.fen();
    if(s1!=s2){r=false;}
  }

  
  {
    string s1="k2r4/4P3/8/8/8/8/8/K7 w - - 0 1";
    cp.fen(s1);
    Move m1(e7,d8,promo_queen,WhitePawn(),BlackRook());
    cp.force_move(m1);
    cp.undo_forced_move(m1);
    string s2=cp.fen();
    if(s1!=s2){r=false;}
  }


  
  //----------------- queencheck squares: --------------------
  {
    string s1="8/8/4k3/8/8/8/8/K7 w - - 0 1";
    cp.fen(s1);
    cp.find_pinned_pieces();
    BitBoard qcs{c8,d7,e8,e7,g8,f7,a6,b6,c6,d6,f6,g6,h6,d5,c4,b3,a2,e5,e4,e3,e2,e1,f5,g4,h3};
    BitBoard bb=cp.white_queencheck_squares();
    
    if(bb!=qcs){r=false;}
  }

  {
    string s1="8/8/4K3/8/8/8/8/k7 w - - 0 1";
    cp.fen(s1);
    cp.find_pinned_pieces();
    BitBoard qcs{c8,d7,e8,e7,g8,f7,a6,b6,c6,d6,f6,g6,h6,d5,c4,b3,a2,e5,e4,e3,e2,e1,f5,g4,h3};
    BitBoard bb=cp.black_queencheck_squares();
    
    if(bb!=qcs){r=false;}
  }


  
 exit_test:
  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






