/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<cmath>
#include"../common/attacks.h"
#include"../board/cposition.h"
#include"../movegen/movegenerator.h"
#include"../eval/evaluation.h"
#include"../search/analyze.h"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest evaluation:
//

void test_evaluation()
{
  cout << "Test Evaluation ......................................." << flush;
  bool r=true;

  Attacks atta;
  Tables tbs;
  ChessPosition cp(tbs);
  Analysis ana(cp,tbs);
  Egt4Pieces x(atta,white_knight_id,white_bishop_id),y(atta,white_queen_id,black_rook_id);
  Evaluation eva(cp,tbs);
  
  array<uint64_t,64> movet;
  array<uint64_t,64> wa;
  array<uint64_t,64> ba;
  wa.fill(0);
  ba.fill(0);
  
  
  
  //--------------------- safe queen checks: -----------------------------------------------------
  //

  {
    cp.fen("8/8/4k3/8/8/8/8/3QK3 w - - 0 1");
    ana.init(cp);
    ana.static_eval();
    if(ana.evaluation().variable_vector().at(w_safe_queencheck)!=7){
      r=false;
      std::cerr << "error(1) safe queen check " << ana.evaluation().variable_vector().at(w_safe_queencheck) << endl;
    }
  }

  {
    cp.fen("8/8/4k3/8/8/5p2/8/3QK3 w - - 0 1");
    ana.init(cp);
    ana.static_eval();
    if(ana.evaluation().variable_vector().at(w_safe_queencheck)!=5){
      r=false;
      std::cerr << "error(2) safe queen check " << ana.evaluation().variable_vector().at(w_safe_queencheck) << endl;
    }
  }

  {
    cp.fen("8/8/4K3/8/8/8/8/3qk3 b - - 0 1");
    ana.init(cp);
    ana.static_eval();
    if(ana.evaluation().variable_vector().at(w_safe_queencheck)!=-7){
      r=false;
      std::cerr << "error(3) safe queen check " << ana.evaluation().variable_vector().at(w_safe_queencheck) << endl;
    }
  }

  {
    cp.fen("3qk3/8/5P2/8/8/4K3/8/8 b - - 0 1");
    ana.init(cp);
    ana.static_eval();
    if(ana.evaluation().variable_vector().at(w_safe_queencheck)!=-5){
      r=false;
      std::cerr << "error(4) safe queen check " << ana.evaluation().variable_vector().at(w_safe_queencheck) << endl;
    }
  }
  //----------------------------------------------------------------------------------------------

  
  //--------------------- eval symmetry: -------------------------------
  //
  {
    cp.fen("8/8/8/6kP/R7/5p1K/8/6r1 w - -");
    cp.find_pinned_pieces();
    MoveGenerator mg(tbs);
    mg.create_movetable(cp,movet);
    eva.init();
    string s1=cp.fen();
    int ev1=eva.static_eval();
    string report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    string s2=cp.fen();
    int ev2=eva.static_eval();
    string report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(1)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

    cp.fen("5k2/p5P1/3P3P/Pb2K3/4B3/8/8/8 b - -");
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s1=cp.fen();
    ev1=eva.static_eval();
    report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s2=cp.fen();
    ev2=eva.static_eval();
    report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(2)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

    cp.fen("1k2rb1r/p1p1qppp/2p5/3pp3/2P5/1P3P2/PBP1QP1P/4R1RK b - -");
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s1=cp.fen();
    ev1=eva.static_eval();
    report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s2=cp.fen();
    ev2=eva.static_eval();
    report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(3)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

    cp.fen("7k/5p2/8/8/6p1/8/6K1/1r6 b - -");
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s1=cp.fen();
    ev1=eva.static_eval();
    report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s2=cp.fen();
    ev2=eva.static_eval();
    report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(4)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

    cp.fen("r4k1r/1pp2ppp/p1n3b1/3q4/7N/3P4/PP2QPPP/R3R1K1 w - -");
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s1=cp.fen();
    ev1=eva.static_eval();
    report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s2=cp.fen();
    ev2=eva.static_eval();
    report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(5)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

    cp.fen("8/8/3k4/8/8/8/pK6/7b b - -");
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s1=cp.fen();
    ev1=eva.static_eval();
    report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s2=cp.fen();
    ev2=eva.static_eval();
    report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(6)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

    cp.fen("8/8/3N4/2Pk3p/7P/4B1KP/r7/8 w - -");
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s1=cp.fen();
    ev1=eva.static_eval();
    report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s2=cp.fen();
    ev2=eva.static_eval();
    report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(7)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

    cp.fen("8/R7/5pk1/1P1K4/3N3P/8/5n2/8 b - -");
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s1=cp.fen();
    ev1=eva.static_eval();
    report1=eva.report();
    cp.flip();
    cp.find_pinned_pieces();
    mg.create_movetable(cp,movet);
    eva.init();
    s2=cp.fen();
    ev2=eva.static_eval();
    report2=eva.report();
    if(ev1!=ev2){
      cout << "eval symmetry error(8)" << endl;
      cout << s1 << "  eval=" << ev1 << endl;
      cout << report1 << endl;
      cout << s2 << "  eval=" << ev2 << endl;
      cout << report2 << endl;
      r=false;
    }

  }

  //--------------------- Bad bishops: ---------------------------------
  //
  {
    string s="7k/3b4/2p1p3/3pP3/3P4/3B4/2P5/7K w - - 0 1";
    cp.fen(s);
    eva.init();
    eva.static_eval();
    auto v=eva.variable_vector();
    if(v[w_bad_bishop]!=2){cout << "Error in bad bishops" << endl;r=false;}
  }
  
  //--------------------- Isolated pawns: ------------------------------
  //
  {
    string s="7k/8/8/8/P3P2P/8/8/7K w - - 0 1";
    cp.fen(s);
    eva.init();
    int e1=eva.static_eval();
    auto v=eva.variable_vector();
    if(v[w_isolated_pawns]!=-3){
      cout << "error(1) in isolated pawns  n=" << v[w_isolated_pawns] << endl;
      r=false;
    }
    cp.flip();
    eva.init();
    int e2=eva.static_eval();
    v=eva.variable_vector();
    if(v[w_isolated_pawns]!=3){
      cout << "error(2) in isolated pawns  n=" << v[w_isolated_pawns] << endl;
      r=false;
    }
    if(e1!=e2){cout << "symmetry error in isolated pawns   e1=" << e1 << "  e2=" << e2 << endl;r=false;}
  }

  


  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}
}






