/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest ChessPosition:
//
void test_fen()
{
  bool r=true;

  cout << "Test fen .............................................." << flush;

  Tables tbs;
  ChessPosition cp(tbs);
  cp.newgame();
  string s="rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";
  if(cp.fen()!=s){r=false;}

  if(!r){cerr << "error 1" << endl;}

  cp.make_move(e2,e4);
  s="rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq - 0 1";
  if(cp.fen()!=s){r=false;}
  if(!r){cerr << "error 2" << endl;}

  cp.make_move(c7,c5);
  s="rnbqkbnr/pp1ppppp/8/2p5/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2";
  if(cp.fen()!=s){r=false;}
  if(!r){cerr << "error 3" << endl;}

  cp.make_move(g1,f3);
  s="rnbqkbnr/pp1ppppp/8/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R b KQkq - 1 2";
  if(cp.fen()!=s){r=false;}
  if(!r){cerr << "error 4" << endl;}

  {
    string s1{s},s2;
    cp.fen(s1);
    s2=cp.fen();
    if(s1!=s2){r=false;}
  }
  if(!r){cerr << "error 5" << endl;}

  {
    cp.newgame();
    cp.make_move(h2,h4);
    cp.make_move(h7,h5);
    cp.make_move(h1,h2);
    cp.make_move(h8,h7);
    string s{"rnbqkbn1/pppppppr/8/7p/7P/8/PPPPPPPR/RNBQKBN1 w Qq - 2 3"};
    if(s!=cp.fen()){r=false;}
  }
  if(!r){cerr << "error 6  fen=" << cp.fen() << endl;}

  {
    cp.newgame();
    cp.make_move(a2,a4);
    cp.make_move(a7,a5);
    cp.make_move(a1,a2);
    cp.make_move(a8,a7);
    string s{"1nbqkbnr/rppppppp/8/p7/P7/8/RPPPPPPP/1NBQKBNR w Kk - 2 3"};
    if(s!=cp.fen()){r=false;}
  }
  if(!r){cerr << "error 7" << endl;}

  {
    cp.newgame();
    cp.make_move(e2,e4);
    cp.make_move(e7,e5);
    cp.make_move(e1,e2);
    cp.make_move(e8,e7);
    string s{"rnbq1bnr/ppppkppp/8/4p3/4P3/8/PPPPKPPP/RNBQ1BNR w - - 2 3"};
    if(s!=cp.fen()){r=false;}
  }
  if(!r){cerr << "error 8" << endl;}


  {
    string f="k7/8/8/3pP3/8/8/8/7K w - d6 0 10";
    cp.fen(f);
    if(cp.ep()!=d6){r=false;}
  }
  if(!r){cerr << "error 9" << endl;}

  {
    string f="bbqnnrkr/pppppppp/8/8/8/8/PPPPPPPP/BBQNNRKR w FHfh - 0 5";
    cp.fen(f);
    string s=cp.fen();
    if(s!=f){r=false;}
  }
  if(!r){cerr << "error 10   cp.fen()=" << cp.fen() << endl;}

  {
    string f="bbqnnrkr/pppppppp/8/8/8/8/PPPPPPPP/BBQNNRKR w FHfh - 0 5";
    cp.fen(f);
    string s=cp.initial_position();
    if(s!=f){r=false;}
  }
  if(!r){cerr << "error 11   cp.fen()=" << cp.fen() << endl;}
  

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}

