/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../movegen/movegenerator.h"
#include"../board/hasher.cpp"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest Hasher
//


void test_hasher()
{
  cout << "Test Hasher ..........................................." << flush;
  bool r=true;

  Tables tbs;
  ChessPosition cp(tbs);
  Hasher<1> tt(tbs);
  Hasher<2> hh2(tbs);


  {
    uint64_t h1,h2,ph1,ph2;
    Move m;

    cp.newgame();
    tt.init(cp);
    hh2.init(cp);
    if(tt.hash()==hh2.hash()){
      cout << "error in Hasher<2>" << endl;
      r=false;
    }
    m=cp.convert_to_move(e2,e4);
    cp.make_move(m);
    tt.make_move(m);
    h1=tt.hash();
    ph1=tt.pawnhash();
    tt.init(cp);
    h2=tt.hash();
    ph2=tt.pawnhash();
    if(h1!=h2){cout << "error 1" << endl;r=false;}
    if(ph1!=ph2){cout << "error 1.1" << endl;r=false;}
  }


  {
    uint64_t h1,h2,ph1,ph2;
    Move m;

    cp.newgame();
    tt.init(cp); 
    m=cp.convert_to_move(e2,e4);
    cp.make_move(m);
    tt.make_move(m);
    m=cp.convert_to_move(a7,a6);
    cp.make_move(m);
    tt.make_move(m);
    m=cp.convert_to_move(d2,d4);
    cp.make_move(m);
    tt.make_move(m);
    h1 = tt.hash();
    ph1 = tt.pawnhash();

    cp.newgame();
    tt.init(cp);
    m=cp.convert_to_move(d2,d4);
    cp.make_move(m);
    tt.make_move(m);
    m=cp.convert_to_move(a7,a6);
    cp.make_move(m);
    tt.make_move(m);
    m=cp.convert_to_move(e2,e4);
    cp.make_move(m);
    tt.make_move(m);
    h2 = tt.hash();
    ph2 = tt.pawnhash();

    if(h1!=h2){cout << "error 2" << endl;r=false;}
    if(ph1!=ph2){cout << "error 2.1" << endl;r=false;}
  }

  {
    Move m;
    cp.newgame();
    tt.init(cp); 

    m=cp.convert_to_move(e2,e4);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(e7,e6);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(e4,e5);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d7,d5);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(e5,d6);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(g8,f6);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d6,c7);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(f8,d6);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(c7,b8);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(e8,g8);
    cp.make_move(m);
    tt.make_move(m);

    uint64_t h1,h2,ph1,ph2;
    h1 = tt.hash();
    ph1 = tt.pawnhash();
    tt.init(cp);
    h2 = tt.hash();
    ph2 = tt.pawnhash();
    if(h1!=h2){cout << "error 3" << endl;r=false;}
    if(ph1!=ph2){cout << "error 3.1" << endl;r=false;}
  }

  {
    Move m;
    cp.newgame();
    tt.init(cp); 

    m=cp.convert_to_move(a2,a3);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d7,d5);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(a3,a4);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d5,d4);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(e2,e4);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d4,e3);
    cp.make_move(m);
    tt.make_move(m);

    uint64_t h1,h2,ph1,ph2;
    h1 = tt.hash();
    ph1 = tt.pawnhash();
    tt.init(cp);
    h2 = tt.hash();
    ph2 = tt.pawnhash();
    if(h1!=h2){cout << "error 4" << endl;r=false;}
    if(ph1!=ph2){cout << "error 4.1" << endl;r=false;}
  }


  {
    Move m;
    cp.newgame();
    tt.init(cp); 

    m=cp.convert_to_move(d2,d3);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d7,d6);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(c1,e3);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(c8,e6);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d1,d2);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(d8,d7);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(b1,c3);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(b8,c6);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(e1,c1);
    cp.make_move(m);
    tt.make_move(m);

    m=cp.convert_to_move(e8,c8);
    cp.make_move(m);
    tt.make_move(m);

    uint64_t h1,h2,ph1,ph2;
    h1 = tt.hash();
    ph1 = tt.pawnhash();
    tt.init(cp);
    h2 = tt.hash();
    ph2 = tt.pawnhash();
    if(h1!=h2){cout << "error 5" << endl;r=false;}
    if(ph1!=ph2){cout << "error 5.1" << endl;r=false;}
  }



  {
    // initial position:
    string s="rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x463b96181691fc9c){cout << "error 6" << endl;r=false;}

    // 1. e4
    s="rnbqkbnr/pppppppp/8/8/4P3/8/PPPP1PPP/RNBQKBNR b KQkq - 0 1";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x823c9b50fd114196){cout << "error 7" << endl;r=false;}

    // 1. e4 d5
    s="rnbqkbnr/ppp1pppp/8/3p4/4P3/8/PPPP1PPP/RNBQKBNR w KQkq - 0 2";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x756b94461c50fb0){cout << "error 8" << endl;r=false;}

    // 1. e4 d5 2. e5
    s="rnbqkbnr/ppp1pppp/8/3pP3/8/8/PPPP1PPP/RNBQKBNR b KQkq - 0 2";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x662fafb965db29d4){cout << "error 9" << endl;r=false;}

    // 1. e4 d5 2. e5 f5
    s="rnbqkbnr/ppp1p1pp/8/3pPp2/8/8/PPPP1PPP/RNBQKBNR w KQkq f6 0 3";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x22a48b5a8e47ff78){cout << "error 10" << endl;r=false;}

    // 1. e2e4 d7d5 2. e4e5 f7f5 3. e1e2
    s="rnbqkbnr/ppp1p1pp/8/3pPp2/8/8/PPPPKPPP/RNBQ1BNR b kq - 0 3";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x652a607ca3f242c1){cout << "error 11" << endl;r=false;}

    // 1. e2e4 d7d5 2. e4e5 f7f5 3. e1e2 e8f7
    s="rnbq1bnr/ppp1pkpp/8/3pPp2/8/8/PPPPKPPP/RNBQ1BNR w - - 0 4";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0xfdd303c946bdd9){cout << "error 12" << endl;r=false;}

    // 1. a2a4 b7b5 h2h4 b5b4 c2c4
    s="rnbqkbnr/p1pppppp/8/8/PpP4P/8/1P1PPPP1/RNBQKBNR b KQkq c3 0 3";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x3c8123ea7b067637){cout << "error 13" << endl;r=false;}

    // 1. a2a4 b7b5 2. h2h4 b5b4 3. c2c4 b4c3 4. a1a3
    s="rnbqkbnr/p1pppppp/8/8/P6P/R1p5/1P1PPPP1/1NBQKBNR b Kkq - 0 4";
    cp.fen(s);tt.init(cp);
    if(tt.hash()!=0x5c3f9b829b279560){cout << "error 14" << endl;r=false;}

  }


  
  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






