/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../common/numerals.h"
#include"../eval/materialindex.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest materialindex:
//

void test_materialindex()
{
  bool r=true;

  {
    MaterialIndex mi;
    mi.set_whitepawn(2);
    mi.set_whiterook(1);
    if(mi.str()!="KRPPk")r=false;
  }

  {
    MaterialIndex mi;
    int64_t ind{17600775979008};
    mi.set_index(ind);
    if(mi.str()!="KRPPk")r=false;
  }

  {
    MaterialIndex mi;
    mi.set_whitequeen(1);
    mi.set_whiterook(1);
    mi.set_whitebishop(5);
    mi.set_whiteknight(1);
    mi.set_whitepawn(8);
    mi.set_blackqueen(11);
    mi.set_blackrook(2);
    mi.set_blackbishop(2);
    mi.set_blackknight(0);
    mi.set_blackpawn(8);

    if(mi.white_wins()){r=false;}
    if(!mi.black_wins()){r=false;}
    if(mi.equal()){r=false;}
    if(mi.str()!="KQRBBBBBNPPPPPPPPkqqqqqqqqqqqrrbbpppppppp"){r=false;cerr << "error(1)" << endl;}
  }
  
  {
    MaterialIndex mi;
    mi.set_whitequeen(1);
    mi.set_blackrook(1);
    if(mi.str()!="KQkr"){r=false;cout << "error(2)" << endl;}
    mi.set_whitepawn(8);
    mi.set_blackpawn(4);
    if(!mi.white_wins()){r=false;cout << "error(2a)" << endl;}
    if(mi.black_wins()){r=false;cout << "error(2b)" << endl;}
    if(mi.equal()){r=false;cout << "error(2c)" << endl;}
    mi.flip_colors();
    if(mi.str()!="KRPPPPkqpppppppp"){r=false;cout << "error(2d)" << endl;}
  }

  {
    MaterialIndex mi;
    mi.set_whitebishop(2);
    mi.set_blackknight(2);
    if(mi.str()!="KBBknn"){r=false;cout << "error(3)" << endl;}
    if(!mi.white_wins()){r=false;cout << "error(3a)" << endl;}
    if(mi.black_wins()){r=false;cout << "error(3b)" << endl;}
    if(mi.equal()){r=false;cout << "error(3c)" << endl;}
    mi.flip_colors();
    if(mi.str()!="KNNkbb"){r=false;cout << "error(4)" << endl;}
    mi.set_whitebishop(2);
    mi.set_blackknight(2);
    if(mi.str()!="KBBNNkbbnn"){r=false;cout << "error(5)" << endl;}
    if(!mi.equal()){r=false;cout << "error(5a)" << endl;}
    if(mi.str()!="KBBNNkbbnn"){r=false;cout << "error(6)" << endl;}
  }


  {
    MaterialIndex mi;
    mi.set_whitequeen(1);
    mi.set_whiterook(1);
    mi.set_whitebishop(1);
    mi.set_whiteknight(1);
    mi.set_whitepawn(1);
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(7)" << endl;}
  }


  {
    MaterialIndex mi;
    mi.set_whitequeen(1);
    mi.set_whitepawn(1);
    mi.set_blackqueen(1);
    mi.set_blackknight(1);
    mi.set_blackpawn(1);
    mi.add_piece(white_queen_id);
    mi.add_piece(white_pawn_id);
    mi.add_piece(black_queen_id);
    mi.add_piece(black_knight_id);
    mi.add_piece(black_pawn_id);
    if(mi.get_whitequeen()!=2){r=false;cout << "error(8)" << endl;}
    if(mi.get_whitepawn()!=2){r=false;cout << "error(9)" << endl;}
    if(mi.get_blackqueen()!=2){r=false;cout << "error(10)" << endl;}
    if(mi.get_blackknight()!=2){r=false;cout << "error(11)" << endl;}
    if(mi.get_blackpawn()!=2){r=false;cout << "error(12)" << endl;}
    mi.remove_piece(white_queen_id);
    mi.remove_piece(white_pawn_id);
    mi.remove_piece(black_queen_id);
    mi.remove_piece(black_knight_id);
    mi.remove_piece(black_pawn_id);
    if(mi.get_whitequeen()!=1){r=false;cout << "error(13)" << endl;}
    if(mi.get_whitepawn()!=1){r=false;cout << "error(14)" << endl;}
    if(mi.get_blackqueen()!=1){r=false;cout << "error(15)" << endl;}
    if(mi.get_blackknight()!=1){r=false;cout << "error(16)" << endl;}
    if(mi.get_blackpawn()!=1){r=false;cout << "error(17)" << endl;}

    MaterialIndex mi2{mi};
    mi2.add_piece(no_piece_id);
    if(mi2!=mi){r=false;}
  }

  
  //------------------------ pawnless positions: -----------------------------
  {
    MaterialIndex mi;
    mi.set_whiterook(1);
    mi.set_whitebishop(1);
    mi.set_blackrook(1);
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(18)" << endl;}
  }

  {
    MaterialIndex mi;
    mi.set_whiterook(1);
    mi.set_whiteknight(1);
    mi.set_blackrook(1);
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(20)" << endl;}
  }

  {
    MaterialIndex mi;
    mi.set_whiterook(1);
    mi.set_blackknight(1);
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(22)" << endl;}
  }

  {
    MaterialIndex mi;
    mi.set_whiterook(1);
    mi.set_blackbishop(1);
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(24)" << endl;}
  }

  {
    MaterialIndex mi;
    mi.set_whiterook(1);
    mi.set_blackrook(1);
    mi.set_whitebishop(2);
    if(mi.value()<600){r=false;cout << "error(26)" << endl;}
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(27)" << endl;}
  }
  
  {
    MaterialIndex mi;
    mi.set_whiterook(1);
    mi.set_blackrook(1);
    mi.set_whiteknight(2);
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(29)" << endl;}
  }
  
  {
    MaterialIndex mi;
    mi.set_whitequeen(1);
    mi.set_blackrook(1);
    if(mi.value()<400){r=false;cout << "error(30)" << endl;}
    int value=mi.value();
    mi.flip_colors();
    if(mi.value()!=-value){r=false;cout << "error(31)" << endl;}
  }
  //--------------------------------------------------------------------------

  
  cout << "Test MaterialIndex ...................................." << flush;
  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






