/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<stdexcept>
#include"../board/cposition.h"
#include"../eval/mobility.h"
#include"../movegen/movegenerator.h"
#include"../common/tables.h"
#include"../common/prebb.h"
#include"../search/analyze.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest Mobility:
//

void test_mobility()
{
  cout << "Test Mobility ........................................." << flush;
  bool r=true;

  Tables tbs;
  ChessPosition cp(tbs);
  Analysis ana(cp,tbs);

  
  array<uint64_t,64> movet;
  

  //------------------ safe mobility: ------------------------------
  {
    string s="k7/8/8/r3p3/3P4/2N5/8/7K w - - 0 1";
    cp.fen(s);
    MoveGenerator mg(tbs);
    mg.create_movetable(cp,movet);
    Mobility mb(tbs.attacks());
    mb.compute(cp);
    if(mb.safe_mobility()!=40){r=false;cout << "error(1) safe mobility: " << mb.safe_mobility() << endl;}
  }


  {
    //--------------------- test king neighbourhood: ------------------------------
    string s="4k3/8/8/8/8/8/8/4K3 w - - 0 1";
    cp.fen(s);
    Mobility mb(tbs.attacks());
    mb.compute(cp);
    if(mb.white_king_neighbourhood()!=BitBoard(0x3838ull)){r=false;}
    if(mb.black_king_neighbourhood()!=BitBoard(0x3838000000000000ull)){r=false;}

    s="k7/8/8/8/8/8/8/7K w - - 0 1";
    cp.fen(s);
    mb.compute(cp);
    if(mb.white_king_neighbourhood()!=BitBoard(0xc0c0ull)){r=false;}
    if(mb.black_king_neighbourhood()!=BitBoard(0x0303000000000000ull)){r=false;}
  }

  {
    //--------------------- test king attacks: ------------------------------
    string s="Q3kr2/3P4/8/2b3N1/2B3n1/8/3p4/q3KR2 w - - 0 1";
    cp.fen(s);
    Mobility mb(tbs.attacks());
    mb.compute(cp);
    if(mb.king_safety()!=0){r=false;cerr << "error(2) kingsafety=" << mb.king_safety()<<endl;}

    s="r6r/pQp1kpqp/3p1nn1/2bPp3/4P3/2N3P1/PPP2P2/R1B1K1NR w KQ - 1 16";
    cp.fen(s);
    mb.compute(cp);
    if(mb.king_safety()!=0){r=false;cerr << "error(3) kingsafety=" << mb.king_safety()<<endl;}

    s="4k3/7N/8/2Q5/7B/8/8/3RK3 w - - 0 1";
    cp.fen(s);
    mb.compute(cp);
    if(mb.king_safety()!=67500){r=false;cerr << "error(4) kingsafety=" << mb.king_safety()<<endl;}

    s="4K3/7n/8/2q5/7b/8/8/3rk3 b - - 0 1";
    cp.fen(s);
    mb.compute(cp);
    if(mb.king_safety()!=-67500){r=false;cerr << "error(5) kingsafety=" << mb.king_safety()<<endl;}

    s="4k3/8/8/8/1Q6/B7/8/4K3 w - - 0 1";
    cp.fen(s);
    mb.compute(cp);
    if(mb.king_safety()!=25600){r=false;cerr << "error(6) kingsafety=" << mb.king_safety()<<endl;}

    s="4K3/8/8/8/1q6/b7/8/4k3 b - - 0 1";
    cp.fen(s);
    mb.compute(cp);
    if(mb.king_safety()!=-25600){r=false;cerr << "error(7) kingsafety=" << mb.king_safety()<<endl;}
  }


  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






