/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<stdexcept>
#include<utility>
#include"../board/cposition.h"
#include"../movegen/move.h"
#include"../hashtable/ttmove.h"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest Move: 
//

void test_move()
{
  cout << "Test Move ............................................." << flush;
  bool r=true;

  
  do
    {
      {
	if(sizeof(Move)!=4){
	  r=false;
	  cerr << endl << "size_of(Move)=" << sizeof(Move) << endl;
	}
      }
      
      if(!r){cerr << "test_move error reason 0" << endl;}
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	cp.newgame();
	Move m1{e2,e4,normal_move,WhitePawn(),NoPiece()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(ttm.capture(cp)){r=false;}
      }
      
      if(!r){cerr << "test_move error reason 1" << endl;}
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	cp.newgame();
	Move m1{no_square,no_square,nullmove,NoPiece(),NoPiece()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 2" << endl;}
	if(ttm.capture(cp)){r=false;}
	if(!r){cerr << "test_move error reason 3" << endl;}
      }
      
      
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	string s1{"r3k2r/8/8/8/8/8/8/R3K2R w KQkq - 0 1"};
	cp.fen(s1);
	Move m1{e1,g1,castling,WhiteKing(),NoPiece()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 4" << endl;}
	if(ttm.capture(cp)){r=false;}
	if(!r){cerr << "test_move error reason 5" << endl;}
      }
      
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	string s1{"r3k2r/8/8/8/8/8/8/R3K2R w KQkq - 0 1"};
	cp.fen(s1);
	Move m1{e1,c1,castling,WhiteKing(),NoPiece()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 6" << endl;}
	if(ttm.capture(cp)){r=false;}
	if(!r){cerr << "test_move error reason 7" << endl;}
      }
      
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	string s1{"r3k2r/8/8/8/8/8/8/R3K2R b KQkq - 0 1"};
	cp.fen(s1);
	Move m1{e8,g8,castling,BlackKing(),NoPiece()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 8" << endl;}
	if(ttm.capture(cp)){r=false;}
	if(!r){cerr << "test_move error reason 9" << endl;}
      }
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	string s1{"r3k2r/8/8/8/8/8/8/R3K2R b KQkq - 0 1"};
	cp.fen(s1);
	Move m1{e8,c8,castling,BlackKing(),NoPiece()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 10" << endl;}
	if(ttm.capture(cp)){r=false;}
	if(!r){cerr << "test_move error reason 11" << endl;}
      }
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	string s1{"rnbqkbnr/ppp1pppp/8/3pP3/8/8/PPPP1PPP/RNBQKBNR w KQkq d6 0 1"};
	cp.fen(s1);
	Move m1{e5,d6,en_passant,WhitePawn(),BlackPawn()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 12" << endl;}
	if(ttm.capture(cp)!=BlackPawn()){r=false;}
	if(!r){cerr << "test_move error reason 13" << endl;}
      }
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	string s1{"r3k2r/1P6/8/8/8/8/8/R3K2R w KQkq - 0 1"};
	cp.fen(s1);
	Move m1{b7,a8,promo_rook,WhitePawn(),BlackRook()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 14" << endl;}
	if(ttm.capture(cp)!=BlackRook()){r=false;}
	if(!r){cerr << "test_move error reason 15" << endl;}
      }
      
      {
	Tables tbs;
	ChessPosition cp(tbs);
	string s1{"r3k2r/1P6/8/8/8/8/8/R3K2R w KQkq - 0 1"};
	cp.fen(s1);
	Move m1{b7,b8,promo_rook,WhitePawn(),NoPiece()};
	TTmove ttm{m1};
	Move m2=ttm(cp);
	if(m1!=m2){r=false;}
	if(!r){cerr << "test_move error reason 16" << endl;}
	if(ttm.capture(cp)){r=false;}
	if(!r){cerr << "test_move error reason 17" << endl;}
      }
      
      {
	NullMove m;
	if(m.from().name()!="no_square")r=false;
	if(m.to().name()!="no_square")r=false;
      }
    }
  while(false);
  
  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}
  
}






