/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../movegen/movegenerator.h"
#include"../eval/evaluation.h"
#include"../search/analyze.h"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest movegenerator:
//

void test_movegenerator()
{
  cout << "Test MoveGenerator ...................................." << flush;
  bool r=true;

  Tables tbs;
  ChessPosition cp(tbs);  
  MoveGenerator mg(tbs);
  array<uint64_t,64> bb_moves;

  //--------------- test pawns ------------------------------
  {
    string f="k4r1r/6P1/8/8/8/8/8/K7 w - - 0 10";
    string s1="g7f8,g7f8,g7f8,g7f8,g7h8,g7h8,g7h8,g7h8,g7g8,g7g8,g7g8,g7g8,a1b1,a1b2,a1a2";
    cp.fen(f);
    MoveList ml;
    mg.generate_moves(cp,ml);
    if(ml.size()!=15){
      r=false;
      cout << "reason2   size=" << ml.size() << endl;
      for(uint i=0;i<ml.size();i++){cout << ml[i].algebraic() << " ";}cout << endl;
    }

    f="k7/8/8/3pP3/8/8/8/7K w - d6 0 10";
    s1="e5d6,e5e6,h1h2,h1g2,h1g1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    if(ml.size()!=5){
      r=false;
      cout << "reason3   size=" << ml.size() << endl;
      for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << " ";}cout << endl;
    }


    f="8/5kpp/8/8/3P2PP/p1n1b3/2b5/7K b - - 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    if(ml.size()!=36){r=false;cout << "reason3.5 size=" << ml.size() << endl;}
  }

  //--------------- test king -------------------------------
  {
    string f="4k3/8/8/r7/8/8/8/K2Q4 w - - 0 10";
    string s1="";
    cp.fen(f);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=3){r=false;cout << "error in test king" << endl;}
  }

  //--------------- test queen ------------------------------
  {
    string f="k7/8/3p4/2p5/3Q4/8/8/7K w - - 0 10";
    cp.fen(f);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=26){
      r=false;
      cout << "reason4  ml.size()=" << ml.size() << endl;
      for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << " ";}cout << endl;
    }
  }

  //--------------- test rook endgame ------------------------------
  {
    string f="8/8/6R1/5rp1/8/6k1/4K3/8 b - - 4 63";
    cp.fen(f);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=19)
      {
	r=false;
	cout << "reason5  " << ml.size() << endl;
      }
  }


  //--------------- test pin ------------------------------
  {
    string f="r1bqkbnr/ppp1pppp/2n5/8/4p3/3P4/PPPK1PPP/RNBQ1BNR w kq - 0 4";
    cp.fen(f);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=28){r=false;cout << "reason6" << endl;}
  }

  //--------------- test check mate -----------------------
  {
    string f="8/8/8/8/8/1k6/8/1K1r4 w - - 0 1";
    cp.fen(f);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=0){r=false;cout << "not check mate ??" << endl;}
  }

  //--------------- test castling -------------------------
  {
    MoveList ml;
    string f="r1bqk2r/2pp1ppp/p1n5/1pb5/3pR3/1B3N2/PPP2PPP/RNBQ2K1 b kq - 0 9";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].to()==g8){r=false;}}
  }

  {
    MoveList ml;
    string f="rnbqk2r/ppppppbp/5np1/8/8/5NP1/PPPPPPBP/RNBQK2R w Qkq - 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].to()==g1 && ml[i].piece().white_king()){
	cout << "castling error  " << ml[i].algebraic() << endl;r=false;}
    }
  }

  {
    MoveList ml;
    string f="r3k3/8/8/8/4Pp2/3K4/8/R5Rr b q - 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){
      if(ml[i].code()==castling && ml[i].check()==false){cout << "missing check in castling move" << endl;r=false;}
    }
  }

  {
    MoveList ml;
    string f="r1bqkb1r/pppp1ppp/2n1pn2/8/2B1P3/2N5/PPPP1PPP/R1BQK1NR w KQkq - 0 4";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){
      if(ml[i].code()==castling){cout << "castling move to occupied square" << endl;r=false;}
    }
  }


  //--------------- test EP  ------------------------------
  {
    string f="8/8/8/5k2/3pP3/8/8/K7 b - e3 0 1";
    cp.fen(f);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=9){cout << "error in test EP (1)" << endl;r=false;}

    f="8/8/R7/8/3pP2k/8/8/K7 b - e3 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    if(ml.size()!=7){cout << "error in test EP (2)" << endl;r=false;}

    f="8/8/8/8/R2pP2k/8/8/K7 b - e3 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    if(ml.size()!=6){cout << "error in test EP (3)" << endl;r=false;}

    f="k7/8/8/3pP3/2K5/8/8/8 w - d6 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    if(ml.size()!=9){
      r=false;
      cout << "error in test EP (4)" << endl;
    }

    f="k7/8/8/K2pP2r/8/8/8/8 w - d6 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    if(ml.size()!=6){
      r=false;
      cout << "error in test EP (5)" << endl;
    }

    f="r6r/Pp1pkppp/1P3nbN/nPp5/BB2P3/q4N2/Pp1P2PP/R2Q1RK1 w - c6 0 1";
    cp.fen(f);
    mg.generate_moves(cp,ml);
    if(ml[2].code()!=en_passant){
      r=false;
      cout << "error in test EP (6)" << endl;
      for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << " ";}cout << endl;
    }
  }

  //--------------- test generate captures -----------------------------
  {
    string s="r3k3/pP1p1p2/8/Q3N2B/8/7N/6PP/5RK1 w - - 0 1";
    cp.fen(s);
    MoveList ml;
    mg.create_movetable(cp,bb_moves);mg.serialize_captures(cp,bb_moves,ml);
    if(ml.size()!=13)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << endl;}
	cout << "reason12  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
    
    s="1Qb5/3q3k/1Q2p3/4p1p1/4P3/6PP/3B3K/8 b - - 0 1";
    cp.fen(s);
    mg.create_movetable(cp,bb_moves);mg.serialize_captures(cp,bb_moves,ml);
    if(ml.size()!=1 || !ml[0].check())
      {
	r=false;
	cout << endl;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].dump() << endl;}
	cout << cp.fen() << endl;
      }
    
  }


  //--------------- test promotions ------------------------------------
  {
    int count{0};
    string s="rnbq1b1r/ppppP1pp/5k1n/8/8/8/PPPP1PPP/RNBQKBNR w KQ - 0 1";
    cp.fen(s);
    MoveList ml;mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
  }


  //--------------- test generate checks -------------------------------
  {
    int count{0};
    string s="8/8/3k4/8/4P3/8/3NK3/B2Q1R2 w - - 0 1";
    cp.fen(s);
    MoveList ml;mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
    if(count!=7)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << "   check=" << (ml[i].check()?"T":"F") << endl;}
	cout << "error1 generate checks  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
    
    count=0;
    s="r2k4/8/7b/1q4n1/4p3/3P4/3K4/6n1 b - - 0 1";
    cp.fen(s);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
    if(count!=12)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){cout << ml[i].algebraic() << endl;}}
	cout << "error2 generate checks  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
    
    count=0;
    s="8/8/8/k7/8/8/3K3P/8 w - - 0 1";
    cp.fen(s);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
    if(count!=0)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << endl;}
	cout << "error3 generate checks  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
    
    count=0;
    s="8/p3k3/8/8/7K/8/8/8 b - - 0 1";
    cp.fen(s);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
    if(count!=0)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << endl;}
	cout << "error4 generate checks  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
    
    count=0;
    s="4k3/8/8/8/8/8/4P3/K3R3 w - - 0 1";
    cp.fen(s);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
    if(count!=0)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << "  " << (ml[i].check()?"check":"") << endl;}
	cout << "error5 generate checks  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
    
    count=0;
    s="r3k2r/p1ppqpb1/1n2pnp1/3PN3/1p2P3/2N2Q1p/PPPBbPPP/R4R1K b kq - 0 1";
    cp.fen(s);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
    if(count!=1)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << "  " << (ml[i].check()?"check":"") << endl;}
	cout << "error6 generate checks  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
    
    count=0;
    s="4k3/8/8/8/8/8/qp3K2/R2Q4 b - - 0 1";
    cp.fen(s);
    mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){if(ml[i].check()){count++;}}
    if(count!=10)
      {
	r=false;
	for(unsigned i=0;i<ml.size();i++){cout << ml[i].algebraic() << "  " << (ml[i].check()?"check":"") << endl;}
	cout << "error7 generate checks  " << ml.size() << endl;
	cout << cp.fen() << endl;
      }
  }


  
  //----------------- test chess960 ------------------------------------
  {
    string s{"r1k1r2q/p1ppp1pp/8/8/8/8/P1PPP1PP/R1K1R2Q w EAea -"};
    cp.fen(s);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=23){r=false;}
  }

  {
    string s{"2r5/8/8/8/8/8/6PP/k2KR3 w E -"};
    cp.fen(s);
    MoveList ml;mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){
      if(ml[i].code()==castling && ml[i].check()==false){cout << "chess960 missing check in castling move" << endl;r=false;}
    }
  }
  
  {
    string s{"4r3/3k4/8/8/8/8/6PP/qR1K1R2 w BF -"};
    cp.fen(s);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=19){r=false;}
  }

  cp.set_chess960(true);
  {
    cp.fen("nnrkqbbr/pppppppp/8/8/8/8/PPPPPPPP/NNRKQBBR w KQkq - 0 1");
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=20){r=false;}
  }
  
  {
    cp.fen("nnrkqbbr/pppppppp/8/8/8/8/PPPPPPPP/NNRKQBBR w CHch - 0 1");
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=20){r=false;}
  }
  
  {
    cp.fen("4k3/8/8/8/8/8/8/r1RK3R w CH - 0 1");
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=17){r=false;}
  }
  cp.set_chess960(false);

  
  //--------------- test illegal position ------------------------------

  {
    string s="R3k3/5PN1/8/8/B7/8/4Q3/4K3 w - - 0 1";
    cp.fen(s);
    MoveList ml;mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){
      if(ml[i].capture().king()){
	r=false;
	cerr << "error4 in test illegal position" << endl;
	for(unsigned i=0;i<ml.size();i++){cerr << ml[i].algebraic() << " ";}cerr << endl;
	break;
      }
    }
  }

  {
    string s="4k3/4q3/8/b7/8/8/5pn1/r3K3 b - - 0 1";
    cp.fen(s);
    MoveList ml;mg.generate_moves(cp,ml);
    for(unsigned i=0;i<ml.size();i++){
      if(ml[i].capture().king()){
	r=false;
	cerr << "error4 in test illegal position" << endl;
	for(unsigned i=0;i<ml.size();i++){cerr << ml[i].algebraic() << " ";}cerr << endl;
	break;
      }
    }
  }


  //--------------- test special position ------------------------------
  {
    string s="rnbqkb1r/pppp1ppp/2n1p3/4P3/2PP4/P7/1P3PPP/RNBQKBNR w KQkq - 1 6";
    cp.fen(s);
    cp.make_move(f2,f4);
    cp.make_move(g7,g6);
    MoveList ml;mg.generate_moves(cp,ml);
    if(ml.size()!=32)
      {
	r=false;
	cout << "reason13  " << ml.size() << endl;
	for(uint i=0;i<ml.size();i++){cout << ml[i].algebraic() << " ";}cout << endl;
	cout << cp.fen() << endl;
      }
  }

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






