/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../board/cposition.h"
#include"../hashtable/pawnhashtable.h"
#include"../board/hasher.h"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest Pawn hashtable     
//


void test_pawnhashtable()
{
  bool r=true;
  cout << "Test PawnHashTable ...................................." << flush;


  Tables tbs;
  ChessPosition cp(tbs);
  string s="4k3/pp6/8/8/8/8/5PP1/4K3 w - - 0 1";
  cp.fen(s);

  auto key=cp.get_pawnhash();

  cp.make_move(e1,d1);
  cp.make_move(e8,d8);
  auto k=cp.get_pawnhash();
  if(key!=k){r=false;}

  PHTrecord phtr;
  phtr.set_key(key);
  BitBoard bb0,bb1;
  bb1.set_square(a7);
  bb1.set_square(b7);
  bb1.set_square(f2);
  bb1.set_square(g2);
  phtr.set_pawns(bb1,bb0);

  PawnHashTable pht;
  pht.set_current(phtr);
  pht.write();

  BitBoard bb2;
  bb2.set_square(e7);
  bb2.set_square(e2);
  phtr.set_pawns(bb2,bb0);
  pht.set_current(phtr);

  pht.read(key);
  if(pht.get_current().get_white_passed_pawns()!=bb1){r=false;}

  {
    cp.newgame();
    uint64_t ph0=cp.get_pawnhash();
    cp.make_move(e2,e4);
    uint64_t ph1=cp.get_pawnhash();
    cp.make_move(e7,e5);
    uint64_t ph2=cp.get_pawnhash();
    cp.make_move(d2,d4);
    uint64_t ph3=cp.get_pawnhash();
    cp.make_move(e5,d4);
    uint64_t ph4=cp.get_pawnhash();
    cp.make_move(d1,d4);
    //int64_t ph5=cp.get_pawnhash();
    cp.unmake_move();
    uint64_t ph6=cp.get_pawnhash();
    cp.unmake_move();
    uint64_t ph7=cp.get_pawnhash();
    cp.unmake_move();
    uint64_t ph8=cp.get_pawnhash();
    cp.unmake_move();
    uint64_t ph9=cp.get_pawnhash();
    cp.unmake_move();
    uint64_t ph10=cp.get_pawnhash();
    if(ph0!=ph10 || ph1!=ph9 || ph2!=ph8 || ph3!=ph7 || ph4!=ph6){r=false;}
  }

  if(r){cout << "OK" << endl;}else{cout << "FAIL" << endl;}

}






