/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include"../hashtable/phtrecord.h"
#include"../board/cposition.h"
#include"../common/tables.h"
using namespace std;

//-------------------------------------------------------
//
//           unittest PHTrecord:
//

void test_phtrecord()
{
  cout << "Test PHTrecord ........................................" << flush;
  bool r=true;


  {
    PHTrecord A;
    BitBoard bb,bb0;
    bb.set_square(a2);
    bb.set_square(b2);
    
    uint32_t k=0xFFFFFFFFull;
    A.set_key(k);
    if(A.get_key()!=0xFFFFFFFF){r=false;cerr << "error 1" << endl;}

    A.set_pawns(bb,bb0);
    if(A.get_white_passed_pawns()!=bb){r=false;cerr << "error 2" << endl;}
  }


  //--------------------- Isolated pawns: ------------------------------
  //
  {
    PHTrecord A;
    Tables tbs;
    ChessPosition cp(tbs);
    string s="7k/8/8/8/P3P2P/8/8/7K w - - 0 1";
    cp.fen(s);
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    if(A.get_isolated_pawns()!=-3){r=false;cerr << "error(1): isolated weak pawns  n=" << A.get_isolated_pawns() << endl;}
    cp.flip();
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    if(A.get_isolated_pawns()!=3){r=false;cerr << "error(2): isolated weak pawns  n=" << A.get_isolated_pawns() << endl;}    
  }
  

  {
    //------------------- weak pawns: ----------------------------------
    PHTrecord A;
    Tables tbs;
    ChessPosition cp(tbs);
    cp.fen("4k3/7p/4p2P/1p1pP1P1/p1pP4/2P5/P7/4K3 w - - 0 1");
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    if(A.get_white_weak_pawns().popcount()!=3){r=false;cerr << "error: white weak pawns" << endl;}
    if(A.get_black_weak_pawns().popcount()!=3){r=false;cerr << "error: black weak pawns" << endl;}
  }

  {
    //------------------- pawn islands: --------------------------------
    PHTrecord A;
    Tables tbs;
    ChessPosition cp(tbs);
    string s="4k3/p3p2p/8/8/8/1PP1P3/8/4K3 b - - 0 1";
    cp.fen(s);
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    if(A.get_pawn_islands()!=1){r=false;cerr << "error 3" << endl;}

    s="4k3/pp2pp1p/7p/8/2P3P1/1PP1PP2/8/4K3 b - - 0 1";
    cp.fen(s);
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    if(A.get_pawn_islands()!=1){r=false;cerr << "error 4" << endl;}    
  }

  
  {
    Tables tbs;
    ChessPosition cp(tbs);
    string s="7k/8/1PP5/8/8/1pp5/8/7K w - - 0 1";
    cp.fen(s);
    PHTrecord A;
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    if(A.get_weighted_white_passed_pawns()!=400){r=false;cerr << "error 5    " << A.get_weighted_white_passed_pawns() <<endl;}
    if(A.get_weighted_black_passed_pawns()!=400){r=false;cerr << "error 6    " << A.get_weighted_black_passed_pawns() <<endl;}
  }
  
  {
    Tables tbs;
    ChessPosition cp(tbs);
    string s="8/8/8/8/1P5P/p1PpkpP1/8/2K5 w - - 0 1";
    cp.fen(s);
    PHTrecord A;
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    
    if(A.get_weighted_white_passed_pawns()!=100){r=false;cerr << "error 7    " << A.get_weighted_white_passed_pawns() << endl;}
    if(A.get_weighted_black_passed_pawns()!=300){r=false;cerr << "error 8    " << A.get_weighted_black_passed_pawns() << endl;}
  }
  
  {
    Tables tbs;
    ChessPosition cp(tbs);
    string s="8/8/2k1p1p1/3pP1P1/pPpP4/1p4K1/5P2/8 w - - 0 1";
    cp.fen(s);
    PHTrecord A;
    A.set_pawns(cp.white_pawn(),cp.black_pawn());
    if(A.get_weighted_white_passed_pawns()!=25){r=false;cerr << "error 9    " << A.get_weighted_white_passed_pawns() << endl;}
    if(A.get_weighted_black_passed_pawns()!=400){r=false;cerr << "error 10  " << A.get_weighted_black_passed_pawns() << endl;}
  }
  
  
  if(r){cout << "OK" << "  sizeof(PHTrecord): " << sizeof(PHTrecord) << endl;}else{cout << "FAIL" << endl;}
}  




