/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"../common/square.h"
#include"bposition.h"
#include<iostream>
#include<iomanip>
#include<sstream>
#include<utility>
#include<cassert>
using std::cout;
using std::cin;
using std::endl;
using std::hex;
using std::dec;
using std::ostringstream;
using std::swap;



BoardPosition::BoardPosition() : m_piece(vector<BitBoard>(15))
{
  m_stm=1;
};

BoardPosition::~BoardPosition()
{
};

BoardPosition::BoardPosition(const BoardPosition& x)
{
  m_stm=x.m_stm;
  m_piece=x.m_piece;
  m_board=x.m_board;
  m_white_pieces=x.m_white_pieces;
  m_black_pieces=x.m_black_pieces;
  m_pieces=x.m_pieces;
}

BoardPosition& BoardPosition::operator=(const BoardPosition& x)
{
  m_stm=x.m_stm;
  m_piece=x.m_piece;
  m_board=x.m_board;
  m_white_pieces=x.m_white_pieces;
  m_black_pieces=x.m_black_pieces;
  m_pieces=x.m_pieces;
  return *this;
}

void BoardPosition::clear()
{
  m_stm=1;
  for(auto& bb : m_piece){bb.clear();}
  for(auto& piece : m_board){piece=no_piece_id;}
};

void BoardPosition::fen(const string& s)
{
  clear();
  Piece p;
  string s1,s2,s3,s4,s5{"0"},s6{"1"};
  istringstream istr(s);
  istr >> s1 >> s2 >> s3 >> s4 >> s5 >> s6;
  try{stoi(s5);}catch(...){s5="0";}
  try{stoi(s6);}catch(...){s6="1";}

  Square square=a8;
  int i=0;
  for(auto& c : s1){
    switch(c)
      {
      case 'P':set_piece_on(square++,white_pawn_id);break;
      case 'N':set_piece_on(square++,white_knight_id);break;
      case 'B':set_piece_on(square++,white_bishop_id);break;
      case 'R':set_piece_on(square++,white_rook_id);break;
      case 'Q':set_piece_on(square++,white_queen_id);break;
      case 'K':set_piece_on(square++,white_king_id);break;
      case 'p':set_piece_on(square++,black_pawn_id);break;
      case 'n':set_piece_on(square++,black_knight_id);break;
      case 'b':set_piece_on(square++,black_bishop_id);break;
      case 'r':set_piece_on(square++,black_rook_id);break;
      case 'q':set_piece_on(square++,black_queen_id);break;
      case 'k':set_piece_on(square++,black_king_id);break;
      case '1':square++;break;
      case '2':square+=2;break;
      case '3':square+=3;break;
      case '4':square+=4;break;
      case '5':square+=5;break;
      case '6':square+=6;break;
      case '7':square+=7;break;
      case '8':square+=8;break;
      case '/':square-=16;break;
      default:
	ostringstream E;
	E << "ChessPosition::fen(" << s << ") \nunkown piece 0x" << hex << int(s[i]) << dec;
	throw std::logic_error(E.str());
      }      
    i++;
  }
  
  if(s2=="w"){set_white_to_move();}else{set_black_to_move();}
  compute_pieces();
};

void BoardPosition::set_piece_on(const Square& sq,int p)
{
  m_piece[p]|=sq.bitboard();
  m_board[sq.id()]=p;
}

void BoardPosition::remove_piece_from(const Square& sq,int p)
{
  m_piece[p]&=~sq.bitboard();
  m_board[sq.id()]=0;
}

void BoardPosition::compute_pieces()
{
  m_white_pieces=m_piece[white_pawn_id] | m_piece[white_knight_id] | m_piece[white_bishop_id]
    | m_piece[white_rook_id] |  m_piece[white_queen_id] |  m_piece[white_king_id];
  m_black_pieces=m_piece[black_pawn_id] | m_piece[black_knight_id] | m_piece[black_bishop_id]
    | m_piece[black_rook_id] |  m_piece[black_queen_id] |  m_piece[black_king_id];
  m_pieces=m_white_pieces | m_black_pieces;
}

void BoardPosition::make_move(const Move& m)
{
  switch(m.code())
    {
    case normal_move:
      remove_piece_from(m.to(),m.capture_id());
      remove_piece_from(m.from(),m.piece_id());
      set_piece_on(m.to(),m.piece_id());
      break;
    case en_passant:
      remove_piece_from(m.to()-8*(2*m_stm-1),m.capture_id());
      remove_piece_from(m.from(),m.piece_id());
      set_piece_on(m.to(),m.piece_id());
      break;
    case castling:
      {
	if(m.piece_id()==white_king_id){
	  remove_piece_from(m.from(),white_king_id);
	  remove_piece_from(m.to(),white_rook_id);
	  if(m.to()>m.from()){
	    set_piece_on(g1,white_king_id);
	    set_piece_on(f1,white_rook_id);
	  }
	  else{
	    set_piece_on(c1,white_king_id);
	    set_piece_on(d1,white_rook_id);
	  }
	}else{
	  remove_piece_from(m.from(),black_king_id);
	  remove_piece_from(m.to(),black_rook_id);
	  if(m.to()>m.from()){
	    set_piece_on(g8,black_king_id);
	    set_piece_on(f8,black_rook_id);
	  }
	  else{
	    set_piece_on(c8,black_king_id);
	    set_piece_on(d8,black_rook_id);
	  }
	}
      }
      break;
    case promo_queen:
      remove_piece_from(m.to(),m.capture_id());
      remove_piece_from(m.from(),m.piece_id());
      set_piece_on(m.to(),m.piece_id()+4);
      break;
    case promo_knight:
      remove_piece_from(m.to(),m.capture_id());
      remove_piece_from(m.from(),m.piece_id());
      set_piece_on(m.to(),m.piece_id()+1);
      break;
    case promo_bishop:
      remove_piece_from(m.to(),m.capture_id());
      remove_piece_from(m.from(),m.piece_id());
      set_piece_on(m.to(),m.piece_id()+2);
      break;
    case promo_rook:
      remove_piece_from(m.to(),m.capture_id());
      remove_piece_from(m.from(),m.piece_id());
      set_piece_on(m.to(),m.piece_id()+3);
      break;
    case nullmove:
      break;
    }
  compute_pieces();
  m_stm^=1;
}


void BoardPosition::unmake_move(const Move& m)
{
  switch(m.code())
    {
    case normal_move:
      remove_piece_from(m.to(),m.piece_id());
      set_piece_on(m.from(),m.piece_id());
      set_piece_on(m.to(),m.capture_id());
      break;
    case en_passant:
      remove_piece_from(m.to(),m.piece_id());
      set_piece_on(m.from(),m.piece_id());
      set_piece_on(m.to()+8*(2*m_stm-1),m.capture_id());
      break;
    case castling:
      {
	if(m.piece_id()==white_king_id){
	  if(m.to()>m.from()){
	    remove_piece_from(g1,white_king_id);
	    remove_piece_from(f1,white_rook_id);
	    set_piece_on(m.from(),white_king_id);
	    set_piece_on(m.to(),white_rook_id);
	  }
	  else{
	    remove_piece_from(c1,white_king_id);
	    remove_piece_from(d1,white_rook_id);
	    set_piece_on(m.from(),white_king_id);
	    set_piece_on(m.to(),white_rook_id);
	  }
	}else{
	  if(m.to()>m.from()){
	    remove_piece_from(g8,black_king_id);
	    remove_piece_from(f8,black_rook_id);
	    set_piece_on(m.from(),black_king_id);
	    set_piece_on(m.to(),black_rook_id);
	  }
	  else{
	    remove_piece_from(c8,black_king_id);
	    remove_piece_from(d8,black_rook_id);
	    set_piece_on(m.from(),black_king_id);
	    set_piece_on(m.to(),black_rook_id);
	  }
	}
      }
      break;
    case promo_queen:
      remove_piece_from(m.to(),m.piece_id()+4);
      set_piece_on(m.to(),m.capture_id());
      set_piece_on(m.from(),m.piece_id());
      break;
    case promo_knight:
      remove_piece_from(m.to(),m.piece_id()+1);
      set_piece_on(m.to(),m.capture_id());
      set_piece_on(m.from(),m.piece_id());
      break;
    case promo_bishop:
      remove_piece_from(m.to(),m.piece_id()+2);
      set_piece_on(m.to(),m.capture_id());
      set_piece_on(m.from(),m.piece_id());
      break;
    case promo_rook:
      remove_piece_from(m.to(),m.piece_id()+3);
      set_piece_on(m.to(),m.capture_id());
      set_piece_on(m.from(),m.piece_id());
      break;
    case nullmove:break;
    }
  m_piece[0].clear();  // in case capture was NoPiece();
  compute_pieces();
  m_stm^=1;
}

int BoardPosition::stm()const
{
  return m_stm;
}

bool BoardPosition::white_to_move()const
{
  return m_stm==1;
}

bool BoardPosition::black_to_move()const
{
  return m_stm==0;
}

void BoardPosition::set_white_to_move()
{
  m_stm=1;
}

void BoardPosition::set_black_to_move()
{
  m_stm=0;
}

void BoardPosition::flip()
{
  for(auto& bb : m_piece){
    bb.flip();
  }
  swap(m_piece[white_pawn_id],m_piece[black_pawn_id]);
  swap(m_piece[white_knight_id],m_piece[black_knight_id]);
  swap(m_piece[white_bishop_id],m_piece[black_bishop_id]);
  swap(m_piece[white_rook_id],m_piece[black_rook_id]);
  swap(m_piece[white_queen_id],m_piece[black_queen_id]);
  swap(m_piece[white_king_id],m_piece[black_king_id]);

  for(unsigned int i=0;i<32;i++){
    swap(m_board[i],m_board[i^56]);
  }
  for(int i=0;i<64;i++){
    if(m_board[i]>0){
      m_board[i]^=8;
    }
  }  
  switch_color_to_move();
  compute_pieces();
}

void BoardPosition::flop()
{
  for(auto& bb : m_piece){
    bb.flop();
  }
  for(int rank=0;rank<8;rank++){
    for(int file=0;file<4;file++){
      swap(m_board[8*rank+file],m_board[8*rank+(7-file)]);
    }
  }
  compute_pieces();
}

void BoardPosition::switch_color_to_move()
{
  m_stm^=1;
}


const BitBoard& BoardPosition::white_pawn()const{return m_piece[white_pawn_id];}
const BitBoard& BoardPosition::white_knight()const{return m_piece[white_knight_id];}
const BitBoard& BoardPosition::white_bishop()const{return m_piece[white_bishop_id];}
const BitBoard& BoardPosition::white_rook()const{return m_piece[white_rook_id];}
const BitBoard& BoardPosition::white_queen()const{return m_piece[white_queen_id];}
const BitBoard& BoardPosition::white_king()const{return m_piece[white_king_id];}
const BitBoard& BoardPosition::black_pawn()const{return m_piece[black_pawn_id];}
const BitBoard& BoardPosition::black_knight()const{return m_piece[black_knight_id];}
const BitBoard& BoardPosition::black_bishop()const{return m_piece[black_bishop_id];}
const BitBoard& BoardPosition::black_rook()const{return m_piece[black_rook_id];}
const BitBoard& BoardPosition::black_queen()const{return m_piece[black_queen_id];}
const BitBoard& BoardPosition::black_king()const{return m_piece[black_king_id];}
const uint64_t& BoardPosition::piece(int x)const{return BitBoard::to_ull(m_piece[x]);}

const BitBoard& BoardPosition::white_pieces()const
{
  return m_white_pieces; 
}

const BitBoard& BoardPosition::black_pieces()const
{
  return m_black_pieces;
}

const BitBoard& BoardPosition::pieces()const
{
  return m_pieces;
}

const int& BoardPosition::operator[](const int& x)const
{
  return m_board[x];
}


bool BoardPosition::corrupt(uint64_t mc)const
{
  if(m_piece[white_king_id].popcount()!=1){return true;}
  if(m_piece[black_king_id].popcount()!=1){return true;}
  
  for(int i=white_pawn_id;i<=black_king_id;i++){
    for(int j=i+1;j<=black_king_id;j++){
      if(m_piece[i]&m_piece[j]){return true;}
    }
  }
  return false;
}

bool BoardPosition::operator==(const BoardPosition& x)const
{  
  return m_stm==x.m_stm && m_piece==x.m_piece;
}

bool BoardPosition::operator!=(const BoardPosition& x)const
{
  return m_stm!=x.m_stm || m_piece!=x.m_piece;
}

string BoardPosition::dump()const
{
  ostringstream oss;
  oss << "side to move: " << m_stm << endl;
  oss << "white pawn: " << endl;
  oss << m_piece[white_pawn_id].dump() << endl;
  oss << "white knight: " << endl;
  oss << m_piece[white_knight_id].dump() << endl;
  oss << "white bishop: " << endl;
  oss << m_piece[white_bishop_id].dump() << endl;
  oss << "white rook: " << endl;
  oss << m_piece[white_rook_id].dump() << endl;
  oss << "white queen: " << endl;
  oss << m_piece[white_queen_id].dump() << endl;
  oss << "white king: " << endl;
  oss << m_piece[white_king_id].dump() << endl;
  oss << "black pawn: " << endl;
  oss << m_piece[black_pawn_id].dump() << endl;
  oss << "black knight: " << endl;
  oss << m_piece[black_knight_id].dump() << endl;
  oss << "black bishop: " << endl;
  oss << m_piece[black_bishop_id].dump() << endl;
  oss << "black rook: " << endl;
  oss << m_piece[black_rook_id].dump() << endl;
  oss << "black queen: " << endl;
  oss << m_piece[black_queen_id].dump() << endl;
  oss << "black king: " << endl;
  oss << m_piece[black_king_id].dump() << endl;
  return oss.str();
}

bool BoardPosition::consistent()const
{
  for(int i=0;i<64;i++){
    Square sq{i};
    switch(m_board[i]){
    case no_piece_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case white_pawn_id:
      if(sq.bitboard() & white_pawn())break;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case white_knight_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())break;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case white_bishop_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())break;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case white_rook_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())break;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case white_queen_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())break;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case white_king_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())break;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case black_pawn_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())break;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case black_knight_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())break;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case black_bishop_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())break;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case black_rook_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())break;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())return false;
      break;
    case black_queen_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())break;
      if(sq.bitboard() & black_king())return false;
      break;
    case black_king_id:
      if(sq.bitboard() & white_pawn())return false;
      if(sq.bitboard() & white_knight())return false;
      if(sq.bitboard() & white_bishop())return false;
      if(sq.bitboard() & white_rook())return false;
      if(sq.bitboard() & white_queen())return false;
      if(sq.bitboard() & white_king())return false;
      if(sq.bitboard() & black_pawn())return false;
      if(sq.bitboard() & black_knight())return false;
      if(sq.bitboard() & black_bishop())return false;
      if(sq.bitboard() & black_rook())return false;
      if(sq.bitboard() & black_queen())return false;
      if(sq.bitboard() & black_king())break;
      break;
    }
  }
  return true;
}

