/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<fstream>
#include<random>
#include<cassert>
#include"../movegen/move.h"
#include"hasher.h"
#include"cposition.h"
using std::cout;
using std::endl;
using std::hex;
using std::dec;
using std::ofstream;
using std::ios_base;


template<int I>
Hasher<I>::Hasher(const Tables& x) : m_tbl(x)
{
  m_hash.clear();m_hash.push_back(0);
  m_pawnhash.clear();m_pawnhash.push_back(0);
}

template<int I>
void Hasher<I>::init(const ChessPosition& x)
{
  m_pcp=&x;
  init(m_pcp);
}

template<int I>
void Hasher<I>::init(const ChessPosition* x)
{
  BitBoard bb_sq,p;
  Square sq;
  m_pcp=x;
  //Piece p;
  m_hash.clear();m_hash.push_back(0);
  m_pawnhash.clear();m_pawnhash.push_back(0);

  p=m_pcp->white_pawn();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+sq.id());
    m_pawnhash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+sq.id());
  }
  p=m_pcp->white_knight();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(white_knight_id*64+sq.id());
  }
  p=m_pcp->white_bishop();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(white_bishop_id*64+sq.id());
  }
  p=m_pcp->white_rook();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(white_rook_id*64+sq.id());
  }
  p=m_pcp->white_queen();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(white_queen_id*64+sq.id());
  }
  p=m_pcp->white_king();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(white_king_id*64+sq.id());
  }

  p=m_pcp->black_pawn();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+sq.id());
    m_pawnhash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+sq.id());
  }
  p=m_pcp->black_knight();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(black_knight_id*64+sq.id());
  }
  p=m_pcp->black_bishop();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(black_bishop_id*64+sq.id());
  }
  p=m_pcp->black_rook();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(black_rook_id*64+sq.id());
  }
  p=m_pcp->black_queen();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(black_queen_id*64+sq.id());
  }
  p=m_pcp->black_king();
  while(p){
    bb_sq=p.get_reset_square();
    sq=bb_sq.get_first_square();
    m_hash.back()^=m_tbl.piece_square<I>(black_king_id*64+sq.id());
  }


}

template<int I>
void Hasher<I>::make_move(const Move& m)
{
  //Move m{mm};
  m_hash.push_back(m_hash.back());
  m_pawnhash.push_back(m_pawnhash.back());
  switch(m.code())
    {
    case normal_move:
      m_hash.back()^=m_tbl.piece_square<I>(m.piece_id()*64+m.from().id());
      m_hash.back()^=m_tbl.piece_square<I>(m.piece_id()*64+m.to().id());
      m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
      m_pawnhash.back()^=m_tbl.pawn_square<I>(m.piece_id()*64+m.from().id());
      m_pawnhash.back()^=m_tbl.pawn_square<I>(m.piece_id()*64+m.to().id());
      m_pawnhash.back()^=m_tbl.pawn_square<I>(m.capture_id()*64+m.to().id());
      break;
    case promo_queen:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(white_queen_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}
      else
	{
	  m_hash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(black_queen_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}      
      break;
    case promo_rook:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(white_rook_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}
      else
	{
	  m_hash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(black_rook_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}      
      break;
    case promo_bishop:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(white_bishop_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}
      else
	{
	  m_hash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(black_bishop_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}      
      break;
    case promo_knight:
      if(m.to()>m.from())
	{
	  m_hash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(white_knight_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}
      else
	{
	  m_hash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_pawnhash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.from().id());
	  m_hash.back()^=m_tbl.piece_square<I>(black_knight_id*64+m.to().id());
	  m_hash.back()^=m_tbl.piece_square<I>(m.capture_id()*64+m.to().id());
	}      
      break;
    case castling:
      if(m.piece().white_king()){
	m_hash.back()^=m_tbl.piece_square<I>(white_king_id*64+m.from().id());
	m_hash.back()^=m_tbl.piece_square<I>(white_rook_id*64+m.to().id());
	if(m.from()<m.to()){
	  m_hash.back()^=m_tbl.piece_square<I>(white_king_id*64+g1);
	  m_hash.back()^=m_tbl.piece_square<I>(white_rook_id*64+f1);
	}else{
	  m_hash.back()^=m_tbl.piece_square<I>(white_king_id*64+c1);
	  m_hash.back()^=m_tbl.piece_square<I>(white_rook_id*64+d1);
	}
      }else{
	m_hash.back()^=m_tbl.piece_square<I>(black_king_id*64+m.from().id());
	m_hash.back()^=m_tbl.piece_square<I>(black_rook_id*64+m.to().id());
	if(m.from()<m.to()){
	  m_hash.back()^=m_tbl.piece_square<I>(black_king_id*64+g8);
	  m_hash.back()^=m_tbl.piece_square<I>(black_rook_id*64+f8);
	}else{
	  m_hash.back()^=m_tbl.piece_square<I>(black_king_id*64+c8);
	  m_hash.back()^=m_tbl.piece_square<I>(black_rook_id*64+d8);
	}
      }
      break;
    case en_passant:
      m_hash.back()^=m_tbl.piece_square<I>(m.piece_id()*64+m.from().id());
      m_hash.back()^=m_tbl.piece_square<I>(m.piece_id()*64+m.to().id());
      m_pawnhash.back()^=m_tbl.pawn_square<I>(m.piece_id()*64+m.from().id());
      m_pawnhash.back()^=m_tbl.pawn_square<I>(m.piece_id()*64+m.to().id());
      if(m.to()>m.from()){
	m_hash.back()^=m_tbl.piece_square<I>(black_pawn_id*64+m.to().id()-8);
	m_pawnhash.back()^=m_tbl.pawn_square<I>(black_pawn_id*64+m.to().id()-8);
      }else{
	m_hash.back()^=m_tbl.piece_square<I>(white_pawn_id*64+m.to().id()+8);
	m_pawnhash.back()^=m_tbl.pawn_square<I>(white_pawn_id*64+m.to().id()+8);
      }
      break;
    case nullmove:
      break;
    }
}

template<int I>
void Hasher<I>::unmake_move()
{
  m_hash.pop_back();
  m_pawnhash.pop_back();
}

template<int I>
uint64_t Hasher<I>::castle()const
{
  constexpr short castle_K=0x0080;
  constexpr short castle_Q=0x0001;
  constexpr short castle_k=0x8000;
  constexpr short castle_q=0x0100;
  uint64_t r{0};
  int c=m_pcp->castlingrights();
  if(c==0)return 0;
  if(c&castle_Q){r^=m_tbl.castle_Q<I>();}
  if(c&castle_K){r^=m_tbl.castle_K<I>();}
  if(c&castle_q){r^=m_tbl.castle_q<I>();}
  if(c&castle_k){r^=m_tbl.castle_k<I>();}
  if(c & 0x7E7E){r^=static_cast<uint64_t>(c);}   // only for chess960
  
  return r;
}

template<int I>
uint64_t Hasher<I>::hash()const
{
  uint64_t r=m_hash.back();
  r^=castle();
  if(m_pcp->ep()!=no_square){
    r^=m_tbl.en_passant<I>((m_pcp->ep()).file_number());
  }
  r^=m_tbl.turn<I>()*m_pcp->stm();
  return r;
}

template<int I>
uint64_t Hasher<I>::pawnhash()const
{
  return m_pawnhash.back();
}

