/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"rep3filter.h"
#include<iostream>
#include<cassert>
using namespace std;


Rep3filter::Rep3filter()
{
}

void Rep3filter::init(uint64_t hash)
{
  mhash.clear();
  mhash.push_back(hash);
}

bool Rep3filter::repetition()const
{
  int i=mhash.size()-2;
  while(i>=0)
    {
      if(mhash[i]==0){break;}
      if(mhash[i]==mhash.back()){return true;} // twofold repetition!
      i--;
    }
  return false;
}

void Rep3filter::add(const Move& m,uint64_t hash)
{
  if(m.piece().pawn() || m.capture()){mhash.push_back(0);}
  mhash.push_back(hash);
}

void Rep3filter::remove()
{
  mhash.pop_back();
  if(mhash.back()==0){
    mhash.pop_back();
  }
}

bool Rep3filter::operator==(Rep3filter x)const
{
  if(mhash!=x.mhash)return false;
  return true;
}

bool Rep3filter::operator!=(Rep3filter x)const
{
  if(*this==x){return false;}else{return true;}
}

string Rep3filter::result()const
{
  string r{"*"};
  if(mhash.size()<3){return r;}
  uint64_t h1,h2;
  h1=mhash[mhash.size()-1];
  h2=mhash[mhash.size()-2];
  int ch1{0},ch2{0};
  for(auto& h : mhash){
    if(h!=0){
      if(h==h1){ch1++;}
      if(h==h2){ch2++;}
    }
  }
  if(ch1>=3 || ch2>=3){r="1/2-1/2 {Draw by threefold repetition}";}
  return r;
}
