/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"attacks.h"
#include"prebb.h"
#include"magicmoves.h"


Attacks::Attacks()
{
  //--------------------- knight attacks: -------------------
  array<BitBoard,64> r;
  BitBoard bb,attackbb;
  for(int i=0;i<64;i++){
    bb=BitBoard({Square(i)});
    attackbb=(bb<<10 | bb>>6) & not_ab_file;
    attackbb|=(bb<<17 | bb>>15) & not_a_file;
    attackbb|=(bb<<15 | bb>>17) & not_h_file;
    attackbb|=(bb<<6 | bb>>10) & not_gh_file;
    r[i]=attackbb;
  }
  m_knight_attacks=r;
  //---------------------------------------------------------

  //--------------------- king attacks: ---------------------
  for(int i=0;i<64;i++){
    bb=BitBoard({Square(i)});
    attackbb=bb<<1 & not_a_file;
    attackbb|=bb>>1 & not_h_file;
    attackbb|=attackbb<<8 | attackbb>>8;
    attackbb|=bb<<8 | bb>>8;
    r[i]=attackbb;
  }
  //---------------------------------------------------------
  m_king_attacks=r;
}

const BitBoard& Attacks::knight(const BitBoard& sq)const
{
  return m_knight_attacks[sq.get_unique_square().id()];
}

const BitBoard& Attacks::bishop(const BitBoard& sq,const BitBoard& pieces)const
{
  return m_mm.get_magic_moves<'B'>(sq,pieces);
}

const BitBoard& Attacks::rook(const BitBoard& sq,const BitBoard& pieces)const
{
  return m_mm.get_magic_moves<'R'>(sq,pieces);
}

BitBoard Attacks::queen(const BitBoard& sq,const BitBoard& pieces)const
{
  BitBoard r;
  r=m_mm.get_magic_moves<'B'>(sq,pieces);
  r|=m_mm.get_magic_moves<'R'>(sq,pieces);
  return r;
}

const BitBoard& Attacks::king(const BitBoard& sq)const
{
  return m_king_attacks[sq.get_unique_square().id()];
}


const BitBoard& Attacks::knight(const Square& sq)const
{
  return m_knight_attacks[sq.id()];
}

const BitBoard& Attacks::bishop(const Square& sq,const BitBoard& pieces)const
{
  return m_mm.get_magic_moves<'B'>(sq,pieces);
}

const BitBoard& Attacks::rook(const Square& sq,const BitBoard& pieces)const
{
  return m_mm.get_magic_moves<'R'>(sq,pieces);
}

BitBoard Attacks::queen(const Square& sq,const BitBoard& pieces)const
{
  BitBoard r;
  r=m_mm.get_magic_moves<'B'>(sq,pieces);
  r|=m_mm.get_magic_moves<'R'>(sq,pieces);
  return r;
}

const BitBoard& Attacks::king(const Square& sq)const
{
  return m_king_attacks[sq.id()];
}
