/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"connections.h"
#include<iostream>
using std::cout;
using std::endl;


Connections::Connections()
{
  //----------------------------- rook attacks: --------------------
  for(Square s=a1;s<=h8;s++){
    int n=s.id();
    int r=s.rank_number();
    for(int i=0;i<8;i++){
      m_rook_attacks[n]|=Square(8*r+i).bitboard();      
    }
    int f=s.file_number();
    for(int i=0;i<8;i++){
      m_rook_attacks[n]|=Square(8*i+f).bitboard();      
    }
    m_rook_attacks[n]&=~s.bitboard();
  }
  //----------------------------------------------------------------

  
  //----------------------------- bishop attacks: --------------------
  for(Square s1=a1;s1<=h8;s1++){
    int n=s1.id();
    for(int i=0;i<8;i++){
      if(s1.id() + 8*i+i >63){break;}
      Square s2(s1.id() + 8*i+i);
      if(s2.file_number()>s1.file_number()){
	m_bishop_attacks[n]|=s2.bitboard();
      }
    }
    for(int i=0;i<8;i++){
      if(s1.id() + 8*i-i >63){break;}
      Square s2(s1.id() + 8*i-i);
      if(s2.file_number()<s1.file_number()){
	m_bishop_attacks[n]|=s2.bitboard();
      }
    }
    for(int i=0;i<8;i++){
      if(s1.id() - 8*i-i <0){break;}
      Square s2(s1.id() - 8*i-i);
      if(s2.file_number()<s1.file_number()){
	m_bishop_attacks[n]|=s2.bitboard();
      }
    }
    for(int i=0;i<8;i++){
      if(s1.id() - 8*i+i <0){break;}
      Square s2(s1.id() - 8*i+i);
      if(s2.file_number()>s1.file_number()){
	m_bishop_attacks[n]|=s2.bitboard();
      }
    }
  }
  //----------------------------------------------------------------

  
  //----------------------------- queen attacks: -------------------
  for(Square s=a1;s<=h8;s++){
    int n=s.id();
    m_queen_attacks[n] = m_rook_attacks[n] | m_bishop_attacks[n];
  }
  //----------------------------------------------------------------

  
  //-------------------------line segments: ------------------------
  for(Square s1=a1;s1<=h8;s1++){
    int n1=s1.id();
    for(Square s2=s1+1;s2<=h8;s2++){
      int n2=s2.id();

      if(s1.rank_number()==s2.rank_number()){
	for(Square s=s1+1;s<s2;s++){m_line_segment[n1][n2]|=s.bitboard();m_line_segment[n2][n1]=m_line_segment[n1][n2];}
      }else if(s1.file_number()==s2.file_number()){
	for(Square s=s1+8;s<s2;s+=8){m_line_segment[n1][n2]|=s.bitboard();m_line_segment[n2][n1]=m_line_segment[n1][n2];}
      }else if(s1.file_number()-s2.file_number()==s1.rank_number()-s2.rank_number()){
	for(Square s=s1+9;s<s2;s+=9){m_line_segment[n1][n2]|=s.bitboard();m_line_segment[n2][n1]=m_line_segment[n1][n2];}
      }else if(s1.file_number()-s2.file_number()==-(s1.rank_number()-s2.rank_number())){
	for(Square s=s1+7;s<s2;s+=7){m_line_segment[n1][n2]|=s.bitboard();m_line_segment[n2][n1]=m_line_segment[n1][n2];}
      }
    }
  }
  //----------------------------------------------------------------

  
  //----------------------closed line segments: --------------------
  for(Square s1=a1;s1<=h8;s1++){
    int n1=s1.id();
    for(Square s2=a1;s2<=h8;s2++){
      int n2=s2.id();
      m_closed_line_segment[n1][n2]=m_line_segment[n1][n2]|s1.bitboard()|s2.bitboard();
    }
  }  
  //----------------------------------------------------------------
  
}

const BitBoard& Connections::rook_attacks(const Square& s1)const
{
  return m_rook_attacks[s1.id()];
}

const BitBoard& Connections::bishop_attacks(const Square& s1)const
{
  return m_bishop_attacks[s1.id()];
}

const BitBoard& Connections::queen_attacks(const Square& s1)const
{
  return m_queen_attacks[s1.id()];
}

const BitBoard& Connections::line_segment(const Square& s1,const Square& s2)const
{
  return m_line_segment[s1.id()][s2.id()];
}

const BitBoard& Connections::closed_line_segment(const Square& s1,const Square& s2)const
{
  return m_closed_line_segment[s1.id()][s2.id()];
}

