/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef PIECE_H
#define PIECE_H

#include"numerals.h"
#include<string>
using std::string;



enum piece_t {no_piece_id,white_pawn_id,white_knight_id,white_bishop_id,white_rook_id,white_queen_id,white_king_id,
	      black_pawn_id=9,black_knight_id,black_bishop_id,black_rook_id,black_queen_id,black_king_id};


class Piece
{
 protected:
  int p;
 public:
  Piece();
  Piece(const int&);
  Piece(const Piece&);
  Piece& operator=(const Piece&);
  operator bool()const;
  bool operator==(const Piece&)const;
  bool operator!=(const Piece&)const;
  bool black_piece()const;
  bool white_piece()const;

  bool pawn()const;
  bool knight()const;
  bool bishop()const;
  bool rook()const;
  bool queen()const;
  bool king()const;

  bool white_pawn()const;
  bool white_knight()const;
  bool white_bishop()const;
  bool white_rook()const;
  bool white_queen()const;
  bool white_king()const;
  bool no_piece()const;
  bool black_pawn()const;
  bool black_knight()const;
  bool black_bishop()const;
  bool black_rook()const;
  bool black_queen()const;
  bool black_king()const;

  void set_white_pawn();
  void set_white_knight();
  void set_white_bishop();
  void set_white_rook();
  void set_white_queen();
  void set_white_king();
  void set_no_piece();
  void set_black_pawn();
  void set_black_knight();
  void set_black_bishop();
  void set_black_rook();
  void set_black_queen();
  void set_black_king();
  void set_id(int);

  string name()const;
  int value()const;
  int abs_value()const;
  const int& id()const;

};


class NoPiece : public Piece{};
class WhitePawn : public Piece{public: WhitePawn(){p=white_pawn_id;}};
class WhiteKnight : public Piece{public: WhiteKnight(){p=white_knight_id;}};
class WhiteBishop : public Piece{public: WhiteBishop(){p=white_bishop_id;}};
class WhiteRook : public Piece{public: WhiteRook(){p=white_rook_id;}};
class WhiteQueen : public Piece{public: WhiteQueen(){p=white_queen_id;}};
class WhiteKing : public Piece{public: WhiteKing(){p=white_king_id;}};
class BlackPawn : public Piece{public: BlackPawn(){p=black_pawn_id;}};
class BlackKnight : public Piece{public: BlackKnight(){p=black_knight_id;}};
class BlackBishop : public Piece{public: BlackBishop(){p=black_bishop_id;}};
class BlackRook : public Piece{public: BlackRook(){p=black_rook_id;}};
class BlackQueen : public Piece{public: BlackQueen(){p=black_queen_id;}};
class BlackKing : public Piece{public: BlackKing(){p=black_king_id;}};


#endif
