/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef PREBB_H
#define PREBB_H

#include"../common/bitboard.h"

//------------------- Precomputed bitboards: ----------------------------------------------
//
//

const BitBoard a_file{a1,a2,a3,a4,a5,a6,a7,a8};
const BitBoard b_file{b1,b2,b3,b4,b5,b6,b7,b8};
const BitBoard c_file{c1,c2,c3,c4,c5,c6,c7,c8};
const BitBoard d_file{d1,d2,d3,d4,d5,d6,d7,d8};
const BitBoard e_file{e1,e2,e3,e4,e5,e6,e7,e8};
const BitBoard f_file{f1,f2,f3,f4,f5,f6,f7,f8};
const BitBoard g_file{g1,g2,g3,g4,g5,g6,g7,g8};
const BitBoard h_file{h1,h2,h3,h4,h5,h6,h7,h8};
const BitBoard rank1={a1,b1,c1,d1,e1,f1,g1,h1};
const BitBoard rank2={a2,b2,c2,d2,e2,f2,g2,h2};
const BitBoard rank3={a3,b3,c3,d3,e3,f3,g3,h3};
const BitBoard rank4={a4,b4,c4,d4,e4,f4,g4,h4};
const BitBoard rank5={a5,b5,c5,d5,e5,f5,g5,h5};
const BitBoard rank6={a6,b6,c6,d6,e6,f6,g6,h6};
const BitBoard rank7={a7,b7,c7,d7,e7,f7,g7,h7};
const BitBoard rank8={a8,b8,c8,d8,e8,f8,g8,h8};
const BitBoard rank3_7=rank3|rank4|rank5|rank6|rank7;
const BitBoard rank3_8=rank3|rank4|rank5|rank6|rank7|rank8;
const BitBoard rank6_2=rank6|rank5|rank4|rank3|rank2;
const BitBoard rank6_1=rank6|rank5|rank4|rank3|rank2|rank1;
const BitBoard not_a_file{~a_file};
const BitBoard not_h_file{~h_file};
const BitBoard ab_file{a1,a2,a3,a4,a5,a6,a7,a8,b1,b2,b3,b4,b5,b6,b7,b8};
const BitBoard gh_file{g1,g2,g3,g4,g5,g6,g7,g8,h1,h2,h3,h4,h5,h6,h7,h8};
const BitBoard not_ab_file{~ab_file};
const BitBoard not_gh_file{~gh_file};

const BitBoard centre={d4,e4,d5,e5};
const BitBoard bigcentre={c3,d3,e3,f3,c4,d4,e4,f4,c5,d5,e5,f5,c6,d6,e6,f6};
const BitBoard lower_half=rank1|rank2|rank3|rank4;
const BitBoard upper_half=rank5|rank6|rank7|rank8;
const BitBoard queens_half=a_file|b_file|c_file|d_file;
const BitBoard kings_half=e_file|f_file|g_file|h_file;
const BitBoard lower_left=lower_half & queens_half;
const BitBoard upper_left=upper_half & queens_half;
const BitBoard lower_right=lower_half & kings_half;
const BitBoard upper_right=upper_half & kings_half;
const BitBoard whole_board=kings_half | queens_half;
const BitBoard diagonal={a1,b2,c3,d4,e5,f6,g7,h8};
const BitBoard antidiagonal={a8,b7,c6,d5,e4,f3,g2,h1};
const BitBoard diagonals=diagonal | antidiagonal;

const BitBoard hborder{a1,b1,c1,d1,e1,f1,g1,h1,a8,b8,c8,d8,e8,f8,g8,h8};
const BitBoard vborder{a1,a2,a3,a4,a5,a6,a7,a8,h1,h2,h3,h4,h5,h6,h7,h8};
const BitBoard border{a1,b1,c1,d1,e1,f1,g1,h1,a8,b8,c8,d8,e8,f8,g8,h8,a2,a3,a4,a5,a6,a7,h2,h3,h4,h5,h6,h7};
const BitBoard border_minus1{b2,c2,d2,e2,f2,g2,b7,c7,d7,e7,f7,g7,b3,b4,b5,b6,g3,g4,g5,g6};
const BitBoard border_minus2{c3,d3,e3,f3,c6,d6,e6,f6,c4,c5,f4,f5};
const BitBoard white_corner{h1,a8};
const BitBoard black_corner{a1,h8};
const BitBoard corner{a1,h1,a8,h8};
const BitBoard castlesquares{a1,b1,c1,g1,h1,a2,b2,g2,h2,a8,b8,c8,g8,h8,a7,b7,g7,h7};
const BitBoard white_squares{b1,d1,f1,h1,a2,c2,e2,g2,b3,d3,f3,h3,a4,c4,e4,g4,b5,d5,f5,h5,a6,c6,e6,g6,b7,d7,f7,h7,a8,c8,e8,g8};
const BitBoard black_squares{~white_squares};

const BitBoard f1g1={f1,g1};
const BitBoard f8g8={f8,g8};
const BitBoard b1c1d1={b1,c1,d1};
const BitBoard b8c8d8={b8,c8,d8};
const BitBoard g1h1h2={g1,h1,h2};
const BitBoard g8h8h7={g8,h8,h7};
const BitBoard a1b1c1a2={a1,b1,c1,a2};
const BitBoard a8b8c8a7={a8,b8,c8,a7};

const BitBoard a1b2c3d4={a1,b2,c3,d4};
const BitBoard a1b1c1={a1,b1,c1};
const BitBoard a8b8c8={a8,b8,c8};
const BitBoard g1h1={g1,h1};
const BitBoard g8h8={g8,h8};
const BitBoard d3e3={d3,e3};
const BitBoard d6e6={d6,e6};
const BitBoard c2f2b3g3b6g6c7f7={c2,f2,b3,g3,b6,g6,c7,f7};

const BitBoard a8_g8={a8,b8,c8,d8,e8,f8,g8};
const BitBoard a1_g1={a1,b1,c1,d1,e1,f1,g1};
const BitBoard b8_h8={b8,c8,d8,e8,f8,g8,h8};
const BitBoard b1_h1={b1,c1,d1,e1,f1,g1,h1};
const BitBoard a2_a4_c4{a2,a3,a4,b3,b4,c4};

const BitBoard white_kingside_battlefield{e1,f1,g1,h1,e2,f2,g2,h2,e3,f3,g3,h3,e4,f4,g4,h4,e5,f5,g5,h5};
const BitBoard black_kingside_battlefield{e8,f8,g8,h8,e7,f7,g7,h7,e6,f6,g6,h6,e5,f5,g5,h5,e4,f4,g4,h4};
const BitBoard white_queenside_battlefield{a1,b1,c1,d1,a2,b2,c2,d2,a3,b3,c3,d3,a4,b4,c4,d4,a5,b5,c5,d5};
const BitBoard black_queenside_battlefield{a8,b8,c8,d8,a7,b7,c7,d7,a6,b6,c6,d6,a5,b5,c5,d5,a4,b4,c4,d4};
const BitBoard white_centre_battlefield{c1,d1,e1,f1,c2,d2,e2,f2,c3,d3,e3,f3,c4,d4,e4,f4,c5,d5,e5,f5};
const BitBoard black_centre_battlefield{c8,d8,e8,f8,c7,d7,e7,f7,c6,d6,e6,f6,c5,d5,e5,f5,c4,d4,e4,f4};
const BitBoard white_space{c1,d1,e1,f1,c2,d2,e2,f2,c3,d3,e3,f3,c4,d4,e4,f4};
const BitBoard black_space{c8,d8,e8,f8,c7,d7,e7,f7,c6,d6,e6,f6,c5,d5,e5,f5};

#endif
