/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef SQUARE_H
#define SQUARE_H

#include<array>
#include<string>
using std::string;
using std::array;


class BitBoard;

class Square
{
 private:
  int msquare;
  static const array<BitBoard,65> bb;   // 64 squares + NoSquare();

 public:
  Square();
  Square(const Square&);
  Square& operator=(const Square&);
  Square(int);
  Square(string);
  const int& id()const;
  string name()const;
  char file()const;
  char rank()const;
  int file_number()const;
  int rank_number()const;
  int color()const;
  const BitBoard& bitboard()const;
  

  bool operator==(Square)const;
  bool operator!=(Square)const;
  bool operator<(Square)const;
  bool operator>(Square)const;
  bool operator<=(Square)const;
  bool operator>=(Square)const;
  Square& operator+=(int);
  Square& operator-=(int);
  Square& operator++();
  Square operator++(int);
  Square& operator--();
  Square operator--(int);
};

Square operator+(Square,int);
Square operator+(int,Square);
Square operator-(Square,int);
Square operator-(int,Square);

int operator-(Square,Square);

Square flip(const Square&);
Square flop(const Square&);


#endif
