/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"tables.h"
#include"trandom.h"



namespace PolyglotPiece 
{
  constexpr int black_pawn=0,white_pawn=1,black_knight=2,white_knight=3,
    black_bishop=4,white_bishop=5,black_rook=6,white_rook=7,
    black_queen=8,white_queen=9,black_king=10,white_king=11;
}


Tables::Tables()
{
  int i=0;
  while(i<64)
    {
      m_piece_square1[no_piece_id*64+i]=0;
      m_piece_square1[white_pawn_id*64+i]=rndh[PolyglotPiece::white_pawn*64+i];
      m_piece_square1[white_knight_id*64+i]=rndh[PolyglotPiece::white_knight*64+i];
      m_piece_square1[white_bishop_id*64+i]=rndh[PolyglotPiece::white_bishop*64+i];
      m_piece_square1[white_rook_id*64+i]=rndh[PolyglotPiece::white_rook*64+i];
      m_piece_square1[white_queen_id*64+i]=rndh[PolyglotPiece::white_queen*64+i];
      m_piece_square1[white_king_id*64+i]=rndh[PolyglotPiece::white_king*64+i];
      m_piece_square1[black_pawn_id*64+i]=rndh[PolyglotPiece::black_pawn*64+i];
      m_piece_square1[black_knight_id*64+i]=rndh[PolyglotPiece::black_knight*64+i];
      m_piece_square1[black_bishop_id*64+i]=rndh[PolyglotPiece::black_bishop*64+i];
      m_piece_square1[black_rook_id*64+i]=rndh[PolyglotPiece::black_rook*64+i];
      m_piece_square1[black_queen_id*64+i]=rndh[PolyglotPiece::black_queen*64+i];
      m_piece_square1[black_king_id*64+i]=rndh[PolyglotPiece::black_king*64+i];
      i++;
    }

  i=0;
  while(i<64)
    {
      m_pawn_square1[no_piece_id*64+i]=0;
      m_pawn_square1[white_pawn_id*64+i]=rndh[PolyglotPiece::white_pawn*64+i];
      m_pawn_square1[white_knight_id*64+i]=0;
      m_pawn_square1[white_bishop_id*64+i]=0;
      m_pawn_square1[white_rook_id*64+i]=0;
      m_pawn_square1[white_queen_id*64+i]=0;
      m_pawn_square1[white_king_id*64+i]=0;
      m_pawn_square1[black_pawn_id*64+i]=rndh[PolyglotPiece::black_pawn*64+i];
      m_pawn_square1[black_knight_id*64+i]=0;
      m_pawn_square1[black_bishop_id*64+i]=0;
      m_pawn_square1[black_rook_id*64+i]=0;
      m_pawn_square1[black_queen_id*64+i]=0;
      m_pawn_square1[black_king_id*64+i]=0;
      i++;
    }

  i=768;
  m_castle_K1=rndh[i++];
  m_castle_Q1=rndh[i++];
  m_castle_k1=rndh[i++];
  m_castle_q1=rndh[i++];

  for(int j=0;j<8;j++)
    {
      m_en_passant1[j]=rndh[i++];
    }
  m_turn1=rndh[i];


  //----------------------------------------------------------------------------------  
  std::mt19937_64 rndgen(5633491);
  std::uniform_int_distribution<uint64_t> xrand(1,0xffffffffffffffffull);
  for(int i=0;i<15*64;i++){
    m_piece_square2[i]=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
  }
  for(int i=0;i<15*64;i++){
    m_pawn_square2[i]=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
  }
  m_castle_K2=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
  m_castle_Q2=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
  m_castle_k2=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
  m_castle_q2=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
  for(int j=0;j<8;j++)
    {
      m_en_passant2[j]=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
    }
  m_turn2=xrand(rndgen)&xrand(rndgen)&xrand(rndgen);
}


