/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef TABLES_H
#define TABLES_H



#include"../hashtable/transposition.h"
#include"../hashtable/pawnhashtable.h"
#include"../common/attacks.h"
#include"../common/connections.h"
#include"../eval/material.h"
#include"../eval/pst.h"
#include"../egt/egt4pieces.h"
#include"../search/lmr.h"


class Tables
{
private:
  TranspositionTable m_tt;
  Attacks m_atta;
  Connections m_con;
  LmrTable m_lmr;
  //-------------------- hasher: -----------------------------------
  uint64_t m_piece_square1[15*64];
  uint64_t m_pawn_square1[15*64];
  uint64_t m_castle_K1;
  uint64_t m_castle_Q1;
  uint64_t m_castle_k1;
  uint64_t m_castle_q1;
  uint64_t m_en_passant1[8];
  uint64_t m_turn1;
  
  uint64_t m_piece_square2[15*64];
  uint64_t m_pawn_square2[15*64];
  uint64_t m_castle_K2;
  uint64_t m_castle_Q2;
  uint64_t m_castle_k2;
  uint64_t m_castle_q2;
  uint64_t m_en_passant2[8];
  uint64_t m_turn2;
  //----------------------------------------------------------------

  
public: 
  Tables();
  TranspositionTable& transpositiontable(){return m_tt;}
  const Attacks& attacks()const{return m_atta;}
  const Connections& connections()const{return m_con;}
  const LmrTable& lmr()const{return m_lmr;}
  template<int I> const uint64_t& piece_square(int i)const{return I==1?m_piece_square1[i]:m_piece_square2[i];}
  template<int I> const uint64_t& pawn_square(int i)const{return I==1?m_pawn_square1[i]:m_pawn_square2[i];}
  template<int I> const uint64_t& castle_K()const{return I==1?m_castle_K1:m_castle_K2;}
  template<int I> const uint64_t& castle_Q()const{return I==1?m_castle_Q1:m_castle_Q2;}
  template<int I> const uint64_t& castle_k()const{return I==1?m_castle_k1:m_castle_k2;}
  template<int I> const uint64_t& castle_q()const{return I==1?m_castle_q1:m_castle_q2;}
  template<int I> const uint64_t& en_passant(int i)const{return I==1?m_en_passant1[i]:m_en_passant2[i];}
  template<int I> const uint64_t& turn()const{return I==1?m_turn1:m_turn2;}
};


#endif
