/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"communicator.h"
#include<iostream>
#include<fstream>
#include<sstream>
#include<stdexcept>
using std::istringstream;
using std::ostringstream;
using std::logic_error;
using std::cout;
using std::endl;
using std::ofstream;
using std::chrono::time_point;
using std::chrono::high_resolution_clock;


Communicator::Communicator()
{
  time_point<high_resolution_clock> t0,t1;
  t1=high_resolution_clock::now();
  auto d=t1-t0;
  m_id=d.count()&0x1ffull;
}

void Communicator::lock_interface_to_engine()
{
  m_mutex_interface_to_engine.lock();
}

void Communicator::lock_engine_to_interface()
{
  m_mutex_engine_to_interface.lock();
}

void Communicator::unlock_interface_to_engine()
{
  m_mutex_interface_to_engine.unlock();
}

void Communicator::unlock_engine_to_interface()
{
  m_mutex_engine_to_interface.unlock();
}


string Communicator::from_interface()
{
  string m_command;
  lock_interface_to_engine();
  if(m_interface_to_engine.size()>0){m_command=m_interface_to_engine[0];m_interface_to_engine.pop_front();}
  unlock_interface_to_engine();
  if(m_debug && !m_command.empty()){m_ofs << "Communicator::from_interface  " << m_command << endl;}  
  return m_command;
}

string Communicator::from_engine()
{
  string m_command;
  lock_engine_to_interface();
  if(m_engine_to_interface.size()>0){m_command=m_engine_to_interface[0];m_engine_to_interface.pop_front();}
  unlock_engine_to_interface();
  if(m_debug && !m_command.empty()){m_ofs << "Communicator::from_engine     " << m_command << endl;}  
  return m_command;
}

void Communicator::to_interface(string s)
{
  lock_engine_to_interface();
  m_engine_to_interface.push_back(s);
  unlock_engine_to_interface();
  if(m_debug && !s.empty()){m_ofs << "Communicator::to_interface    " << s << endl;}  
}

void Communicator::to_engine(string s)
{
  lock_interface_to_engine();
  m_interface_to_engine.push_back(s);
  unlock_interface_to_engine();
  if(m_debug && !s.empty()){m_ofs << "Communicator::to_engine       " << s << endl;}  
}

void Communicator::set_id(int x)
{
  m_id=x;
}

int Communicator::id()const
{
  return m_id;
}

void Communicator::set_log(bool x)
{
  m_debug=x;
  ostringstream oss;
  if(x){
    oss << "comm_" << m_id << ".log";
    m_ofs.open(oss.str());
  }
}

void Communicator::clear()
{
  lock_engine_to_interface();
  m_engine_to_interface.clear();
  unlock_engine_to_interface();

  lock_interface_to_engine();
  m_interface_to_engine.clear();
  unlock_interface_to_engine();
}
