/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef COMMUNICATOR_H
#define COMMUNICATOR_H

#include<functional>
#include<chrono>
#include<mutex>
#include<deque>
#include<string>
#include<fstream>
using std::deque;
using std::string;
using std::mutex;
using std::ofstream;


class Communicator
{
 private:
  int m_id;
  bool m_debug{false};
  ofstream m_ofs;
  mutex m_mutex_interface_to_engine;
  mutex m_mutex_engine_to_interface;
  deque<string> m_interface_to_engine;
  deque<string> m_engine_to_interface;

 public:
  Communicator();
  Communicator(const Communicator&)=delete;
  Communicator& operator=(const Communicator&)=delete;
  Communicator(Communicator&&)=delete;
  Communicator& operator=(Communicator&&)=delete;
  void lock_interface_to_engine();
  void lock_engine_to_interface();
  void unlock_interface_to_engine();
  void unlock_engine_to_interface();
  string from_interface();
  string from_engine();
  void to_engine(string);
  void to_interface(string);
  void set_id(int);
  int id()const;
  void set_log(bool);
  void clear();
};


#endif
