/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"egt4pieces.h"
#include"../common/numerals.h"
#include<fstream>
#include<sstream>





Egt4Pieces::Egt4Pieces(const Attacks& a,piece_t p0,piece_t p1) : m_atta(a)
{
  m_egt_name="unknown";
  m_p_id[0]=p0;
  m_p_id[1]=p1;
  if(p0==white_knight_id && p1==white_bishop_id){m_egt_name="KBNK";}
  if(p0==white_queen_id && p1==black_rook_id){m_egt_name="KQKR";}
  if(p0==white_rook_id && p1==black_knight_id){m_egt_name="KRKN";}
}

Egt4Pieces::~Egt4Pieces()
{
  if(m_pp && m_memory_owner){
    delete[] m_pp;
  }
}

bool Egt4Pieces::busy()const
{
  return (m_depth>=0 && m_depth<=100);
}

bool Egt4Pieces::ready()const
{
  return m_depth>100;
}

void Egt4Pieces::clear()
{
  m_depth=-1;
  if(m_pp && m_memory_owner){
    delete[] m_pp;
    m_pp=nullptr;
  }
}

void Egt4Pieces::set_memory(unsigned char* pc)
{
  m_depth=-1;
  if(m_pp && m_memory_owner){
    delete[] m_pp;
    m_pp=nullptr;
  }
  m_memory_owner=false;
  m_pp=pc;
}

bool Egt4Pieces::operator==(const Egt4Pieces& x)const
{
  if(m_p_id[0]!=x.m_p_id[0] || m_p_id[1]!=x.m_p_id[1] || m_size!=x.m_size){return false;}
  for(unsigned i=0;i<m_size;i++){
    if(m_pp[i]!=x.m_pp[i]){return false;}
  }
  return true;
}

bool Egt4Pieces::operator!=(const Egt4Pieces& x)const
{
  return !operator==(x);
}

unsigned Egt4Pieces::operator[](unsigned x)const
{
  return static_cast<unsigned>(m_pp[x]);
}

int Egt4Pieces::stm()const
{
  return m_stm;
}

unsigned Egt4Pieces::size()const
{
  return m_size;
}

void Egt4Pieces::fen(string s)
{
  m_p[0].clear();
  m_p[1].clear();
  m_wk.clear();
  m_bk.clear();
  m_sign=1;

  string s1,s2;
  istringstream istr(s);
  istr >> s1 >> s2;

  Square square=a8;
  for(auto& c : s1){
    switch(c){
    case 'N':if(m_p_id[0]==white_knight_id){m_p[0]|=square.bitboard();}else{m_p[1]=square.bitboard();};square++;break;
    case 'B':if(m_p_id[0]==white_bishop_id){m_p[0]|=square.bitboard();}else{m_p[1]=square.bitboard();};square++;break;
    case 'R':if(m_p_id[0]==white_rook_id){m_p[0]|=square.bitboard();}else{m_p[1]=square.bitboard();};square++;break;
    case 'Q':if(m_p_id[0]==white_queen_id){m_p[0]|=square.bitboard();}else{m_p[1]=square.bitboard();};square++;break;
    case 'K':m_wk=square.bitboard();square++;break;
    case 'n':if(m_p_id[0]==white_knight_id && m_p_id[1]!=black_knight_id){m_p[0]|=square.bitboard();m_sign=-1;}else{m_p[1]=square.bitboard();};square++;break;
    case 'b':if(m_p_id[0]==white_bishop_id && m_p_id[1]!=black_bishop_id){m_p[0]|=square.bitboard();m_sign=-1;}else{m_p[1]=square.bitboard();};square++;break;
    case 'r':if(m_p_id[0]==white_rook_id && m_p_id[1]!=black_rook_id){m_p[0]|=square.bitboard();m_sign=-1;}else{m_p[1]=square.bitboard();};square++;break;
    case 'q':if(m_p_id[0]==white_queen_id && m_p_id[1]!=black_queen_id){m_p[0]|=square.bitboard();m_sign=-1;}else{m_p[1]=square.bitboard();};square++;break;
    case 'k':m_bk=square.bitboard();square++;break;
    case '1':square++;break;
    case '2':square+=2;break;
    case '3':square+=3;break;
    case '4':square+=4;break;
    case '5':square+=5;break;
    case '6':square+=6;break;
    case '7':square+=7;break;
    case '8':square+=8;break;
    case '/':square-=16;break;
    default:
      ostringstream E;
      E << "Egt4Pieces::fen(" << s << ") \nwrong piece";
      throw std::logic_error(E.str());
    }      
  }  
  m_stm=s2=="w"?white:black;

  if(m_sign==-1){
    m_stm^=1;
    swap(m_wk,m_bk);
  }
}

string Egt4Pieces::fen()const
{
  int empty{0};
  std::ostringstream oss;
  Square left_square{a8};
  for(int i=0;i<8;i++){
    for(Square s=left_square;s<left_square+8;s++){
      if(m_wk & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	oss << "K";
      }else if(m_bk & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	oss << "k";
      }else if(m_p[0] & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	if(m_p_id[0]==white_knight_id){
	  oss << "N";
	}else if(m_p_id[0]==white_bishop_id){
	  oss << "B";
	}else if(m_p_id[0]==white_rook_id){
	  oss << "R";
	}else if(m_p_id[0]==white_queen_id){
	  oss << "Q";
	}
      }else if(m_p[1] & s.bitboard()){
	if(empty>0){oss << empty;empty=0;}
	if(m_p_id[1]==white_bishop_id){
	  oss << "B";
	}else if(m_p_id[1]==black_knight_id){
	  oss << "n";
	}else if(m_p_id[1]==black_rook_id){
	  oss << "r";
	}
      }else{
	empty++;
      }
    }
    if(empty>0){oss << empty;empty=0;}
    if(left_square>a1){
      oss << "/";    
    }else{
      oss << " " << (m_stm==white?"w":"b") << " - - 0 1";
    }
    left_square-=8;
  }
  return oss.str();
}

void Egt4Pieces::set_key(unsigned key)
{
  Square wk_square[10]={a1,b1,c1,d1,b2,c2,d2,c3,d3,d4};
  Square sK=Square(wk_square[key>>19 & 0x3f].id());
  Square sk=Square(key>>13  & 0x3f);
  Square s2=Square(key>>7 & 0x3f);
  Square s1=Square(key>>1  & 0x3f);
  m_sign=1;
  m_stm=key & 0x1;
  m_wk=sK.bitboard();
  m_bk=sk.bitboard();
  m_p[1]=s2.bitboard();
  m_p[0]=s1.bitboard();
  if(m_p[0]==m_bk){m_p[0].clear();}
  if(m_p[1]==m_bk){m_p[1].clear();}
  if(m_p[0]==m_wk){m_p[0].clear();}
  if(m_p[1]==m_wk){m_p[1].clear();}
}

unsigned Egt4Pieces::get_key()const
{
  int wk_square_id[28]={0,1,2,3,64,64,64,64,64,4,5,6,64,64,64,64,64,64,7,8,64,64,64,64,64,64,64,9};

  BitBoard bb_wk{m_wk},bb_bk{m_bk},bb_p1{m_p[0]},bb_p2{m_p[1]};
  //---------------------------- use symmetry: ----------------------------
  //
  //------------ missing piece: -------------------------------------------
  //
  if(!bb_p2){bb_p2=bb_bk;}
  if(!bb_p1){bb_p1=bb_bk;}
  //
  //------------ white king in a1-d1-d4 triangle: -------------------------
  //
  
  if(bb_wk.get_unique_square().id()>=32){
    bb_wk.flip();
    bb_bk.flip();
    bb_p1.flip();
    bb_p2.flip();
  }
  if(bb_wk.get_unique_square().bitboard() & BitBoard(0xf0f0f0f0f0f0f0f0)){
    bb_wk.flop();
    bb_bk.flop();
    bb_p1.flop();
    bb_p2.flop();
  }
  if(bb_wk & a2_a4_c4){
    Square wk=bb_wk.get_unique_square();
    Square bk=bb_bk.get_unique_square();
    Square p1=bb_p1.get_unique_square();
    Square p2=bb_p2.get_unique_square();
    
    wk=Square(wk.file_number()*8 + wk.rank_number());
    bk=Square(bk.file_number()*8 + bk.rank_number());
    p1=Square(p1.file_number()*8 + p1.rank_number());
    p2=Square(p2.file_number()*8 + p2.rank_number());

    bb_wk=wk.bitboard();
    bb_bk=bk.bitboard();
    bb_p2=p2.bitboard();
    bb_p1=p1.bitboard();
  }

  unsigned key=wk_square_id[bb_wk.get_unique_square().id()]<<19;
  key|=bb_bk.get_unique_square().id()<<13;
  key|=bb_p2.get_unique_square().id()<<7;
  key|=bb_p1.get_unique_square().id()<<1;
  key|=m_stm;
  return key;
}

unsigned Egt4Pieces::get_mirror_key(unsigned key)const
{
  Square sk=Square(key>>13  & 0x3f);
  Square s2=Square(key>>7 & 0x3f);
  Square s1=Square(key>>1  & 0x3f);
  unsigned stm=key&1;
  
  sk=Square(sk.file_number()*8 + sk.rank_number());
  s2=Square(s2.file_number()*8 + s2.rank_number());
  s1=Square(s1.file_number()*8 + s1.rank_number());
  
  key&=0x3F << 19;
  key|=sk.id()<<13;
  key|=s2.id()<<7;
  key|=s1.id()<<1;
  key|=stm;
  
  return key;
}

string Egt4Pieces::get_key_str(unsigned key)const
{
  Square wk_square[10]={a1,b1,c1,d1,b2,c2,d2,c3,d3,d4};
  ostringstream oss;
  oss << "K: ";
  Square sK=Square(wk_square[key>>19&0x3f]);
  oss << sK.name() << "   ";
  oss << "k: ";
  Square sk=Square(key>>13&0x3f);
  oss << sk.name() << "   ";
  if(m_p_id[0]==white_knight_id){
    oss << "N: ";
  }else if(m_p_id[0]==white_bishop_id){
    oss << "B: ";
  }else if(m_p_id[0]==white_rook_id){
    oss << "R: ";
  }else if(m_p_id[0]==white_queen_id){
    oss << "Q: ";
  }
  Square s1=Square(key>>1&0x3f);
  oss << s1.name() << "   ";
  if(m_p_id[1]==white_knight_id){
    oss << "N: ";
  }else if(m_p_id[1]==black_knight_id){
    oss << "n: ";
  }else if(m_p_id[1]==white_bishop_id){
    oss << "B: ";
  }else if(m_p_id[1]==black_bishop_id){
    oss << "b: ";
  }else if(m_p_id[1]==white_rook_id){
    oss << "R: ";
  }else if(m_p_id[1]==black_rook_id){
    oss << "r: ";
  }else if(m_p_id[1]==white_queen_id){
    oss << "Q: ";
  }else if(m_p_id[1]==black_queen_id){
    oss << "q: ";
  }
  Square s2=Square(key>>7&0x3f);
  oss << s2.name() << "   ";
  int color=key&1;
  oss << (color==white?"(white)":"(black)");
  return oss.str();
}

string Egt4Pieces::get_key_str()const
{
  unsigned key=get_key();
  return get_key_str(key);
}

string Egt4Pieces::mate_in(int x)const
{
  ostringstream oss;
  for(unsigned i=0;i<m_size;i++){
    if(m_pp[i]==x){
      oss << "mate in " << x << ":   " << fen() << endl;
    }
  }
  return oss.str();
}

int Egt4Pieces::diff(const Egt4Pieces& x)const
{
  unsigned d1{m_size},p{0};
  for(unsigned i=0;i<m_size;i++){
    if(m_pp[i]!=x.m_pp[i]){
      if(abs(m_pp[i])<d1){
	p=i;
	d1=abs(m_pp[i]);
      }
    }
  }
  return p;
}

int Egt4Pieces::eval()const
{
  unsigned x=get_key();
  return eval(x);
}

int Egt4Pieces::eval(string s)
{
  fen(s);
  unsigned x=get_key();
  return eval(x);
}

int Egt4Pieces::eval(unsigned x)const
{
  if(!m_pp || m_depth<0){return static_cast<int>(0xFF);}
  unsigned char c=m_pp[x];
  if(c==0xFD){return 0;}
  if(c==0xFE || c==0xFF){return static_cast<int>(c);}
  if(c&0x80){ // black wins
    c&=0x7f;
    return (-mate_value + static_cast<int>(c))*m_sign; 
  }else{      // white wins
    return (mate_value - static_cast<int>(c))*m_sign; 
  }
}

bool Egt4Pieces::create()
{
  if(m_depth<0){
    m_tim.start();
    m_depth=0;
    if(!m_pp && m_memory_owner){
      m_pp=new unsigned char[m_size];
    }
    set_initial_eval();
    m_tim.stop();
    return false;
  }else if(m_depth>100){
    return true;
  }

  m_tim.start();
  create_next();
  m_tim.stop();
  if(m_depth>=100){
    for(unsigned i=0;i<m_size;i++){
      if(m_pp[i]==0xFF){m_pp[i]=0xFD;}
    }
    return true;
  }else{
    return false;
  }
}

void Egt4Pieces::create_next()
{
  for(unsigned i=0;i<m_size;i++){
    if((m_pp[i]&0x7F)!=m_depth){continue;}

    //------------------------------- stm is losing: ----------------------------------------------------------
    //
    if(   ((i&1)==white && (m_pp[i]&0x80)) || (((i&1)==black) && !(m_pp[i]&0x80))   ){
      set_key(i);
      vector<Move> ml;
      generate_previous_moves(ml);
      for(auto& m : ml){
	unmake_move(m);
	unsigned j=get_key();
	if(m_pp[j]==0xFF){
	  m_pp[j]=m_pp[i]+1;
	  if(m_wk & diagonals){
	    m_pp[get_mirror_key(j)]=m_pp[j];
	  }
	}
	make_move(m);
      }
    }

    //------------------------------- stm is winning: ---------------------------------------------------------
    //
    if(((i&1)==white && !(m_pp[i]&0x80)) || ((i&1)==black && (m_pp[i]&0x80))){
      set_key(i);
      vector<Move> ml;
      generate_previous_moves(ml);
      for(auto& m : ml){
	unmake_move(m);
	unsigned j=get_key();
	if(m_pp[j]==0xFF){
	  bool finished=true;
	  vector<Move> ml2;
	  generate_moves(ml2);
	  for(auto& m2 : ml2){
	    make_move(m2);
	    unsigned t=get_key();
	    if(m_pp[t]==0xFF){
	      unmake_move(m2);finished=false;break;
 	    }else if(m_pp[t]==0xFD){
	      unmake_move(m2);finished=false;break;
 	    }else if(m_pp[t]==0xFE){
	      unmake_move(m2);continue;
	    }else if(((t&1)==white && (m_pp[t]&0x80)) || ((t&1)==black && !(m_pp[t]&0x80))){
	      unmake_move(m2);finished=false;break;
	    }
	    unmake_move(m2);
	  }
	  if(finished){
	    m_pp[j]=m_pp[i]+1;
	    if(m_wk & diagonals){
	      m_pp[get_mirror_key(j)]=m_pp[j];
	    }
	  }
	}
	make_move(m);
      }
    }
	 
  }
  
  m_depth++;
}

void Egt4Pieces::set_initial_eval()
{
  for(unsigned i=0;i<m_size;i++){
    m_pp[i]=0xFF;
    set_initial_eval(i);
  }
}

void Egt4Pieces::set_initial_eval(unsigned key)
{
  if(!is_legal_key(key)){m_pp[key]=0xFE;return;}
  
  set_key(key);
  if(!is_legal()){m_pp[key]=0xFE;return;}


  //------------------- check? -----------------------------------------
  BitBoard pieces=m_wk | m_p[0] | m_p[1] | m_bk;
  BitBoard p0_moves,p1_moves;

  if(m_p[0]){
    switch(m_p_id[0]){
    case white_knight_id:p0_moves=m_atta.knight(m_p[0]);break;
    case white_bishop_id:p0_moves=m_atta.bishop(m_p[0],pieces);break;
    case white_rook_id:p0_moves=m_atta.rook(m_p[0],pieces);break;
    case white_queen_id:p0_moves=m_atta.queen(m_p[0],pieces);break;
    default: throw logic_error("Egt4Pieces::set_initial_eval(unsigned key)  unknown piece1");
    }
  }
  if(m_p[1]){
    switch(m_p_id[1]){
    case white_knight_id:p1_moves=m_atta.knight(m_p[1]);break;
    case white_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces);break;
    case white_rook_id:p1_moves=m_atta.rook(m_p[1],pieces);break;
    case white_queen_id:p1_moves=m_atta.queen(m_p[1],pieces);break;
    case black_knight_id:p1_moves=m_atta.knight(m_p[1]);break;
    case black_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces);break;
    case black_rook_id:p1_moves=m_atta.rook(m_p[1],pieces);break;
    case black_queen_id:p1_moves=m_atta.queen(m_p[1],pieces);break;
    default: throw logic_error("Egt4Pieces::set_initial_eval(unsigned key)  unknown piece2");
    }
  }
  bool check{false};
  if(m_bk & p0_moves){check=true;}
  if(m_p_id[1]<black_pawn_id && (m_bk & p1_moves)){check=true;}
  if(m_p_id[1]>=black_pawn_id && (m_wk & p1_moves)){check=true;}
  //--------------------------------------------------------------------
  
  vector<Move> ml;
  generate_moves(ml);
  if(check){
    for(auto& m : ml){
      make_move(m);
      if(is_legal()){return;}
      unmake_move(m);
    }
    m_pp[key]=m_stm==black?0x00:0x80;  // checkmate    
  }else{
    for(auto& m : ml){
      make_move(m);
      if(is_legal()){return;}
      unmake_move(m);
    }
    m_pp[key]=0xFD;  // stalemate
  }
  return;
}

bool Egt4Pieces::is_legal_key(unsigned key)const
{
  if((key>>19 & 0x3f)>10){return false;}
  int wk_square[10]={0,1,2,3,9,10,11,18,19,27};
  int sK=wk_square[key>>19 & 0x3f];
  int sk=key>>13 & 0x3f;
  int s2=key>>7 & 0x3f;
  int s1=key>>1 & 0x3f;
  if(s1==sK || s2==sK || sk==sK || (s1==s2 && s1!=sk)){
    return false;
  }else{
    return true;
  }
}

bool Egt4Pieces::is_legal()const
{
  if(m_wk.popcount()!=1 || m_bk.popcount()!=1){return false;}
  BitBoard pieces=m_wk | m_bk | m_p[0] | m_p[1];
  BitBoard p0_moves,p1_moves;
  if(m_p[0]){
    switch(m_p_id[0]){
    case white_knight_id:p0_moves=m_atta.knight(m_p[0]);break;
    case white_bishop_id:p0_moves=m_atta.bishop(m_p[0],pieces);break;
    case white_rook_id:p0_moves=m_atta.rook(m_p[0],pieces);break;
    case white_queen_id:p0_moves=m_atta.queen(m_p[0],pieces);break;
    default: throw logic_error("Egt4Pieces::is_legal() unkown piece 1");
    }
  }
  if(m_p[1]){
    switch(m_p_id[1]){
    case white_knight_id:p1_moves=m_atta.knight(m_p[1]);break;
    case black_knight_id:p1_moves=m_atta.knight(m_p[1]);break;
    case white_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces);break;
    case black_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces);break;
    case white_rook_id:p1_moves=m_atta.rook(m_p[1],pieces);break;
    case black_rook_id:p1_moves=m_atta.rook(m_p[1],pieces);break;
    case white_queen_id:p1_moves=m_atta.queen(m_p[1],pieces);break;
    case black_queen_id:p1_moves=m_atta.queen(m_p[1],pieces);break;
    default: throw logic_error("Egt4Pieces::is_legal() unkown piece 2");
    }
  }

  
  BitBoard bkmoves=m_atta.king(m_bk);
  if(m_wk & bkmoves){return false;}

  
  if(m_p_id[1]<black_pawn_id){
    if(m_stm==white){
      if(p0_moves & m_bk){return false;}
      if(p1_moves & m_bk){return false;}
    }
  }else{
    if(m_stm==white){
      if(p0_moves & m_bk){return false;}
    }else{
      if(p1_moves & m_wk){return false;}
    }
  }
  
  return true;
}

bool Egt4Pieces::is_consistent()
{
  bool ok{false};
  vector<Move> vm;
  for(unsigned i=0;i<size();i++){
    if(!is_legal_key(i)){continue;}
    set_key(i);
    if(!is_legal()){continue;}
    string ifen=fen();
    int ev=eval();
    
    // test for white winning: 
    if(ev>0 && ev<30000 && ev!=0xFE && ev!=0xFF){
      generate_moves(vm);
      if(m_stm==white){
	ok=false;
	for(auto& m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev+1){ok=true;}
	    if(eval()>ev+1){
	      std::cerr << "test1 for white winning" << endl;
	      std::cerr << "  sign=" << m_sign << endl;
	      std::cerr << "new eval > old eval + 1" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto& m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }else{
	ok=false;
	for(auto& m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev+1){ok=true;}
	    if(eval()<=ev){
	      std::cerr << "test2 for white winning" << endl;
	      std::cerr << "  sign=" << m_sign << endl;
	      std::cerr << "new eval < old eval" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto& m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }
      if(!ok){break;}
    }
    
    // test for black winning: 
    if(ev<0 && ev>-30000 && ev!=0xFE && ev!=0xFF){
      generate_moves(vm);
      if(m_stm==black){
	ok=false;
	for(auto& m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev-1){ok=true;}
	    if(eval()<ev-1){
	      std::cerr << "test1 for black winning" << endl;
	      std::cerr << "  sign=" << m_sign << endl;
	      std::cerr << "new eval > old eval + 1" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto& m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }else{
	ok=false;
	for(auto& m : vm){
	  make_move(m);
	  if(is_legal()){
	    if(eval()==ev-1){ok=true;}
	    if(eval()>=ev){
	      std::cerr << "test2 for black winning" << endl;
	      std::cerr << "  sign=" << m_sign << endl;
	      std::cerr << "new eval < old eval" << endl;
	      std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	      std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	      for(auto& m : vm){std::cerr << m.algebraic()<< (m.capture()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	      ok=false;
	      break;
	    }
	  }
	  unmake_move(m);
	}
      }
      if(!ok){break;}
    }

    // test for draw: 
    if(ev==0){
      generate_moves(vm);
      ok=true;
      for(auto& m : vm){
	make_move(m);
	if(is_legal()){
	  if(m_stm==black && eval()>ev){
	    std::cerr << "Test for draw:" << endl;
	    std::cerr << "new eval > old eval" << endl;
	    std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	    std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	    for(auto& m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	    ok=false;
	    break;
	  }else if(m_stm==white && eval()<ev){
	    std::cerr << "Test for draw:" << endl;
	    std::cerr << "new eval < old eval" << endl;
	    std::cerr << "orig fen: " << ifen <<  "  eval()=" << ev << "   key=" << i << endl;
	    std::cerr << "next fen: " << fen() << "  eval()=" << eval() << "   key=" << get_key() << endl;
	    for(auto& m : vm){std::cerr << m.algebraic()<< (!m.capture().no_piece()?"("+m.capture().name()+")":"") << " ";}std::cerr << endl;
	    ok=false;
	    break;
	  }
	}
	unmake_move(m);
      }
      if(!ok){
	//throw logic_error("Egt4Pieces::is_consistent():  Draw not ok");
      } 
    }
  }

  if(ok){return true;}else{return false;}
}

void Egt4Pieces::make_move(Move& m)
{
  m_make_move_counter++;
  BitBoard bb_move{m.from().bitboard() | m.to().bitboard()};
  if(m.piece_id()==m_p_id[0]){
    m_p[0]^=bb_move;
  }else if(m.piece_id()==m_p_id[1]){
    m_p[1]^=bb_move;
  }else if(m.piece_id()==white_king_id){
    m_wk^=bb_move;
  }else{
    m_bk^=bb_move;
  }

  if(m.capture_id()==m_p_id[0]){
    m_p[0].clear();
  }else if(m.capture_id()==m_p_id[1]){
    m_p[1].clear();
  }

  m_stm=m_stm==white?black:white;
}

void Egt4Pieces::unmake_move(Move& m)
{
  BitBoard bb_move{m.from().bitboard() | m.to().bitboard()};
  if(m.piece_id()==m_p_id[0]){
    m_p[0]^=bb_move;
  }else if(m.piece_id()==m_p_id[1]){
    m_p[1]^=bb_move;
  }else if(m.piece_id()==white_king_id){
    m_wk^=bb_move;
  }else{
    m_bk^=bb_move;
  }

  if(m.capture_id()==m_p_id[0]){
    m_p[0]=m.to().bitboard();
  }else if(m.capture_id()==m_p_id[1]){
    m_p[1]=m.to().bitboard();
  }

  m_stm=m_stm==white?black:white;
}
 
void Egt4Pieces::generate_moves(vector<Move>& ml)
{
  ml.clear();
  BitBoard wpieces=m_wk;
  BitBoard bpieces=m_bk;
  if(m_p_id[0]<=white_king_id){wpieces|=m_p[0];}else{bpieces|=m_p[0];}
  if(m_p_id[1]<=white_king_id){wpieces|=m_p[1];}else{bpieces|=m_p[1];}

  BitBoard pieces=wpieces | bpieces;
  BitBoard wkmoves=m_atta.king(m_wk) & ~wpieces;
  BitBoard bkmoves=m_atta.king(m_bk) & ~bpieces;
  BitBoard p0_moves,p1_moves;
  if(m_p[0]){
    switch(m_p_id[0]){
    case no_piece_id:throw logic_error("Egt4Pieces::generate_moves   no_piece_id(1) error");
    case white_pawn_id:throw logic_error("Egt4Pieces::generate_moves   pawns(1) not supported yet");
    case white_knight_id:p0_moves=m_atta.knight(m_p[0])&~wpieces;break;
    case white_bishop_id:p0_moves=m_atta.bishop(m_p[0],pieces)&~wpieces;break;
    case white_rook_id:p0_moves=m_atta.rook(m_p[0],pieces)&~wpieces;break;
    case white_queen_id:p0_moves=m_atta.queen(m_p[0],pieces)&~wpieces;break;
    case white_king_id:throw logic_error("Egt4Pieces::generate_moves   piece1==white_king");
    case black_pawn_id:throw logic_error("Egt4Pieces::generate_moves   pawns(2) not supported yet");
    case black_knight_id:p0_moves=m_atta.knight(m_p[0])&~bpieces;break;
    case black_bishop_id:p0_moves=m_atta.bishop(m_p[0],pieces)&~bpieces;break;
    case black_rook_id:p0_moves=m_atta.rook(m_p[0],pieces)&~bpieces;break;
    case black_queen_id:p0_moves=m_atta.queen(m_p[0],pieces)&~bpieces;break;
    case black_king_id:throw logic_error("Egt4Pieces::generate_moves   piece1==black_king");
    }
  }
  if(m_p[1]){
    switch(m_p_id[1]){
    case no_piece_id:throw logic_error("Egt4Pieces::generate_moves   no_piece_id(2) error");
    case white_pawn_id:throw logic_error("Egt4Pieces::generate_moves   pawns(3) not supported yet");
    case white_knight_id:p1_moves=m_atta.knight(m_p[1])&~wpieces;break;
    case white_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces)&~wpieces;break;
    case white_rook_id:p1_moves=m_atta.rook(m_p[1],pieces)&~wpieces;break;
    case white_queen_id:p1_moves=m_atta.queen(m_p[1],pieces)&~wpieces;break;
    case white_king_id:throw logic_error("Egt4Pieces::generate_moves   piece2==white_king");
    case black_pawn_id:throw logic_error("Egt4Pieces::generate_moves   pawns(4) not supported yet");
    case black_knight_id:p1_moves=m_atta.knight(m_p[1])&~bpieces;break;
    case black_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces)&~bpieces;break;
    case black_rook_id:p1_moves=m_atta.rook(m_p[1],pieces)&~bpieces;break;
    case black_queen_id:p1_moves=m_atta.queen(m_p[1],pieces)&~bpieces;break;
    case black_king_id:throw logic_error("Egt4Pieces::generate_moves   piece2==black_king");
    }
  }

  
  if(m_stm==white){
    if(m_p_id[0]<=white_king_id && m_p[0]){
      Square p1_square=m_p[0].get_unique_square();
      //p0_moves&=~wpieces;
      while(p0_moves){
	Square to=p0_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(p1_square,to,normal_move,m_p_id[0],no_piece_id);
	if(to.bitboard() & m_p[1]){egtmove.capture(m_p_id[1]);}
	ml.push_back(egtmove);
      }
    }
    if(m_p_id[1]<=white_king_id && m_p[1]){
      Square p2_square=m_p[1].get_unique_square();
      //p1_moves&=~wpieces;
      while(p1_moves){
	Square to=p1_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(p2_square,to,normal_move,m_p_id[1],no_piece_id);
	ml.push_back(egtmove);
      }
    }
    Square kingsquare=m_wk.get_unique_square();
    while(wkmoves){
      Square to=wkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(kingsquare,to,normal_move,white_king_id,no_piece_id);
      if(to.bitboard() & m_p[1]){egtmove.capture(m_p_id[1]);}
      ml.push_back(egtmove);
    }
  }else{
    if(m_p_id[0]>=black_pawn_id && m_p[0]){
      Square p1_square=m_p[0].get_unique_square();
      //p0_moves&=~(bpieces | m_wk);
      while(p0_moves){
	Square to=p0_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(p1_square,to,normal_move,m_p_id[0],no_piece_id);
	if(to.bitboard() & m_p[1]){egtmove.capture(m_p_id[1]);}
	ml.push_back(egtmove);
      }
    }
    if(m_p_id[1]>=black_pawn_id && m_p[1]){
      Square p2_square=m_p[1].get_unique_square();
      //p1_moves&=~(bpieces | m_wk);
      while(p1_moves){
	Square to=p1_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(p2_square,to,normal_move,m_p_id[1],no_piece_id);
	if(to.bitboard() & m_p[0]){egtmove.capture(m_p_id[0]);}
	ml.push_back(egtmove);
      }
    }
    Square kingsquare=m_bk.get_unique_square();
    while(bkmoves){
      Square to=bkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(kingsquare,to,normal_move,black_king_id,no_piece_id);
      if(to.bitboard() & m_p[0]){egtmove.capture(m_p_id[0]);}
      if(to.bitboard() & m_p[1]){egtmove.capture(m_p_id[1]);}
      ml.push_back(egtmove);
    }
  }
}


void Egt4Pieces::generate_previous_moves(vector<Move>& ml)
{
  ml.clear();
  BitBoard pieces=m_wk | m_bk | m_p[0] | m_p[1];
  BitBoard wkmoves=m_atta.king(m_wk);
  BitBoard bkmoves=m_atta.king(m_bk);
  BitBoard p0_moves,p1_moves;
  if(m_p[0]){
    switch(m_p_id[0]){
    case no_piece_id:throw logic_error("Egt4Pieces::generate_previous_moves   no_piece_id(1) not supported yet");
    case white_pawn_id:throw logic_error("Egt4Pieces::generate_previous_moves   pawns(1) not supported yet");
    case white_knight_id:p0_moves=m_atta.knight(m_p[0]);break;
    case white_bishop_id:p0_moves=m_atta.bishop(m_p[0],pieces);break;
    case white_rook_id:p0_moves=m_atta.rook(m_p[0],pieces);break;
    case white_queen_id:p0_moves=m_atta.queen(m_p[0],pieces);break;
    case white_king_id:throw logic_error("Egt4Pieces::generate_previous_moves   piece1==white_king");
    case black_pawn_id:throw logic_error("Egt4Pieces::generate_previous_moves   pawns(2) not supported yet");
    case black_knight_id:p0_moves=m_atta.knight(m_p[0]);break;
    case black_bishop_id:p0_moves=m_atta.bishop(m_p[0],pieces);break;
    case black_rook_id:p0_moves=m_atta.rook(m_p[0],pieces);break;
    case black_queen_id:p0_moves=m_atta.queen(m_p[0],pieces);break;
    case black_king_id:throw logic_error("Egt4Pieces::generate_previous_moves   piece1==black_king");
    }
  }
  if(m_p[1]){
    switch(m_p_id[1]){
    case no_piece_id:throw logic_error("Egt4Pieces::generate_previous_moves   no_piece_id(2) not supported yet");
    case white_pawn_id:throw logic_error("Egt4Pieces::generate_previous_moves   pawns(3) not supported yet");
    case white_knight_id:p1_moves=m_atta.knight(m_p[1]);break;
    case white_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces);break;
    case white_rook_id:p1_moves=m_atta.rook(m_p[1],pieces);break;
    case white_queen_id:p1_moves=m_atta.queen(m_p[1],pieces);break;
    case white_king_id:throw logic_error("Egt4Pieces::generate_previous_moves   piece2==white_king");
    case black_pawn_id:throw logic_error("Egt4Pieces::generate_previous_moves   pawns(4) not supported yet");
    case black_knight_id:p1_moves=m_atta.knight(m_p[1]);break;
    case black_bishop_id:p1_moves=m_atta.bishop(m_p[1],pieces);break;
    case black_rook_id:p1_moves=m_atta.rook(m_p[1],pieces);break;
    case black_queen_id:p1_moves=m_atta.queen(m_p[1],pieces);break;
    case black_king_id:throw logic_error("Egt4Pieces::generate_previous_moves   piece2==black_king");
    }
  }

  //----------- never create a piece: ----------
  p0_moves&=~pieces;
  p1_moves&=~pieces;
  wkmoves&=~pieces;
  bkmoves&=~pieces;
  //-------------------------------------------------------
  
  if(m_stm==black){
    Square kingsquare=m_wk.get_unique_square();
    while(wkmoves){
      Square s=wkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(s,kingsquare,normal_move,WhiteKing(),NoPiece());
      ml.push_back(egtmove);
      if(!m_p[1] && m_p_id[1]>=black_pawn_id){
	egtmove.capture(m_p_id[1]);
	ml.push_back(egtmove);
      }
    }
    if(m_p[0]){
      Square p1_square=m_p[0].get_unique_square();
      while(p0_moves){
	Square s=p0_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(s,p1_square,normal_move,m_p_id[0],no_piece_id);
	ml.push_back(egtmove);
	if(!m_p[1] && m_p_id[1]>=black_pawn_id){
	  egtmove.capture(m_p_id[1]);
	  ml.push_back(egtmove);
	}
      }
    }
    if(m_p_id[1] < black_pawn_id && m_p[1]){
      Square p2_square=m_p[1].get_unique_square();
      while(p1_moves){
	Square s=p1_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(s,p2_square,normal_move,m_p_id[1],no_piece_id);
	ml.push_back(egtmove);
      }
    }
  }else{
    Square kingsquare=m_bk.get_unique_square();
    while(bkmoves){
      Square s=bkmoves.get_reset_square().get_unique_square();
      Move egtmove=Move(s,kingsquare,normal_move,BlackKing(),NoPiece());
      ml.push_back(egtmove);
      if(!m_p[0]){
	egtmove.capture(m_p_id[0]);
	ml.push_back(egtmove);
      }
      if(!m_p[1] && m_p_id[1]<black_pawn_id){
	egtmove.capture(m_p_id[1]);
	ml.push_back(egtmove);
      }
    }
    if(m_p_id[1]>=black_pawn_id && m_p[1]){
      Square p2_square=m_p[1].get_unique_square();
      while(p1_moves){
	Square s=p1_moves.get_reset_square().get_unique_square();
	Move egtmove=Move(s,p2_square,normal_move,m_p_id[1],no_piece_id);
	ml.push_back(egtmove);
	if(!m_p[0]){
	  egtmove.capture(m_p_id[0]);
	  ml.push_back(egtmove);
	}
      }
    }
  }
}

string Egt4Pieces::stats()const
{
  array<int,256> frequency;
  for(auto& i : frequency){i=0;}
  for(unsigned i=0;i<m_size;i++){
    frequency[static_cast<unsigned int>(m_pp[i])]++;
  }

  ostringstream oss;
  for(int i=0;i<256;i++){
    if(i%8==0){oss << endl;}
    oss << setw(7) << frequency[i] << "  ";
  }

  return oss.str();
}

void Egt4Pieces::save(string fn)const
{
  ofstream ofs(fn);
  if(!ofs.good()){throw runtime_error("Egt4Pieces::save  error");}  
  ofs << "begin{"+m_egt_name+"}";
  for(unsigned i=0;i<m_size;i++){
    if(i%16==0){ofs << endl;}
    ofs << (int)m_pp[i] << " ";
  }
  ofs << endl << "end{"+m_egt_name+"}";
}

void Egt4Pieces::load(string fn)
{
  string s;
  int dtm;
  ifstream ifs(fn);
  ifs >> s;
  if(s.find("begin")==string::npos){throw runtime_error("Error: EGT not recognized");}
  if(s.find("KQKR")!=string::npos){
    m_p_id[0]=white_queen_id;
    m_p_id[1]=black_rook_id;
    m_egt_name="KQKR";
  }else if(s.find("KBNK")!=string::npos){
    m_p_id[0]=black_knight_id;
    m_p_id[1]=white_bishop_id;
    m_egt_name="KBNK";
  }
  if(!m_pp && m_memory_owner){
    m_pp=new unsigned char[m_size];
  }
  for(unsigned i=0;i<m_size;i++){
    ifs >> dtm;
    m_pp[i]=dtm;
    if(!ifs.good()){
      ostringstream oss;
      oss << "Egt4Pieces::load(" << fn << ") failed    i=" << i << endl;
      throw runtime_error(oss.str());
    }
  }
  ifs >> s;
  if(s[0]!='e' && s[1]!='n' && s[2]!='d'){throw runtime_error("Error while reading EGT");}

  m_depth=101;
}

