/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef EGTKBNK_H
#define EGTKBNK_H

#include"../common/attacks.h"
#include"../search/timer.h"
#include<string>
using std::string;


//--------------------------------------------------------------
// unsigned char eval:
//
// first bit == 0: white wins
// last seven bits 0...0x64: win in ... plies (distance to mate)
// last eight bits == 0xFF: position not yet evaluated
// last eight bits == 0xFE: illegal position
// last eight bits == 0xFD: draw
//--------------------------------------------------------------




class Egt4Pieces
{
 private:
  unsigned char* m_pp{nullptr};
  const Attacks& m_atta;
  bool m_memory_owner{true};
  unsigned m_size{5242880};          // 10 * 64 * 64 * 64 * 2
  int m_stm;
  BitBoard m_wk,m_bk;
  BitBoard m_p[4];
  piece_t m_p_id[4];
  int m_sign{1};
  int m_depth{-1};
  string m_egt_name;
  uint64_t m_make_move_counter{0};
  Timer m_tim;
  bool m_log{false};
  unsigned m_debug_key{0};
  
  
public:
  Egt4Pieces(const Attacks&,piece_t,piece_t);
  ~Egt4Pieces();
  bool operator==(const Egt4Pieces&)const;
  bool operator!=(const Egt4Pieces&)const;
  bool busy()const;
  bool ready()const;
  unsigned operator[](unsigned)const;
  void set_memory(unsigned char*);
  int stm()const;
  unsigned size()const;
  void fen(string);
  string fen()const;
  void set_key(unsigned);
  unsigned get_key()const;
  unsigned get_mirror_key(unsigned)const;
  string get_key_str()const;
  string get_key_str(unsigned)const;
  string mate_in(int)const;
  int diff(const Egt4Pieces& x)const;
  int eval(unsigned)const;
  int eval(string);
  int eval()const;
  string stats()const;
  
  bool create();
  void create_next();
  void clear();

  void generate_moves(vector<Move>&);
  void generate_previous_moves(vector<Move>&);
  void set_initial_eval();
  void set_initial_eval(unsigned);
  bool is_legal()const;
  bool is_legal_key(unsigned)const;
  bool is_consistent();
  void make_move(Move&);
  void unmake_move(Move&);

  void save(string)const;
  void load(string);
};


#endif
