/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"materialindex.h"
#include<iostream>
#include<sstream>
using std::endl;
using std::ostringstream;

 
bool MaterialIndex::operator<(const MaterialIndex& x)const
{
  return mi<x.mi;
}

bool MaterialIndex::operator==(const MaterialIndex& x)const
{
  return mi==x.mi;
}

bool MaterialIndex::operator!=(const MaterialIndex& x)const
{
  return mi!=x.mi;
}

void MaterialIndex::set_whitequeen(int x)
{
  mi|=x*inc_table[white_queen_id];
}

void MaterialIndex::set_whiterook(int x)
{
  mi|=x*inc_table[white_rook_id];
}

void MaterialIndex::set_whitebishop(int x)
{
  mi|=x*inc_table[white_bishop_id];
}

void MaterialIndex::set_whiteknight(int x)
{
  mi|=x*inc_table[white_knight_id];
}

void MaterialIndex::set_whitepawn(int x)
{
  mi|=x*inc_table[white_pawn_id];
}

void MaterialIndex::set_blackqueen(int x)
{
  mi|=x*inc_table[black_queen_id];
}

void MaterialIndex::set_blackrook(int x)
{
  mi|=x*inc_table[black_rook_id];
}

void MaterialIndex::set_blackbishop(int x)
{
  mi|=x*inc_table[black_bishop_id];
}

void MaterialIndex::set_blackknight(int x)
{
  mi|=x*inc_table[black_knight_id];
}

void MaterialIndex::set_blackpawn(int x)
{
  mi|=x*inc_table[black_pawn_id];
}

void MaterialIndex::add_piece(int piece_id)
{
  mi+=inc_table[piece_id];
}

void MaterialIndex::remove_piece(int piece_id)
{
  mi-=inc_table[piece_id];
}

void MaterialIndex::set_index(const uint64_t& x)
{
  mi=x;
}

int MaterialIndex::get_whitequeen()const
{
  return mi/inc_table[white_queen_id] & 0xF;
}

int MaterialIndex::get_whiterook()const
{
  return mi/inc_table[white_rook_id] & 0xF;
}

int MaterialIndex::get_whitebishop()const
{
  return mi/inc_table[white_bishop_id] & 0xF;
}

int MaterialIndex::get_whiteknight()const
{
  return mi/inc_table[white_knight_id] & 0xF;
}

int MaterialIndex::get_whitepawn()const
{
  return mi/inc_table[white_pawn_id] & 0xF;
}

int MaterialIndex::get_blackqueen()const
{
  return mi/inc_table[black_queen_id] & 0xF;
}

int MaterialIndex::get_blackrook()const
{
  return mi/inc_table[black_rook_id] & 0xF;
}

int MaterialIndex::get_blackbishop()const
{
  return mi/inc_table[black_bishop_id] & 0xF;
}

int MaterialIndex::get_blackknight()const
{
  return mi/inc_table[black_knight_id] & 0xF;
}

int MaterialIndex::get_blackpawn()const
{
  return mi/inc_table[black_pawn_id] & 0xF;
}

bool MaterialIndex::equal()const
{
  return mi>>32 == (mi & 0xffffffff);
}

bool MaterialIndex::white_wins()const
{
  return mi>>32 > (mi & 0xffffffff);
}

bool MaterialIndex::black_wins()const
{
  return mi>>32 < (mi & 0xffffffff);
}

void MaterialIndex::flip_colors()
{
  mi = mi>>32 | (mi & 0xffffffff)<<32;
}

string MaterialIndex::str()const
{
  string r;
  r="K";
  for(int i=0;i<get_whitequeen();i++){r+="Q";}
  for(int i=0;i<get_whiterook();i++){r+="R";}
  for(int i=0;i<get_whitebishop();i++){r+="B";}
  for(int i=0;i<get_whiteknight();i++){r+="N";}
  for(int i=0;i<get_whitepawn();i++){r+="P";}
  r+="k";
  for(int i=0;i<get_blackqueen();i++){r+="q";}
  for(int i=0;i<get_blackrook();i++){r+="r";}
  for(int i=0;i<get_blackbishop();i++){r+="b";}
  for(int i=0;i<get_blackknight();i++){r+="n";}
  for(int i=0;i<get_blackpawn();i++){r+="p";}
  return r;
}

const uint64_t& MaterialIndex::index()const
{
  return mi;
}

void MaterialIndex::make_move(const Move& m)
{
  remove_piece(m.capture_id());
  switch(m.code()){
  case normal_move:break;
  case promo_queen:if(m.piece_id()==white_pawn_id){
      remove_piece(white_pawn_id);
      add_piece(white_queen_id);
    }else{
      remove_piece(black_pawn_id);
      add_piece(black_queen_id);
    }
    break;
  case promo_rook:if(m.piece_id()==white_pawn_id){
      remove_piece(white_pawn_id);
      add_piece(white_rook_id);
    }else{
      remove_piece(black_pawn_id);
      add_piece(black_rook_id);
    }
    break;
  case promo_bishop:if(m.piece_id()==white_pawn_id){
      remove_piece(white_pawn_id);
      add_piece(white_bishop_id);
    }else{
      remove_piece(black_pawn_id);
      add_piece(black_bishop_id);
    }
    break;
  case promo_knight:if(m.piece_id()==white_pawn_id){
      remove_piece(white_pawn_id);
      add_piece(white_knight_id);
    }else{
      remove_piece(black_pawn_id);
      add_piece(black_knight_id);
    }
    break;
  case castling:break;
  case en_passant:break;
  case nullmove: break;
  }
}

void MaterialIndex::unmake_move(const Move& m)
{
  add_piece(m.capture_id());
  switch(m.code()){
  case normal_move:break;
  case promo_queen:if(m.piece_id()==white_pawn_id){
      add_piece(white_pawn_id);
      remove_piece(white_queen_id);
    }else{
      add_piece(black_pawn_id);
      remove_piece(black_queen_id);
    }
    break;
  case promo_rook:if(m.piece_id()==white_pawn_id){
      add_piece(white_pawn_id);
      remove_piece(white_rook_id);
    }else{
      add_piece(black_pawn_id);
      remove_piece(black_rook_id);
    }
    break;
  case promo_bishop:if(m.piece_id()==white_pawn_id){
      add_piece(white_pawn_id);
      remove_piece(white_bishop_id);
    }else{
      add_piece(black_pawn_id);
      remove_piece(black_bishop_id);
    }
    break;
  case promo_knight:if(m.piece_id()==white_pawn_id){
      add_piece(white_pawn_id);
      remove_piece(white_knight_id);
    }else{
      add_piece(black_pawn_id);
      remove_piece(black_knight_id);
    }
    break;
  case castling:break;
  case en_passant:break;
  case nullmove: break;
  }
}

int MaterialIndex::value()const
{
  int wp=get_whitepawn();
  int bp=get_blackpawn();
  int wn=get_whiteknight();
  int bn=get_blackknight();
  int wb=get_whitebishop();
  int bb=get_blackbishop();
  int wr=get_whiterook();
  int br=get_blackrook();
  int wq=get_whitequeen();
  int bq=get_blackqueen();
  int wminor=wn+wb;
  int bminor=bn+bb;
  int wmajor=wq+wr;
  int bmajor=bq+br;
  
  //------------- knight and rook values: --------------
  int knightbonus[9]{-20,-15,-10,-5,0,0,5,10,15};
  int rookbonus[9]{0,0,0,0,0,-5,-10,-20,-30};
  int score=knightbonus[wp]+rookbonus[wp]-knightbonus[bp]-rookbonus[bp];
  
  //------------- redundant rook penalty: --------------
  if(wq==bq){
    if(wr>1){score-=25;}
    if(br>1){score+=25;}
  }

  //------------- exchange: ----------------------------
  if(wr!=br && wq==bq && wminor+wmajor == bminor+bmajor){
    if(wr>br){
      score-=25.0*(wmajor-1);
    }else{
      score+=25.0*(bmajor-1);
    }
  }
  
  //------------- pair of bishops: ---------------------
  if(wb==2){score+=50;}
  if(bb==2){score-=50;}

  //------------- pieces: ------------------------------
  // white:
  score+=wp*100;
  if(wminor+wmajor > 1){
    score+=(wn+wb)*325;
  }else{
    score+=(wn+wb)*250;
  }
  score+= wr * 525;
  score+= wq * 1000;
  
  // black: 
  score-=bp*100;
  if(bminor+bmajor > 1){
    score-=(bn+bb)*325;
  }else{
    score-=(bn+bb)*250;
  }
  score-= br * 525;
  score-= bq * 1000;

  //------------- sufficient mating material? ----------
  if(wp==0 && bp==0){
    if(score>-400 && score<400){
      score/=10;
    }
    if(wminor==0 && bminor==0 && wmajor==bmajor){
      if(wq==1 && bq==0 && br==1){
	score=450;
      }else if(wq==0 && bq==1 && wr==1){
	score=-450;
      }
    }
  }else if(wp==0 && wmajor==0 && wminor==1 && bmajor==0 && bminor==0 && bp<3){
    score=-50;
  }else if(bp==0 && bmajor==0 && bminor==1 && wmajor==0 && wminor==0 && wp<3){
    score=50;
  }else if(wp==1 && wmajor==0 && wminor==1 && bmajor==1 && bminor==0 && br==1 && bp==0){
    score=50;
  }else if(bp==1 && bmajor==0 && bminor==1 && wmajor==1 && wminor==0 && wr==1 && wp==0){
    score=-50;
  }


  //-------------- make progress: ----------------------
  if(score>70 && wp>0){
    score+=wp*10;
    score-=(bminor+bmajor)*10;
  }else if(score<-70 && bp>0){
    score-=bp*10;
    score+=(wminor+wmajor)*10;
  }
  
  return score;
}
