/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MOBILITY_H
#define MOBILITY_H

#include"../board/cposition.h"
#include"../common/attacks.h"


class Mobility
{
 private:
  const Attacks& m_atta;

  array<int,9> knight_mobility{{-68,-28,-12,0,12,20,28,36,44}};
  array<int,14> bishop_mobility{{-64,-48,-36,-24,-16,-8,-4,0,8,16,24,32,36,40}};
  array<int,15> rook_mobility{{-40,-20,-12,-6,0,4,8,12,16,20,24,26,30,32,34}};
  array<int,28> queen_mobility{{-22,-12,-8,-5,-2,0,4,6,8,10,11,13,14,15,15,17,18,19,20,22,23,24,25,26,27,28,29,30}};
  int m_safe_mobility{0};
  int m_safe_knight_mobility{0};
  int m_safe_bishop_mobility{0};
  int m_safe_rook_mobility{0};
  int m_safe_queen_mobility{0};
  int m_safe_queen_check{0};

  BitBoard m_white_knight_battlefield_attacks;
  BitBoard m_white_bishop_battlefield_attacks;
  BitBoard m_white_rook_battlefield_attacks;
  BitBoard m_white_queen_battlefield_attacks;
  BitBoard m_black_knight_battlefield_attacks;
  BitBoard m_black_bishop_battlefield_attacks;
  BitBoard m_black_rook_battlefield_attacks;
  BitBoard m_black_queen_battlefield_attacks;
  
  BitBoard m_white_king_neighbourhood;
  BitBoard m_black_king_neighbourhood;
  BitBoard m_white_king_battlefield;
  BitBoard m_black_king_battlefield;

  BitBoard m_white_pawn_attacks;
  BitBoard m_white_knight_attacks;
  BitBoard m_white_bishop_attacks;
  BitBoard m_white_rook_attacks;
  BitBoard m_white_queen_attacks;
  BitBoard m_white_safe_queen_attacks;
  BitBoard m_white_safe_queen_checks;
  BitBoard m_white_king_attacks;
  BitBoard m_black_pawn_attacks;
  BitBoard m_black_knight_attacks;
  BitBoard m_black_bishop_attacks;
  BitBoard m_black_rook_attacks;
  BitBoard m_black_queen_attacks;
  BitBoard m_black_safe_queen_attacks;
  BitBoard m_black_safe_queen_checks;
  BitBoard m_black_king_attacks;

  BitBoard m_white_attacks;
  BitBoard m_black_attacks;

  void compute_white_pawn_attacks(const ChessPosition&);
  void compute_white_knight_attacks(const ChessPosition&);
  void compute_white_bishop_attacks(const ChessPosition&);
  void compute_white_rook_attacks(const ChessPosition&);
  void compute_white_queen_attacks(const ChessPosition&);
  void compute_white_safe_queen_checks(const ChessPosition&);
  void compute_white_king_attacks(const ChessPosition&);
  void compute_black_pawn_attacks(const ChessPosition&);
  void compute_black_knight_attacks(const ChessPosition&);
  void compute_black_bishop_attacks(const ChessPosition&);
  void compute_black_rook_attacks(const ChessPosition&);
  void compute_black_queen_attacks(const ChessPosition&);
  void compute_black_safe_queen_checks(const ChessPosition&);
  void compute_black_king_attacks(const ChessPosition&);

  //--------------- king safety: -------------------------------------------------------------
  array<int,16> m_attackweight{{0,0,128,192,225,241,248,253,256,256,256,256,256,256,256,256}};
  void compute_white_king_neighbourhood(const ChessPosition&);
  void compute_black_king_neighbourhood(const ChessPosition&);
  void compute_white_king_battlefield(const ChessPosition&);
  void compute_black_king_battlefield(const ChessPosition&);
  int m_attacked_squares_near_white_king{0};
  int m_attacked_squares_near_black_king{0};

 public:
  Mobility(const Attacks&);
  void compute(const ChessPosition&);
  
  int safe_mobility() const;
  int safe_knight_mobility() const;
  int safe_bishop_mobility() const;
  int safe_rook_mobility() const;
  int safe_queen_mobility() const;
  int safe_queen_checks() const;

  const BitBoard& white_attacks()const;
  const BitBoard& black_attacks()const;
  int undefended_pieces(const ChessPosition&)const;
  int hanging_pieces(const ChessPosition&)const;

  //--------------- king safety: -------------------------
  const BitBoard& white_king_neighbourhood()const;
  const BitBoard& black_king_neighbourhood()const;
  const BitBoard& white_king_battlefield()const;
  const BitBoard& black_king_battlefield()const;
  const BitBoard& white_knight_battlefield_attack()const;
  const BitBoard& white_bishop_battlefield_attack()const;
  const BitBoard& white_rook_battlefield_attack()const;
  const BitBoard& white_queen_battlefield_attack()const;
  const BitBoard& black_knight_battlefield_attack()const;
  const BitBoard& black_bishop_battlefield_attack()const;
  const BitBoard& black_rook_battlefield_attack()const;
  const BitBoard& black_queen_battlefield_attack()const;
  int attackweight_white_king()const;
  int attacks_on_white_king()const;
  int attackweight_black_king()const;
  int attacks_on_black_king()const;
  int king_safety()const;
};


#endif
