/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef PST_H
#define PST_H

#include"../movegen/move.h"
#include<array>
using std::array;




class ChessPosition;

class PieceSquareTables
{
 public:
  PieceSquareTables();
  void init(const ChessPosition&);
  bool verify_score()const;
  void make_move(const Move&);
  void unmake_move(const Move&);
  template<move_t> void promote(const Move&);
  template<move_t> void unpromote(const Move&);
  pair<float,float> get_score()const;
  pair<float,float> get_pawn_score()const;
  pair<float,float> get_knight_score()const;
  pair<float,float> get_bishop_score()const;
  pair<float,float> get_rook_score()const;
  pair<float,float> get_queen_score()const;
  pair<float,float> get_king_score()const;

 private:
  const ChessPosition* pcp;
  array<pair<float,float>,15> m_score;
  array<array<float,64>,15> pst_mg,pst_eg;
  

  const float pawn_mg[64]
  {
    // A1                                                                               H1
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, 
          0,         0,         0,   -1.3915,   -1.3915,         0,         0,         0, 
          0,         0,         0,         0,         0,         0,         0,         0, 
          1,         0,   15.3069,   18.0900,   18.0900,   15.3069,         0,         1, 
          2,    1.3915,    4.1746,    6.9577,    6.9577,    4.1746,    1.3915,         2, 
          4,   -1.3915,    1.3915,    1.3915,    1.3915,    1.3915,   -1.3915,         4, 
          0,         0,         0,         0,         0,         0,         0,         0, 
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000
    // A8                                                                               H8
  };

  const float pawn_eg[64]
  {
    // A1                                                                               H1
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000, 
    -27.7700,  -27.7700,  -27.7700,  -27.7700,  -27.7700,  -27.7700,  -27.7700,  -27.7700, 
    -16.6620,  -16.6620,  -16.6620,  -16.6620,  -16.6620,  -16.6620,  -16.6620,  -16.6620, 
    -5.5540,   -5.5540,   -5.5540,   -5.5540,   -5.5540,   -5.5540,   -5.5540,   -5.5540, 
    5.5540,    5.5540,    5.5540,    5.5540,    5.5540,    5.5540,    5.5540,    5.5540, 
    16.6620,   16.6620,   16.6620,   16.6620,   16.6620,   16.6620,   16.6620,   16.6620, 
    27.7700,   27.7700,   27.7700,   27.7700,   27.7700,   27.7700,   27.7700,   27.7700, 
    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000,    0.0000 
    // A8                                                                               H8
  };

  const float knight_mg[64]
  {
    // A1                                                                               H1
    -13.5461,  -11.2156,   -8.8850,   -6.5545,   -6.5545,   -8.8850,  -11.2156,  -13.5461, 
    -11.2156,   -8.8850,   -0.7283,    1.6022,    1.6022,   -0.7283,   -8.8850,  -11.2156, 
    -7.7198,    1.6022,    9.7590,    8.5937,    8.5937,    9.7590,    1.6022,   -7.7198, 
    -4.2240,    5.0980,   10.9242,    9.7590,    9.7590,   10.9242,    5.0980,   -4.2240, 
    -4.2240,    5.0980,   10.9242,    9.7590,    9.7590,   10.9242,    5.0980,   -4.2240, 
    -5.3893,    3.9327,   14.4200,   13.2547,   13.2547,   14.4200,    3.9327,   -5.3893, 
    -7.7198,   -4.2240,    6.2632,    8.5937,    8.5937,    6.2632,   -4.2240,   -7.7198, 
    -11.2156,   -7.7198,   -4.2240,   -1.8935,   -1.8935,   -4.2240,   -7.7198,  -11.2156 
    // A8                                                                               H8
  };

  const float knight_eg[64]
  {
    // A1                                                                               H1
    -0.0658,   -0.0544,   -0.0431,   -0.0318,   -0.0318,   -0.0431,   -0.0544,   -0.0658, 
    -0.0544,   -0.0431,   -0.0035,    0.0078,    0.0078,   -0.0035,   -0.0431,   -0.0544, 
    -0.0375,    0.0078,    0.0474,    0.0417,    0.0417,    0.0474,    0.0078,   -0.0375, 
    -0.0205,    0.0247,    0.0530,    0.0474,    0.0474,    0.0530,    0.0247,   -0.0205, 
    -0.0205,    0.0247,    0.0530,    0.0474,    0.0474,    0.0530,    0.0247,   -0.0205, 
    -0.0262,    0.0191,    0.0700,    0.0643,    0.0643,    0.0700,    0.0191,   -0.0262, 
    -0.0375,   -0.0205,    0.0304,    0.0417,    0.0417,    0.0304,   -0.0205,   -0.0375, 
    -0.0544,   -0.0375,   -0.0205,   -0.0092,   -0.0092,   -0.0205,   -0.0375,   -0.0544 
    // A8                                                                               H8
  };

  const float bishop_mg[64]
  {
    // A1                                                                               H1
    0.9859,   -0.7129,   -2.1689,   -2.6543,   -2.6543,   -2.1689,   -0.7129,    0.9859, 
    -0.4702,    1.2286,   -0.2275,   -1.1982,   -1.1982,   -0.2275,    1.2286,   -0.4702, 
    -1.6836,    0.0152,    2.1993,    1.2286,    1.2286,    2.1993,    0.0152,   -1.6836, 
    -1.9263,   -0.7129,    1.4712,    3.1700,    3.1700,    1.4712,   -0.7129,   -1.9263, 
    -2.1689,   -0.9556,    1.2286,    2.9273,    2.9273,    1.2286,   -0.9556,   -2.1689, 
    -1.9263,    0.2578,    2.4420,    1.4712,    1.4712,    2.4420,    0.2578,   -1.9263, 
    -0.7129,    1.4712,    0.5005,   -0.4702,   -0.4702,    0.5005,    1.4712,   -0.7129, 
    0.7432,   -0.4702,   -1.4409,   -1.4409,   -1.4409,   -1.4409,   -0.4702,    0.7432 
    // A8                                                                               H8
  };

  const float bishop_eg[64]
  {
    // A1                                                                               H1
    1.7541,   -1.2683,   -3.8589,   -4.7225,   -4.7225,   -3.8589,   -1.2683,    1.7541, 
    -0.8366,    2.1858,   -0.4048,   -2.1319,   -2.1319,   -0.4048,    2.1858,   -0.8366, 
    -2.9954,    0.0270,    3.9129,    2.1858,    2.1858,    3.9129,    0.0270,   -2.9954, 
    -3.4272,   -1.2683,    2.6176,    5.6400,    5.6400,    2.6176,   -1.2683,   -3.4272, 
    -3.8589,   -1.7001,    2.1858,    5.2082,    5.2082,    2.1858,   -1.7001,   -3.8589, 
    -3.4272,    0.4588,    4.3447,    2.6176,    2.6176,    4.3447,    0.4588,   -3.4272, 
    -1.2683,    2.6176,    0.8905,   -0.8366,   -0.8366,    0.8905,    2.6176,   -1.2683, 
    1.3223,   -0.8366,   -2.5636,   -2.5636,   -2.5636,   -2.5636,   -0.8366,    1.3223 
    // A8                                                                               H8
  };

  const float rook_mg[64]
  {
    // A1                                                                               H1
    -32.6900,  -21.7171,  -14.4019,    0.2286,    0.2286,  -14.4019,  -21.7171,  -32.6900, 
    -21.7171,  -14.4019,   -7.0866,    7.5438,    7.5438,   -7.0866,  -14.4019,  -21.7171, 
    -14.4019,   -7.0866,   -3.4290,   11.2015,   11.2015,   -3.4290,   -7.0866,  -14.4019, 
    0.2286,    7.5438,   11.2015,   11.2015,   11.2015,   11.2015,    7.5438,    0.2286, 
    11.2015,   18.5167,   22.1743,   22.1743,   22.1743,   22.1743,   18.5167,   11.2015, 
    -3.4290,    3.8862,    7.5438,   22.1743,   22.1743,    7.5438,    3.8862,   -3.4290, 
    -10.7443,   -3.4290,    3.8862,   18.5167,   18.5167,    3.8862,   -3.4290,  -10.7443, 
    -21.7171,  -10.7443,   -3.4290,   11.2015,   11.2015,   -3.4290,  -10.7443,  -21.7171 
    // A8                                                                               H8
  };

  const float rook_eg[64]
  {
    // A1                                                                               H1
    -12.3400,   -8.1979,   -5.4365,    0.0863,    0.0863,   -5.4365,   -8.1979,  -12.3400, 
    -8.1979,   -5.4365,   -2.6751,    2.8477,    2.8477,   -2.6751,   -5.4365,   -8.1979, 
    -5.4365,   -2.6751,   -1.2944,    4.2284,    4.2284,   -1.2944,   -2.6751,   -5.4365, 
    0.0863,    2.8477,    4.2284,    4.2284,    4.2284,    4.2284,    2.8477,    0.0863, 
    4.2284,    6.9898,    8.3705,    8.3705,    8.3705,    8.3705,    6.9898,    4.2284, 
    -1.2944,    1.4670,    2.8477,    8.3705,    8.3705,    2.8477,    1.4670,   -1.2944, 
    -4.0558,   -1.2944,    1.4670,    6.9898,    6.9898,    1.4670,   -1.2944,   -4.0558, 
    -8.1979,   -4.0558,   -1.2944,    4.2284,    4.2284,   -1.2944,   -4.0558,   -8.1979 
    // A8                                                                               H8
  };

  const float queen_mg[64]
  {
    // A1                                                                               H1
    -14.9789,  -16.0633,  -18.2322,  -11.7256,  -11.7256,  -18.2322,  -16.0633,  -14.9789, 
    -14.9789,   -3.0500,   -5.2189,   -0.8811,   -0.8811,   -5.2189,   -3.0500,  -14.9789, 
    -16.0633,   -4.1344,    7.7944,   12.1322,   12.1322,    7.7944,   -4.1344,  -16.0633, 
    -8.4722,    1.2878,   13.2167,   20.8078,   20.8078,   13.2167,    1.2878,   -8.4722, 
    -3.0500,    6.7100,   18.6389,   26.2300,   26.2300,   18.6389,    6.7100,   -3.0500, 
    -10.6411,    3.4567,   15.3856,   19.7233,   19.7233,   15.3856,    3.4567,  -10.6411, 
    -9.5567,    4.5411,    4.5411,    8.8789,    8.8789,    4.5411,    4.5411,   -9.5567, 
    -9.5567,   -8.4722,   -8.4722,    0.2033,    0.2033,   -8.4722,   -8.4722,   -9.5567 
    // A8                                                                               H8
  };

  const float queen_eg[64]
  {
    // A1                                                                               H1
    -19.5474,  -20.9626,  -23.7929,  -15.3018,  -15.3018,  -23.7929,  -20.9626,  -19.5474, 
    -19.5474,   -3.9802,   -6.8106,   -1.1498,   -1.1498,   -6.8106,   -3.9802,  -19.5474, 
    -20.9626,   -5.3954,   10.1717,   15.8325,   15.8325,   10.1717,   -5.3954,  -20.9626, 
    -11.0562,    1.6805,   17.2477,   27.1540,   27.1540,   17.2477,    1.6805,  -11.0562, 
    -3.9802,    8.7565,   24.3236,   34.2300,   34.2300,   24.3236,    8.7565,   -3.9802, 
    -13.8866,    4.5109,   20.0781,   25.7388,   25.7388,   20.0781,    4.5109,  -13.8866, 
    -12.4714,    5.9261,    5.9261,   11.5869,   11.5869,    5.9261,    5.9261,  -12.4714, 
    -12.4714,  -11.0562,  -11.0562,    0.2653,    0.2653,  -11.0562,  -11.0562,  -12.4714 
    // A8                                                                               H8
  };

  const float king_mg[64]
  {
    // A1                                                                               H1
    5.4000,    5.4000,    5.4000,    5.4000,    5.4000,    5.4000,    5.4000,    5.4000, 
    3.8571,    3.8571,    3.8571,    3.8571,    3.8571,    3.8571,    3.8571,    3.8571, 
    2.3143,    2.3143,    2.3143,    2.3143,    2.3143,    2.3143,    2.3143,    2.3143, 
    0.7714,    0.7714,    0.7714,    0.7714,    0.7714,    0.7714,    0.7714,    0.7714, 
    -0.7714,   -0.7714,   -0.7714,   -0.7714,   -0.7714,   -0.7714,   -0.7714,   -0.7714, 
    -2.3143,   -2.3143,   -2.3143,   -2.3143,   -2.3143,   -2.3143,   -2.3143,   -2.3143, 
    -3.8571,   -3.8571,   -3.8571,   -3.8571,   -3.8571,   -3.8571,   -3.8571,   -3.8571, 
    -5.4000,   -5.4000,   -5.4000,   -5.4000,   -5.4000,   -5.4000,   -5.4000,   -5.4000 
    // A8                                                                               H8
  };

  const float king_eg[64]
  {
    // A1                                                                               H1
    -12.6304,   -9.9888,   -9.1082,   -9.1082,   -9.1082,   -9.1082,   -9.9888,  -12.6304, 
    -9.9888,   -5.5860,   -2.9443,   -2.0638,   -2.0638,   -2.9443,   -5.5860,   -9.9888, 
    -9.1082,   -2.9443,    4.1001,    9.3834,    9.3834,    4.1001,   -2.9443,   -9.1082, 
    -7.3471,    0.5779,   12.0250,   18.1889,   18.1889,   12.0250,    0.5779,   -7.3471, 
    -6.4666,    2.3390,   13.7861,   19.9500,   19.9500,   13.7861,    2.3390,   -6.4666, 
    -4.7054,    4.1001,   11.1445,   16.4278,   16.4278,   11.1445,    4.1001,   -4.7054, 
    -5.5860,    1.4584,    4.1001,    4.9806,    4.9806,    4.1001,    1.4584,   -5.5860, 
    -9.9888,   -5.5860,   -4.7054,   -4.7054,   -4.7054,   -4.7054,   -5.5860,   -9.9888 
    // A8                                                                               H8
  };

};


#endif
