/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef WEIGHTS_H
#define WEIGHTS_H

#include<vector>
#include<string>
using std::vector;
using std::string;


enum
  {
    w_material,
    w_mobility,
    w_knight_mobility,
    w_bishop_mobility,
    w_rook_mobility,
    w_queen_mobility,
    w_development,
    w_space,
    w_pst_pawn,
    w_pst_knight,
    w_pst_bishop,
    w_pst_rook,
    w_pst_queen,
    w_pst_king,
    w_pawn_shield,
    w_king_attack,
    w_castling_rights,
    w_pawn_storm,
    w_open_files_on_king,
    w_safe_queencheck,
    w_knight_king_contact,
    w_bishop_king_contact,
    w_rook_king_contact,
    w_queen_king_contact,
    w_knight_battlefield_contact,
    w_bishop_battlefield_contact,
    w_rook_battlefield_contact,
    w_queen_battlefield_contact,
    w_knight_battlefield_attack,
    w_bishop_battlefield_attack,
    w_rook_battlefield_attack,
    w_queen_battlefield_attack,
    w_king_activity,
    w_locked_up_rook,
    w_undefended_pieces,
    w_own_hanging_material,
    w_rookpawn,
    w_doubled_pawns,
    w_weak_pawns,
    w_isolated_pawns,
    w_blocked_pawns,
    w_passed_pawns,
    w_pawn_islands,
    w_pinned_pieces,
    w_bad_bishop,
    w_opposite_colored_bishops,
    w_stm
  };


class Weight
{
 public:
  vector<string> name
  {
    "material",
      "mobility",
      "knight_mobility",
      "bishop_mobility",
      "rook_mobility",
      "queen_mobility",
      "development",
      "space",
      "pst_pawn",
      "pst_knight",
      "pst_bishop",
      "pst_rook",
      "pst_queen",
      "pst_king",
      "pawn_shield",
      "king_attack",
      "castling_rights",
      "pawn_storm",
      "open_files_on_king",
      "safe_queencheck",
      "knight_king_contact",
      "bishop_king_contact",
      "rook_king_contact",
      "queen_king_contact",
      "knight_battlefield_contact",
      "bishop_battlefield_contact",
      "rook_battlefield_contact",
      "queen_battlefield_contact",
      "knight_battlefield_attack",
      "bishop_battlefield_attack",
      "rook_battlefield_attack",
      "queen_battlefield_attack",
      "king_activity",
      "locked_up_rook",
      "undefended_pieces",
      "own_hanging_material",
      "rookpawn",
      "doubled_pawns",
      "weak_pawns",
      "isolated_pawns",
      "blocked_pawns",
      "passed_pawns",
      "pawn_islands",
      "pinned_pieces",
      "bad_bishop",
      "opposite_colored_bishops",
      "stm"
      };
  float& mg(int i){return m_weight_mg[i];};
  float& eg(int i){return m_weight_eg[i];};
  vector<float>& weight_mg_vector(){return m_weight_mg;};
  vector<float>& weight_eg_vector(){return m_weight_eg;};
  unsigned size()const{return m_weight_mg.size();}
  
private:
  vector<float> m_weight_mg{2.50148,1.04707,0.581864,0.747138,0.000143933,2.91113,1.4882,0.610783,1.63201,0.382588,
      2.19208,0.60714,9.94566e-07,20.1419,2.34221,2.09817,0.0989548,1.15545,0.236645,14.1252,
      0.00010267,9.68724e-05,0.0213516,355.561,19.5868,7.36273e-05,0.000186614,0.0344313,4.84189,7.26411,
      2.20566,4.33753,2.8714e-05,109.926,9.65115,67.9774,43.4654,11.6542,15.4674,2.89348,
      15.0767,0.0350917,24.7771,2.70945,3.22237,22.3661,14};
  vector<float> m_weight_eg{2.05895,9.96812e-07,1.63203e-06,0.905745,3.77674,14.6056,1.15224,1.68053e-05,1.09013,10.2664,
      0.000151478,0.723856,0.519414,1.52514,1.49304e-06,3.56108e-06,6.36035e-06,4.76514e-06,0.0461258,0.00166655,
      7.56091e-05,8.16766e-05,0.000185858,10.2923,0.000113646,3.86269,8.42957e-05,27.9784,0.118689,8.25708,
      0.118689,8.25708,4.90238,1.70761,0.560495,23.5576,3.91959e-06,46.6105,4.572,0.00718341,
      29.9991,3.04884,4.52725e-05,65.7304,9.93572e-07,2.31197,14};
};



#endif
