/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<new>
#include<sstream>
#include<stdexcept>
#include"bucket.h"
using std::bad_alloc;
using std::logic_error;
using std::cout;
using std::endl;
using std::hex;
using std::dec;
using std::ostringstream;



void Bucket::clear()
{
  for(int i=0;i<4;i++){
    m_ttr[i].clear();
  }
}

int Bucket::has_date(unsigned char x)
{
  int r{0};
  for(int i=0;i<4;i++){
    if(m_ttr[i].get_date()==x){r++;}
  }
  return r;
}

bool Bucket::read(TTrecord& x)const
{
  uint64_t key=x.get_key();
  for(int i=0;i<4;i++){
    if(key==m_ttr[i].get_key()){
      x=m_ttr[i];
      return true;
    }
  }
  return false;
}

void Bucket::write(const TTrecord& ttr)
{
  int index_with_least_draft{0};
  int draft=max_search_depth;

  //---------------- is this key already in this bucket? -------------------
  for(int i=0;i<4;i++){
    if(ttr.get_key()==m_ttr[i].get_key()){
      m_ttr[i]=ttr;
      return;
    }
  }

  //---------------- no it is not: -----------------------------------------
  for(int i=0;i<4;i++){
    if(ttr.get_date()!=m_ttr[i].get_date()){
      m_ttr[i]=ttr;
      return;
    }
    if(draft>m_ttr[i].get_draft() && !m_ttr[i].is_exact_value()){
      draft=m_ttr[i].get_draft();
      index_with_least_draft=i;
    }
  }

  //---------------- store ttr in entry with least draft -------------------
  m_ttr[index_with_least_draft]=ttr;
}

string Bucket::dump()const
{
  ostringstream oss;
  oss << hex << m_ttr[0].get_key() << dec << "  score=" << m_ttr[0].get_score(0) << "  draft=" << m_ttr[0].get_draft() << endl;
  oss << hex << m_ttr[1].get_key() << dec << "  score=" << m_ttr[1].get_score(0) << "  draft=" << m_ttr[1].get_draft() <<endl;
  oss << hex << m_ttr[2].get_key() << dec << "  score=" << m_ttr[2].get_score(0) << "  draft=" << m_ttr[2].get_draft() << endl;
  oss << hex << m_ttr[3].get_key() << dec << "  score=" << m_ttr[3].get_score(0) << "  draft=" << m_ttr[3].get_draft() << endl;
  return oss.str();
}
