/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<iomanip>
#include<new>
#include<sstream>
#include<stdexcept>
#include"pawnhashtable.h"
using std::bad_alloc;
using std::logic_error;
using std::cout;
using std::endl;
using std::hex;
using std::dec;
using std::setw;
using std::setprecision;
using std::ostringstream;



PawnHashTable::PawnHashTable()
{
  number_of_entries=1<<14;
  pht=new PHTrecord[number_of_entries]; 
  clear();
}

PawnHashTable::~PawnHashTable()
{
  delete[] pht;
}

void PawnHashTable::clear()
{
  PHTrecord r;
  for(unsigned int i=0;i<number_of_entries;i++){pht[i]=r;}
}

const PHTrecord& PawnHashTable::get_current()const
{
  return phtr;
}

void PawnHashTable::set_current(PHTrecord x)
{
  phtr=x;
}

bool PawnHashTable::read(uint64_t key)
{
  m_total_read_count++;
  int x = key & (number_of_entries-1);
  if(pht[x].get_key()==key){
    phtr=pht[x];
    return true;
  }else{
    if(pht[x].get_key()==0){
      m_first_pht_read++;
    }else{
      m_conflicting_pht_read++;
    }
    return false;
  }
}

void PawnHashTable::write()
{
  uint64_t key=phtr.get_key();
  int x = key & (number_of_entries-1);
  pht[x]=phtr;
}

string PawnHashTable::stats()
{
  PHTrecord r;
  double in_use{0};
  for(unsigned int i=0;i<number_of_entries;i++){
    if(pht[i].get_key()!=r.get_key()){in_use+=1.0;}
  }
  in_use*=100.0;
  in_use/=number_of_entries;
  
  ostringstream oss;
  oss << "PawnHashTable::stats() usage: " << setprecision(3) << in_use << "%"
      << "  first pht reads: " << m_first_pht_read << "  "
      << "  conflicting pht reads: " << m_conflicting_pht_read
      << "  total reads: " << m_total_read_count << endl;
  return oss.str();
}
