/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef PHTRECORD_H
#define PHTRECORD_H

#include"../common/bitboard.h"


class PHTrecord
{
 private:
  uint64_t m_key{0};

  // bitboards:
  BitBoard m_white_passed_pawns;
  BitBoard m_white_passed_pawn_stops;
  BitBoard m_white_passed_pawn_telestops;
  BitBoard m_white_passed_pawn_rearspan;
  BitBoard m_white_weak_pawns;
  BitBoard m_white_open_files;
  BitBoard m_black_passed_pawns;
  BitBoard m_black_passed_pawn_stops;
  BitBoard m_black_passed_pawn_telestops;
  BitBoard m_black_passed_pawn_rearspan;
  BitBoard m_black_weak_pawns;
  BitBoard m_black_open_files;

  // numbers:
  int m_weighted_white_passed_pawns{0};
  int m_weighted_black_passed_pawns{0};
  int m_weighted_doubled_pawns{0};
  int m_weak_pawns{0};
  int m_isolated_pawns{0};
  int m_pawn_islands{0};
  int m_rook_pawns{0};

  static const vector<char> pawn_island_table;

  
 public:
  PHTrecord();
  void clear();

  const BitBoard& get_white_passed_pawns()const{return m_white_passed_pawns;}
  const BitBoard& get_white_passed_pawn_stops()const{return m_white_passed_pawn_stops;}
  const BitBoard& get_white_passed_pawn_telestops()const{return m_white_passed_pawn_telestops;}
  const BitBoard& get_white_passed_pawn_rearspan()const{return m_white_passed_pawn_rearspan;}
  const BitBoard& get_white_weak_pawns()const{return m_white_weak_pawns;}
  const BitBoard& get_white_open_files()const{return m_white_open_files;}
  const BitBoard& get_black_passed_pawns()const{return m_black_passed_pawns;}
  const BitBoard& get_black_passed_pawn_stops()const{return m_black_passed_pawn_stops;}
  const BitBoard& get_black_passed_pawn_telestops()const{return m_black_passed_pawn_telestops;}
  const BitBoard& get_black_passed_pawn_rearspan()const{return m_black_passed_pawn_rearspan;}
  const BitBoard& get_black_weak_pawns()const{return m_black_weak_pawns;}
  const BitBoard& get_black_open_files()const{return m_black_open_files;}

  const int& get_weighted_white_passed_pawns()const{return m_weighted_white_passed_pawns;}
  const int& get_weighted_black_passed_pawns()const{return m_weighted_black_passed_pawns;}
  const int& get_weighted_doubled_pawns()const{return m_weighted_doubled_pawns;}
  const int& get_weak_pawns()const{return m_weak_pawns;}
  const int& get_isolated_pawns()const{return m_isolated_pawns;}
  const int& get_pawn_islands()const{return m_pawn_islands;}
  const int& get_rook_pawns()const{return m_rook_pawns;}
  
  uint64_t get_key()const{return m_key;}
  void set_key(uint64_t k){m_key=k;}
  void set_pawns(const BitBoard&,const BitBoard&);
};


#endif
