/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include"ttmove.h"
#include"../board/cposition.h"
#include<stdexcept>
#include<sstream>
using std::logic_error;
using std::cout;
using std::endl;
using std::ostringstream;


TTmove::TTmove(): m_move{0}
{
}

TTmove::TTmove(uint16_t x): m_move{x}
{
}

TTmove::TTmove(const Move& x)
{
  m_move=x.m_move;
}

Move TTmove::operator()(const ChessPosition& cp)const
{
  Move m;
  if(m_move==0)return m;
  
  m.m_move=m_move;
  m.piece(cp.get_piece_from(m.from()));
  m.capture(cp.get_piece_from(m.to()));
  if(m.code()==en_passant){
    if(m.piece().white_pawn()){
      m.capture(BlackPawn());
    }else{
      m.capture(WhitePawn());
    }
  }
  if(m.code()==castling){
    m.capture(NoPiece());
  }
  return m;
}

TTmove& TTmove::operator=(const Move& x)
{
  m_move=x.m_move;
  return *this;
}

bool TTmove::operator==(const TTmove& x)const
{
  return m_move==x.m_move;
}

bool TTmove::operator!=(const TTmove& x)const
{
  return m_move!=x.m_move;
}

bool TTmove::nomove()const
{
  return !static_cast<bool>(m_move);
}

void TTmove::from(char c)
{
  m_move&=0xf03f;
  m_move|=c<<6;
}

void TTmove::to(char c)
{
  m_move&=0xffc0;
  m_move|=c;
}

Square TTmove::from()const
{
  if(m_move!=0){
    return Square((m_move>>6)&0x3f);
  }else{
    return no_square;
  }
}

Square TTmove::to()const
{
  if(m_move!=0){
    return Square(m_move & 0x3f);
  }else{
    return no_square;
  }
}

Piece TTmove::piece(const ChessPosition& cp)const
{
  Piece r;
  int code=m_move>>12 & 0x0f;
  if(m_move && code!=nullmove){r=cp.get_piece_from(from());}
  return r;
}

Piece TTmove::capture(const ChessPosition& cp)const
{
  Piece r;
  int code=m_move>>12 & 0x0f;
  if(m_move && code!=nullmove){
    r=cp.get_piece_from(to());
    if(code==en_passant){
      if(to()>from()){r=BlackPawn();}else{r=WhitePawn();}
    }
  }
  return r;
}

unsigned char TTmove::code()const
{
  unsigned char r=(m_move>>12)&0x07;
  return r;
}

void TTmove::clear()
{
  m_move=0;
}

string TTmove::algebraic(const ChessPosition& cp)const
{
  string r;
  int code=m_move>>12 & 0x07;
  if(m_move && code!=nullmove){
    r=from().name();
    r+=to().name();
    if((piece(cp).white_pawn() && from()>47) || (piece(cp).black_pawn() && from()<16)){
      switch(code){
      case promo_queen:r+="q";break;
      case promo_rook:r+="r";break;
      case promo_bishop:r+="b";break;
      case promo_knight:r+="n";break;
      }
    }
  }else{r="0000";}
  return r;
}

const uint16_t& TTmove::id()const
{
  return m_move;
}
