/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include"ttrecord.h"
using std::endl;
using std::hex;
using std::dec;


TTrecord::TTrecord()
{
  key=0;
  value=0;
}

TTrecord& TTrecord::operator=(const TTrecord& x)
{
  if(key==x.key && x.value>>48==0){
    value&=0xffff000000000000;           // keep the old move;
    value|=x.value;
  }else{
    value=x.value;
  }
  key=x.key;
  return *this;
}

void TTrecord::clear()
{
  key=0;
  value=0;
}

bool TTrecord::operator==(const TTrecord& x)
{
  return key==x.key && value==x.value;
}

bool TTrecord::operator!=(const TTrecord& x)
{
  return key!=x.key || value!=x.value;
}

const uint64_t& TTrecord::get_key()const
{
  return key;
}

void TTrecord::set_key(const uint64_t& x)
{
  key=x;
}

TTmove TTrecord::get_move()const
{
  TTmove m(value>>48);
  return m;
}

void TTrecord::set_move(const TTmove& x)
{
  uint64_t v=x.id();
  value |=v<<48;
}

int16_t TTrecord::get_score(const int& ply)const
{
  union ui_score
  {
    uint16_t uscore;
    int16_t iscore;
  };
  ui_score v;
  v.uscore = value>>32;
  int16_t score=v.iscore;
  
  if(score>=mate_value-max_search_depth){
    return score-ply;
  }else if(score<=-mate_value+max_search_depth){
    return score+ply;
  }else{
    return score;
  }
}

void TTrecord::set_score(const int& x,const int& ply)
{
  value &= 0xffff0000ffffffff;
  int64_t score=x;
  if(score>=mate_value-max_search_depth){
    score+=ply;
  }else if(score<=-mate_value+max_search_depth){
    score-=ply;
  }

  union ui_score
  {
    uint16_t uscore;
    int16_t iscore;
  };
  ui_score v;
  v.iscore = score;
  uint64_t uscore = v.uscore;
  value |= (uscore<<32 & 0x0000ffff00000000);
}

bool TTrecord::is_exact_value()const
{
  unsigned char bound=value>>24;
  return bound==exact_value;
}

bool TTrecord::is_upperbound()const
{
  unsigned char bound=value>>24;
  return bound==upperbound;
}

bool TTrecord::is_lowerbound()const
{
  unsigned char bound=value>>24;
  return bound==lowerbound;
}

void TTrecord::set_bound(bound_t x)
{
  uint16_t bound=x;
  value |= bound << 24;
}

unsigned char TTrecord::get_draft()const
{
  unsigned char sd = value>>16;
  return sd;
}

void TTrecord::set_draft(const unsigned char& x)
{
  uint16_t sd=x;
  value |= sd << 16;
}

void TTrecord::set_date(const unsigned char& x)
{
  uint16_t date=x;
  value |= date << 8;
}

unsigned char TTrecord::get_date()const
{
  unsigned char date = value>>8;
  return date;
}

void TTrecord::set_value(const uint64_t& x)
{
  value=x;
}

const uint64_t& TTrecord::get_value()const
{
  return value;
}
