/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MOVE_H
#define MOVE_H

#include"../common/bitboard.h"
#include"../common/square.h"
#include"../common/piece.h"
#include<cstdint>
using std::uint32_t;


enum move_t {nullmove,normal_move,promo_queen,promo_rook,promo_bishop,promo_knight,castling,en_passant};
enum xmove_t {hashtable_move,mate_attack,promotion,good_capture,killer,counter_move,historical_move1,historical_move2,historical_move3,xnormal_move,bad_move,losing_capture,no_xmove_t};

class TTmove;
class Move
{
 protected:
  uint32_t m_move;   // (xcode: 8bits, piece: 4bits, capture: 4bits, check: 1bit, code: 3bits, from: 6bits, to: 6bits)
  friend TTmove;
  
 public:
  Move();
  Move(Square,Square,uint32_t,Piece,Piece);
  Move(Square,Square,uint32_t,piece_t,piece_t);
  bool operator==(Move)const;
  bool operator!=(Move)const;
  bool operator<(Move)const;
  string algebraic(bool=false)const;
  string dump()const;

  Square from()const;
  void from(const Square&);
  Square to()const;
  void to(const Square&);
  Piece capture()const;
  int capture_id()const;
  void capture_id(int);
  void capture(const Piece&);
  void capture(uint32_t&&);
  Piece piece()const;
  int piece_id()const;
  void piece_id(int);
  void piece(const Piece&);
  void piece(uint32_t&&);
  move_t code()const;
  void code(const move_t&);
  xmove_t xcode()const;
  void xcode(const xmove_t&);
  bool check()const;
  void check(bool);
  void clear();
};

class NullMove : public Move
{
 public: 
  NullMove(){m_move=0;}
};

#endif
