/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef MOVEGENERATOR_H
#define MOVEGENERATOR_H

#include"../board/cposition.h"
#include"../common/attacks.h"
#include"../movegen/mlist.h"


class MoveGenerator
{
 private:
  const Connections& m_con;
  const Attacks& m_atta;
  unsigned m_current{0};
  
  BitBoard m_pieces,m_pinned_pieces,m_discovering_pieces,m_own_pieces,m_opponent_pieces,m_opponent_pieces_without_king;
  BitBoard m_ownkingsfield,m_opponentkingsfield;
  BitBoard m_rookattacks,m_bishopattacks,m_knightattacks,m_pawnattacks;
  
  vector<Square> m_king_attackers;
  bool is_legal(const ChessPosition&,const Move&)const;
  bool is_legal_pawnmove(const ChessPosition&,const Move&)const;
  template<int> bool is_legal_kingmove(const ChessPosition&,const Move&)const;
  template<int> void set_checkstatus(const ChessPosition&,Move&);
  template<int> bool discovered_check(const ChessPosition&,const BitBoard&,const BitBoard&);
  template<int> void generate_movetable(const ChessPosition&,array<uint64_t,64>&);
  template<int> void generate_captures(const ChessPosition&,array<uint64_t,64>&,MoveList&);
  template<int> void generate_quiet_moves(const ChessPosition&,array<uint64_t,64>&,MoveList&);
  
 public:
  MoveGenerator(const Tables&);
  MoveGenerator(const MoveGenerator&);
  void create_movetable(const ChessPosition&,array<uint64_t,64>&);
  void remove(const Move&,array<uint64_t,64>&);
  void serialize_moves(const ChessPosition&,array<uint64_t,64>&,MoveList&);
  void serialize_captures(const ChessPosition&,array<uint64_t,64>&,MoveList&);
  void generate_moves(const ChessPosition&,MoveList&);
};


#endif
