/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"countermove.h"
#include<iostream>
#include<stdexcept>
using std::endl;
using std::logic_error;



CounterMove::CounterMove()
{
  m_pcm=new cm_array;
  clear();
}

CounterMove::~CounterMove()
{
  delete m_pcm;
}

void CounterMove::clear()
{
  for(auto& v : *m_pcm){
    for(auto& hist : v){
      hist.clear();
    }
  }  
}

void CounterMove::divide_by_8()
{
  for(auto& v : *m_pcm){
    for(auto& hist : v){
      hist.divide_by_8();
    }
  }  
}

void CounterMove::increment(const Move& m1,const Move& m2,int d)
{
  int p=m1.piece_id();
  int s=m1.to().id();
  (*m_pcm)[p][s].increment(m2,d);
}

void CounterMove::decrement(const Move& m1,const Move& m2,int d)
{
  int p=m1.piece_id();
  int s=m1.to().id();
  (*m_pcm)[p][s].decrement(m2,d);
}

const int64_t& CounterMove::value(const Move& m1,const Move& m2)const
{
  int p=m1.piece_id();
  int s=m1.to().id();
  return (*m_pcm)[p][s].value(m2);
}

const int64_t& CounterMove::value(const int& p, const int& s, const Move& m2)const
{
  return (*m_pcm)[p][s].value(m2);
}
