/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"history.h"


HistoryTable::HistoryTable()
{
  clear();
}

void HistoryTable::clear()
{
  for(auto& v : h){
    for(auto& i : v){
      i=0;
    }
  }  
}

void HistoryTable::divide_by_8()
{
  for(auto& v : h){
    for(auto& i : v){
      i>>=3;
    }
  }  
}

void HistoryTable::increment(const Move& m,const unsigned int& d)
{
  int p=m.piece_id();
  int s=m.to().id();
  h[p][s]+=1ull<<static_cast<int64_t>(d&0x3f);
}

void HistoryTable::decrement(const Move& m,const unsigned int& d)
{
  int p=m.piece_id();
  int s=m.to().id();
  h[p][s]-=1ull<<static_cast<int64_t>(d&0x3f);
}

const int64_t& HistoryTable::value(const Move& m)const
{
  int i=m.piece_id();
  int j=m.to().id();
  return h[i][j];
}


