/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"timemanager.h"


TimeManager::TimeManager()
{
  reset();
}

void TimeManager::reset()
{
  m_tim.reset();
  m_movetime=(1ll<<40);
  m_wtime=m_movetime;
  m_btime=m_movetime;
  m_winc=0;
  m_binc=0;
  m_movestogo=999;
  m_mate=999999;
  m_infinite=false;
  m_pondering=false;
}

void TimeManager::start()
{
  m_tim.reset();
  m_tim.start();
}

bool TimeManager::time_to_move()
{
  uint64_t t=m_tim.total_time().milliseconds();
  if(m_infinite){return false;}
  if(m_pondering){return false;}
  if(m_movetime>0 && t>=m_movetime-m_lag){return true;}  
  bool r=false;
  if(m_movetime==1ll<<40){
    uint64_t time,inc;
    if(m_white_to_move){time=m_wtime;inc=m_winc;}
    else{time=m_btime;inc=m_binc;}
    time+=inc*(m_movestogo);
    uint64_t target_time=time/(m_movestogo+2);
    target_time += (target_time/2) * m_movestogo / 40 - target_time/4;
    uint64_t extended_time=2*target_time;
    if(t>extended_time){r=true;}
  }
  return r;
}

int TimeManager::percentage()
{
  int t=m_tim.total_time().milliseconds();
  if(m_infinite){return 0;}
  if(m_pondering){return 0;}
  if(m_movetime>0 && m_movetime<1ll<<40){
    return 100*t / m_movetime;
  }else{
    uint64_t time,inc;
    if(m_white_to_move){time=m_wtime;inc=m_winc;}
    else{time=m_btime;inc=m_binc;}
    time+=inc*(m_movestogo);
    uint64_t target_time=time/(m_movestogo+2);
    target_time += target_time * m_movestogo / 40 - target_time/2;
    return 100*t / target_time;
  }
}

int TimeManager::remaining_time()
{
  int r;
  int t=m_tim.total_time().milliseconds();
  if(m_movetime==1ll<<40){
    int etime;
    if(m_white_to_move){
      etime=m_wtime;
    }else{
      etime=m_btime;
    }
    r=etime-t;
  }else{
    r=m_movetime-t;
  }
  return r;
}

void TimeManager::set_wtime(uint64_t x)
{
  m_wtime=x;
}

void TimeManager::set_btime(uint64_t x)
{
  m_btime=x;
}

void TimeManager::set_winc(uint64_t x)
{
  m_winc=x;
}

void TimeManager::set_binc(uint64_t x)
{
  m_binc=x;
}

void TimeManager::set_movetime(uint64_t x)
{
  m_movetime=x==0?1:x;
}

void TimeManager::set_movestogo(int x)
{
  m_movestogo=x;
  if(m_movestogo<=0){m_movestogo=30;}  // sudden death game
}

void TimeManager::set_mate(int x)
{
  m_mate=x;
}

void TimeManager::set_lag(int x)
{
  m_lag=x;
}

void TimeManager::set_infinite(bool x)
{
  m_infinite=x;
}

void TimeManager::set_pondering(bool x)
{
  m_pondering=x;
}

void TimeManager::set_white_to_move(bool x)
{
  m_white_to_move=x;
}
