/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"board.h"
#include<cairomm/context.h>
#include<gdkmm/general.h> // set_source_pixbuf()
#include<glibmm/fileutils.h>
#include<iostream>



Board::Board()
{
  m_flip=false;
  m_drawing.show();
  add(m_drawing);
  set_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::POINTER_MOTION_MASK);
  signal_button_press_event().connect(sigc::mem_fun(*this,
						    &Board::on_eventbox_button_press));
  signal_button_release_event().connect(sigc::mem_fun(*this,
						    &Board::on_eventbox_button_release));
  signal_motion_notify_event().connect(sigc::mem_fun(*this,
						    &Board::on_eventbox_motion_notify));
  show();
}


Board::~Board()
{
}

bool Board::on_eventbox_button_press(GdkEventButton* E)
{
  int x,y;
  get_pointer(x,y);
  m_drawing.button1_pressed(x,y);
  return true;
}

bool Board::on_eventbox_button_release(GdkEventButton* E)
{
  int x,y;
  get_pointer(x,y);
  m_move=m_drawing.button1_released(x,y);
  signal_piece_moved.emit();
  return true;
}

bool Board::on_eventbox_motion_notify(GdkEventMotion* E)
{
  int x,y;
  get_pointer(x,y);
  m_drawing.drag_piece_to(x,y);
  return true;
}

void Board::initialize(const vector<char>& v)
{
  if(m_flip){
    vector<char> w(64,'?');
    for(int i=0;i<64;i++){w[63-i]=v[i];}
    m_drawing.initialize(w);
  }
  else{
    m_drawing.initialize(v);
  }
}

void Board::set_parent_window(Gtk::Window* ppw)
{
  m_drawing.set_parent_window(ppw);
}

void Board::set_piece_on(char p,int sq)
{
  if(m_flip){sq=63-sq;}
  m_drawing.set_piece_on(p,sq);
}

void Board::remove_piece_from(int sq)
{
  if(m_flip){sq=63-sq;}
  m_drawing.remove_piece_from(sq);
}

pair<int,int> Board::get_move()const
{
  if(m_flip){
    pair<int,int> temp_move{m_move};
    unsigned int to=temp_move.second&0x003F; // separate to-square from promotion choice
    temp_move.second&=0xFFFC0;
    to=63-to;
    temp_move.second|=to;
    temp_move.first=63-temp_move.first;
    return temp_move;
  }
  else{
    return m_move;
  }
}

void Board::flip()
{
  m_flip=!m_flip;
}

vector<Glib::RefPtr<Gdk::Pixbuf>> Board::get_images()const
{
  return m_drawing.get_images();
}
