/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"cecp_engine.h"
#include"sanmove.h"
#include<thread>
#include<string>
#include<iostream>
#include<sstream>
using std::endl;
using std::istringstream;
using std::ostringstream;
using std::to_string;

CecpEngine::CecpEngine()
{
  tbs.transpositiontable().set_size(0);
  m_ponder=false;
}

CecpEngine::~CecpEngine()
{
}

void CecpEngine::newgame()
{
  stop();
  reset_clock();
  m_cp.newgame();
  m_forced=false;
  cecp_cp.newgame();
  m_pcom_engine->to_engine("force");
  m_pcom_engine->to_engine("new");
  m_pcom_engine->to_engine("force");
  m_pcom_engine->to_engine("easy");
  m_pcom_engine->to_engine("post");

  //
  m_level=m_engvar.level();
  ostringstream oss;
  oss << "level " << m_engvar.get_level();
  m_pcom_engine->to_engine(oss.str());
  //
  
  //-------------------- wait a bit: ------------------
  int cnt{0};
  if(m_engvar.ping()){
    ostringstream oss;
    oss << ++m_pingpong_number;
    m_pcom_engine->to_engine("ping "+oss.str());
    string answer;
    while(true){
      answer=m_pcom_engine->from_engine();
      if(answer.size()>0){
	string command;
	int value;
	istringstream iss(answer);
	iss >> command;
	if(command=="pong"){
	  iss >> value;
	  if(value==m_pingpong_number){break;}
	}
      }
      if(++cnt>6000){break;}
      std::this_thread::sleep_for(milliseconds(10));
    }
  }
  else{
    std::this_thread::sleep_for(milliseconds(500));
  }
  //---------------------------------------------------
}

void CecpEngine::set_ponder(bool x)
{
  m_ponder=x;
  if(x){m_pcom_engine->to_engine("hard");}else{m_pcom_engine->to_engine("easy");}
}

void CecpEngine::stop()
{
  if(m_log){*m_pofs << "engine" << m_engine_id << " <- CecpEngine::stop()" << endl;}
  if(m_engvar.get_status()!=EngineStatus::thinking && m_engvar.get_status()!=EngineStatus::pondering){return;}
  if(m_engvar.get_status()==EngineStatus::thinking){
    m_pcom_engine->to_engine("?");
  }
  int count{0};
  while(true){
    string answer=m_pcom_engine->from_engine();
    istringstream iss(answer);
    string command;iss >> command;
    if(command=="move"){
      m_engvar.set_status(EngineStatus::idle);
      m_engvar.move_and_stop_clock();
      string s;
      iss >> s;
      if(s[0]=='o' ||s[0]=='O' || s[0]=='0'){s=castle2move(s);}   // oh || OH || zero
      if(m_engvar.san()){
	SanMove sm(m_cp,m_mg);
	sm=s;
	s=sm.algebraic();
      }
      Move m;
      m=alg2move(s);
      answer="bestmove " + m.algebraic();
      cecp_cp.make_move(m);
    }
    if(++count>1000){
      break;
    }
    if(m_log){*m_pofs << "#";}
    std::this_thread::sleep_for(milliseconds(10));
  }  
}

void CecpEngine::go_infinite()
{
  std::cerr << "CecpEngine::go_infinite() not yet implemented" << endl;
}

void CecpEngine::go()
{
  m_engvar.set_status(EngineStatus::thinking);
  start_clock();
  if(m_engvar.level()!=m_level){
    m_level=m_engvar.level();
    ostringstream oss;
    oss << "level " << m_engvar.get_level();
    m_pcom_engine->to_engine(oss.str());
  }
  if(m_cp==cecp_cp){
    m_forced=false;
    if(m_engvar.time()){
      string s{"time "};
      s+=to_string(get_time().centiseconds());
      m_pcom_engine->to_engine(s);
      s="otim ";
      s+=to_string(m_opponent->get_time().centiseconds());
      m_pcom_engine->to_engine(s);
    }
    m_pcom_engine->to_engine("go");
    return;
  }else{
    ChessPosition cp{m_cp};
    if(cp.history().size()>1){
      cp.unmake_move();
    }
    if(cp==cecp_cp){
      string s;
      if(m_engvar.usermove()){s="usermove ";}
      ChessPosition temp_cp{m_cp};
      Move m=m_cp.last_move();
      SanMove sm(temp_cp,m_mg);
      sm=m;
      if(m_engvar.san()){
	s+=sm.str();
      }else{
	s+=sm.algebraic();
      }
      cecp_cp.make_move(m);
      if(m_engvar.time()){
	string s{"time "};
	s+=to_string(get_time().centiseconds());
	m_pcom_engine->to_engine(s);
	s="otim ";
	s+=to_string(m_opponent->get_time().centiseconds());
	m_pcom_engine->to_engine(s);
      }
      m_pcom_engine->to_engine(s);
      if(m_forced){m_forced=false;m_pcom_engine->to_engine("go");}
      return;
    }else if(m_engvar.setboard()){
      cecp_cp=m_cp;
      string s{"setboard "};
      m_pcom_engine->to_engine(s+m_cp.fen());
      m_forced=false;
      if(m_engvar.time()){
	string s{"time "};
	s+=to_string(get_time().centiseconds());
	m_pcom_engine->to_engine(s);
	s="otim ";
	s+=to_string(m_opponent->get_time().centiseconds());
	m_pcom_engine->to_engine(s);
      }
      m_pcom_engine->to_engine("go");
      return;
    }else if(m_cp.initial_position()=="rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"){
      m_pcom_engine->to_engine("new");
      m_pcom_engine->to_engine("force");
      ChessPosition temp_cp{m_cp};
      SanMove m(temp_cp,m_mg);
      for(size_t i=1;i<m_cp.history().size();i++){
	string s;
	if(m_engvar.usermove()){s="usermove ";}
	m=m_cp.history()[i].last_move;
	if(m_engvar.san()){
	  s+=m.str();
	}else{
	  s+=m.algebraic();
	}
	m_pcom_engine->to_engine(s);      
      }
      cecp_cp=m_cp;
      m_forced=false;
      if(m_engvar.time()){
	string s{"time "};
	s+=to_string(get_time().centiseconds());
	m_pcom_engine->to_engine(s);
	s="otim ";
	s+=to_string(m_opponent->get_time().centiseconds());
	m_pcom_engine->to_engine(s);
      }
      m_pcom_engine->to_engine("go");
      return;
    }else{
      cecp_cp=m_cp;
      BitBoard bb;

      // set color to play:
      if(cecp_cp.stm()==0){
	m_pcom_engine->to_engine("new");
	m_pcom_engine->to_engine("force");
	m_pcom_engine->to_engine("a2a3");
      }

      m_pcom_engine->to_engine("force");
      m_pcom_engine->to_engine("edit");

      // white pieces: 
      m_pcom_engine->to_engine("#");

      bb=cecp_cp.white_king();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="K"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.white_queen();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="Q"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.white_rook();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="R"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.white_bishop();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="B"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.white_knight();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="N"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.white_pawn();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="P"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      
      // black pieces:
      m_pcom_engine->to_engine("c");

      bb=cecp_cp.black_king();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="K"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.black_queen();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="Q"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.black_rook();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="R"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.black_bishop();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="B"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.black_knight();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="N"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      bb=cecp_cp.black_pawn();
      while(bb){
	Square sq=bb.get_reset_square().get_unique_square();
	string s="P"+sq.name();
	m_pcom_engine->to_engine(s);	
      }
      m_pcom_engine->to_engine(".");

      m_forced=false;
      if(m_engvar.time()){
	string s{"time "};
	s+=to_string(get_time().centiseconds());
	m_pcom_engine->to_engine(s);
	s="otim ";
	s+=to_string(m_opponent->get_time().centiseconds());
	m_pcom_engine->to_engine(s);
      }
      m_pcom_engine->to_engine("go");
      return;
    }
  }
}

string CecpEngine::get_answer()
{
  string answer=m_pcom_engine->from_engine();
  if(answer.size()>0){
    istringstream iss(answer);
    string command;iss >> command;
    if(command=="move"){
      m_engvar.set_status(EngineStatus::idle);
      m_engvar.move_and_stop_clock();
      string s;
      iss >> s;
      if(s[0]=='o' ||s[0]=='O' || s[0]=='0'){s=castle2move(s);}   // oh || OH || zero
      if(m_engvar.san()){
	SanMove sm(m_cp,m_mg);
	sm=s;
	s=sm.algebraic();
      }
      Move m;
      m=alg2move(s);
      answer="bestmove " + m.algebraic();
      cecp_cp.make_move(m);
    }
    else if(command.find("resign")!=string::npos || command=="Illegal"){
      m_engvar.set_status(EngineStatus::idle);
      answer="resign";
    }
    else if(command[0]=='#'){
      answer="";
    }
    else if(command[0]>='0' && command[0]<='9'){
      if(command=="0-1" || command=="1-0" || command=="1/2-1/2"){answer="";}
      else{
	int depth,score,time,nodes,nps{0};
	istringstream iss2(answer);
	iss2 >> depth;if(!iss2.good()){return "";}
	iss2 >> score;if(!iss2.good()){return "";}
	iss2 >> time;if(!iss2.good()){return "";}
	iss2 >> nodes;if(!iss2.good()){return "";}
	string pv_line;
	getline(iss2,pv_line);
	if(time>0){nps=static_cast<float>(nodes) / static_cast<float>(time) * 100.0;}
	ostringstream oss;
	oss << "info depth " << depth << " score cp " << score
	    << " time " << time << " nodes " << nodes
	    << " nps " << nps << " " << pv_line;
	answer=oss.str();
      }
    }
  } 
  return answer;
}

string CecpEngine::castle2move(string x)const
{
  if(x=="0-0" || x=="0_0" || x=="O-O" || x=="O_O" || x=="o-o" || x=="o_o"){
    if(cecp_cp.stm()==1){x="e1g1";}else{x="e8g8";}
  }else if(x=="0-0-0" || x=="0_0_0" || x=="O-O-O" || x=="O_O_O" || x=="o-o-o" || x=="o_o_o"){
    if(cecp_cp.stm()==1){x="e1c1";}else{x="e8c8";}
  }
  return x;
}


