/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"clock.h"
#include<iostream>
#include<iomanip>
#include<algorithm>
#include<sstream>
using std::cout;
using std::endl;
using std::setfill;
using std::setw;
using std::count;
using std::istringstream;
using std::ostringstream;


void Clock::reset()
{
  tim.stop();
  m_moves=0;
  set_time(m_base);
}

void Clock::set_movestogo(int x)
{
  m_moves=m_mps-x;
}

void Clock::set_moves(int x)
{
  m_moves=x;
}

int Clock::get_moves()
{
  return m_moves;
}

int Clock::get_movestogo()const
{
  return m_mps-m_moves;
}

Duration Clock::get_inc()const
{
  return m_inc;
}

Duration Clock::get_time()
{
  dura d{tim.total_time().milliseconds()};
  Duration du{d};
  return m_time-du;
}

void Clock::set_time(Duration x)
{
  m_time=x;
  tim.reset();
}

void Clock::start_clock()
{
  tim.start();
}

void Clock::move_and_stop_clock()
{
  tim.stop();
  m_moves++;
  if(m_moves==m_mps){m_time+=m_base;m_moves=0;}
  m_time+=m_inc;
}

void Clock::stop_clock()
{
  tim.stop();
}

string Clock::get_level()
{
  ostringstream oss;
  oss << m_mps << " ";
  int mm=m_base.minutes();
  int ss=m_base.seconds();
  oss << mm;
  if(ss!=0){oss << ":" << setfill('0') << setw(2) << ss << " ";}
  oss << m_inc.seconds();
  return oss.str();
}

Level Clock::level()
{
  Level r;
  r.set_mps(m_mps);
  r.set_base(m_base);
  r.set_inc(m_inc);
  return r;
}

void Clock::set_level(Level x)
{
  m_mps=x.get_mps();
  m_inc=x.get_inc();
  m_base=x.get_base();
  m_moves=0;
  set_time(m_base); 
}

void Clock::set_level(int mps,string base,int increment)
{
  m_mps=mps;

  int minutes{0},seconds{0};
  int x=count(base.begin(),base.end(),':');
  for(auto& c : base){if(c<'0'||c>'9'){c=' ';}}
  istringstream istr(base);
  istr >> minutes;
  if(x>0){istr >> seconds;}
  dura d{(minutes*60+seconds)*1000};
  Duration du{d};
  m_base=du;
  dura inc{increment*1000};
  Duration incu{inc};
  m_inc=incu;
  m_moves=0;
  set_time(m_base); 
}

