/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"clocks.h"
#include<iostream>
#include<sstream>
#include<algorithm>
using std::istringstream;
using std::ostringstream;
using std::cout;
using std::endl;
using std::count;


Clocks::Clocks()
{
  m_white_moves=m_black_moves=0;
  m_green.set_rgba(0.0,1.0,0.0);
  m_white.set_rgba(1.0,1.0,1.0);
  m_grey.set_grey(0.80);

  m_label_white.set_text("White: ");
  m_label_white.set_alignment(Gtk::ALIGN_END,Gtk::ALIGN_CENTER);
  m_label_white.set_width_chars(7);
  m_label_white.override_background_color(m_grey);
  pack_start(m_label_white,Gtk::PACK_SHRINK);

  m_white_clock.set_text("60:00");
  m_white_clock.set_alignment(0.5);
  m_white_clock.set_width_chars(7);
  m_white_clock.override_background_color(m_white);
  pack_start(m_white_clock,Gtk::PACK_SHRINK);

  m_label_black.set_text("Black: ");
  m_label_black.set_alignment(Gtk::ALIGN_END,Gtk::ALIGN_CENTER);
  m_label_black.set_width_chars(7);
  m_label_black.override_background_color(m_grey);
  pack_start(m_label_black,Gtk::PACK_SHRINK);

  m_black_clock.set_text("60:00");
  m_black_clock.set_alignment(0.5);
  m_black_clock.set_width_chars(7);
  m_black_clock.override_background_color(m_white);
  pack_start(m_black_clock,Gtk::PACK_SHRINK);


  white_to_move=false;
  black_to_move=false;
}


Clocks::~Clocks()
{
}


void Clocks::reset()
{
  m_white_moves=m_black_moves=0;
  m_white_centiseconds=m_base;
  m_black_centiseconds=m_base;
  tim_white.reset();
  tim_black.reset();
}

void Clocks::set_white_to_move()
{
  m_label_white.override_background_color(m_green);
  m_label_black.override_background_color(m_grey);
}

void Clocks::set_black_to_move()
{
  m_label_white.override_background_color(m_grey);
  m_label_black.override_background_color(m_green);
}

void Clocks::set_white_clock(int x)
{
  m_white_centiseconds=x;
  tim_white.reset();
  if(white_to_move){tim_white.start();}
}

void Clocks::set_black_clock(int x)
{
  m_black_centiseconds=x;
  tim_black.reset();
  if(black_to_move){tim_black.start();}
}

void Clocks::start_white_clock()
{
  white_to_move=true;
  tim_white.start();
  black_to_move=false;
  tim_black.stop();
  m_black_moves++;
  m_black_centiseconds+=m_inc*100;
  if(m_black_moves==m_mps){
    m_black_moves=0;
    m_black_centiseconds+=m_base;
  }
  m_label_white.override_background_color(m_green);
  m_label_black.override_background_color(m_grey);
}

void Clocks::start_black_clock()
{
  white_to_move=false;
  tim_white.stop();
  black_to_move=true;
  tim_black.start();
  m_white_moves++;
  m_white_centiseconds+=m_inc*100;
  if(m_white_moves==m_mps){
    m_white_moves=0;
    m_white_centiseconds+=m_base;
  }
  m_label_white.override_background_color(m_grey);
  m_label_black.override_background_color(m_green);
}

void Clocks::stop_clocks()
{
  m_label_white.override_background_color(m_grey);
  m_label_black.override_background_color(m_grey);
  white_to_move=false;
  black_to_move=false;
  tim_white.stop();
  tim_black.stop();
}

void Clocks::update_clock_images()
{
  {
    ostringstream oss;
    long long t=tim_white.total_time().centiseconds();
    t=m_white_centiseconds-t;
    t/=100;                         // seconds
    if(t<m_white_steady_time || t>m_white_steady_time+1){
      m_white_steady_time=t;
      int minutes=t/60;
      int seconds=t%60;
      oss << minutes << ":" << seconds;
      m_white_time=oss.str();
      m_white_clock.set_text(m_white_time);
    }
  }
  
  {    
    string txt=m_black_clock.get_text();
    for(auto& c : txt){if(c==':'){c=' ';}}
    istringstream iss(txt);
    int mm,ss;iss >> mm >> ss;
    ss=mm*60+ss;
    
    ostringstream oss;
    long long t=tim_black.total_time().centiseconds();
    t=m_black_centiseconds-t;
    t/=100;                         // seconds
    if(t<m_black_steady_time || t>m_black_steady_time+1){
      m_black_steady_time=t;
      int minutes=t/60;
      int seconds=t%60;
      oss << minutes << ":" << seconds;
      m_black_time=oss.str();
      m_black_clock.set_text(m_black_time);
    }
  }
}




void Clocks::set_level(int mps,string base,float increment)
{
  m_mps=mps;
  m_inc=increment;
  m_white_moves=0;
  m_black_moves=0;

  int minutes{0},seconds{0};
  int x=count(base.begin(),base.end(),':');
  for(auto& c : base){if(c<'0'||c>'9'){c=' ';}}
  istringstream istr(base);
  istr >> minutes;
  if(x>0){istr >> seconds;}
  m_base=(minutes*60+seconds)*100;
  set_white_clock(m_base);    // in centiseconds
  set_black_clock(m_base);
}

