/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef ENGVAR_H
#define ENGVAR_H

#include<vector>
#include<map>
#include<string>
#include"option.h"
#include"clock.h"
#include"../engine/movegen/move.h"
using std::vector;
using std::string;
using std::map;


enum class EngineStatus{idle,thinking,pondering};


class EngineVariables
{
 private:
  //-------------------- engine status: ----------------------
  // mode:
  Clock m_clock;
  bool m_white{true};
  bool m_forced_mode{true};
  EngineStatus m_status{EngineStatus::idle};

  // INSTALLATION:
  bool m_uci;
  string m_enginename;
  string m_filename;
  string m_params;
  
  
  //------------------- engine features / options: -----------
  // features:
  bool m_ping;
  bool m_setboard;
  bool m_playother;
  bool m_san;
  bool m_usermove;
  bool m_time;
  bool m_draw;
  bool m_sigint;
  bool m_sigterm;
  bool m_reuse;
  bool m_analyze;
  string m_myname;
  string m_author;
  string m_variants;
  bool m_colors;
  bool m_ics;
  bool m_name;
  bool m_pause;
  bool m_nps;
  bool m_debug;
  bool m_memory;
  bool m_smp;
  string m_egt;
  int m_done;
  // options:
  map<string,Option> m_option;
  
 public:
  EngineVariables();
  ~EngineVariables();

  // set features:
  void set_default_values();
  void set_ping(bool);
  void set_setboard(bool);
  void set_playother(bool);
  void set_san(bool);
  void set_usermove(bool);
  void set_time(bool);
  void set_draw(bool);
  void set_sigint(bool);
  void set_sigterm(bool);
  void set_reuse(bool);
  void set_analyze(bool);
  void set_myname(string);
  void set_author(string);
  void set_variants(string);
  void set_colors(bool);
  void set_ics(bool);
  void set_name(bool);
  void set_pause(bool);
  void set_nps(bool);
  void set_debug(bool);
  void set_memory(bool);
  void set_smp(bool);
  void set_egt(string);
  void add_option(Option);
  void set_options(map<string,Option>);
  void set_done(int);

  // check features:
  bool ping()const;
  bool setboard()const;
  bool playother()const;
  bool san()const;
  bool usermove()const;
  bool time()const;
  bool draw()const;
  bool sigint()const;
  bool sigterm()const;
  bool reuse()const;
  bool analyze()const;
  string myname()const;
  string author()const;
  string variants()const;
  bool colors()const;
  bool ics()const;
  bool name()const;
  bool pause()const;
  bool nps()const;
  bool debug()const;
  bool memory()const;
  bool smp()const;
  string egt()const;
  map<string,Option> get_options()const;
  int done()const;

  // set || check status:
  bool forced_mode()const;
  bool white()const;
  void set_forced_mode(bool);
  void set_white(bool);
  EngineStatus get_status()const;
  void set_status(EngineStatus);
  bool readyok()const;
  void set_readyok(bool);
  void clear();

  // set || check clock:
  void set_time(Duration);
  Duration get_time();
  Duration get_inc()const;
  int get_movestogo()const;
  void start_clock();
  void stop_clock();
  void reset_clock();
  void move_and_stop_clock();
  void set_level(int,string,int);
  void set_level(Level);
  Level level();
  string get_level();

  // installation:
  void set_uci(bool);
  bool uci()const;
  void set_params(string);
  string& get_params();
  void set_filename(string);
  string& get_filename();
  void set_enginename(string);
  string& get_enginename();
};



#endif
