/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include<iostream>
#include<fstream>
#include<stdexcept>
#include<utility>
#include<sstream>
#include<cassert>
#include"sanmove.h"
#include"game.h"
using namespace std;


Game::Game()
{
  tbs.transpositiontable().set_size(0);
  clear();
}

Game::Game(string sgame)
{
  tbs.transpositiontable().set_size(0);
  *this=sgame;
}

Game& Game::operator=(const string& sgame)
{
  clear();
  try{
    parse_pgn(sgame);
  }
  catch(...){
    std::cerr << sgame << endl;
    return *this;
  }
  m_current={0,0};
  cp.fen(m_initial_fen);
  return *this;
}

void Game::clear()
{
  m_tag_name.clear();
  m_tag.clear();
  m_mse.clear();
  m_mse.push_back(vector<MoveSectionEntry>());
  m_setup=false;
  m_current={0,0};
  m_result="*";
  m_initial_color=white;
  m_initial_movenumber=1;
  cp.newgame();
  m_initial_fen=cp.fen();
}

bool Game::empty()const
{
  if(m_tag.size()==0){return true;}else{return false;}
}

void Game::parse_pgn(string sgame)
{
  //-------------------------- separate tag pair section and movetext section: -------------------
  //
  istringstream iss(sgame);
  ostringstream oss1,oss2;
  string sline;
  while(sline.empty() && !iss.eof()){
    getline(iss,sline);
  }
  while(!iss.eof()){
    if(sline[0]!='%'){
      if(sline.find("[")==string::npos){break;}
      else{
	oss1 << sline << endl;
      }
    }
    getline(iss,sline);
  }
  m_tag_pair_section=oss1.str();

  sline.clear();
  while(sline.empty() && !iss.eof()){
    getline(iss,sline);
  }
  while(!iss.eof()){
    if(sline.size()>0 && sline[0]!='%'){
      oss2 << sline << endl;
    }
    getline(iss,sline);
  }

  MoveSectionEntry mse;
  mse.entry=oss2.str();
  mse.type=mse_t::unknown;
  m_mse[0].push_back(mse);

  
  //-------------------------- parse tags: -------------------------------------------------------
  //
  m_tag_name.clear();
  m_tag.clear();
  string tag_name,tag_value;
  unsigned i{0};
  while(i<m_tag_pair_section.size()){
    tag_name.clear();
    tag_value.clear();
    while(i<m_tag_pair_section.size() && m_tag_pair_section[i]!='['){i++;}
    i++;
    while(i<m_tag_pair_section.size() && m_tag_pair_section[i]!=' ' && m_tag_pair_section[i]!='\"'){
      tag_name+=m_tag_pair_section[i++];
    }
    while(i<m_tag_pair_section.size() && (m_tag_pair_section[i]==' ' || m_tag_pair_section[i]=='\"')){i++;}
    while(i<m_tag_pair_section.size() && m_tag_pair_section[i]!='\"'){
      tag_value+=m_tag_pair_section[i++];
    }
    while(i<m_tag_pair_section.size() && (m_tag_pair_section[i]==' ' || m_tag_pair_section[i]==']')){i++;}
    
    if(!tag_name.empty()){
      m_tag_name.push_back(tag_name);
      m_tag[tag_name]=tag_value;
      if(tag_name=="SetUp"){
	m_setup=true;
      }else if(tag_name=="FEN"){
	string s1,s2,s3,s4,s5;
	m_initial_fen=tag_value;
	istringstream istr(m_initial_fen);
	istr >> s1 >> s2 >> s3 >> s4 >> s5 >> m_initial_movenumber;
	if(s2=="w"){m_initial_color=white;}else{m_initial_color=black;}
	cp.fen(m_initial_fen);
      }else if(tag_name=="Result"){
	m_result=m_tag[tag_name];
      }
      
    }
    i++;
  }

  
  //-------------------------- separate subvariations: -------------------------------------------
  //
  for(unsigned i=0;i<m_mse.size();i++){
    string movetext_section=m_mse[i][0].entry;
    MoveSectionEntry mse;
    int subvariation_depth{0};
    string subvariation,movetext;
    bool comment{false},semicolon_comment{false};
    for(auto c : movetext_section){
      if(c=='{' && !comment && !semicolon_comment){
	comment=true;
      }else if(c=='}' && comment){
	comment=false;
      }else if(c==';' && !comment && !semicolon_comment){
	semicolon_comment=true;
      }else if(c=='\n' && semicolon_comment){
	semicolon_comment=false;
      }else if(c=='(' && !comment && !semicolon_comment){
	subvariation_depth++;
	if(subvariation_depth==1)continue;
      }else if(c==')' && !comment && !semicolon_comment){
	subvariation_depth--;
	if(subvariation_depth==0 && !subvariation.empty()){
	  m_mse.push_back(vector<MoveSectionEntry>());
	  mse.type=mse_t::unknown;
	  mse.entry=subvariation;
	  m_mse.back().push_back(mse);
	  subvariation.clear();
	  movetext+=" (" + to_string(m_mse.size()-1) + ") ";
	  continue;
	}
      }
      if(subvariation_depth>0){subvariation+=c;}else{movetext+=c;}
    }
    m_mse[i][0].entry=movetext;
  }
  
  
  //-------------------------- separate comments: ------------------------------------------------
  //
  {
    MoveSectionEntry mse;
    string s;
    bool comment{false},semicolon_comment{false};
    for(unsigned i=0;i<m_mse.size();i++){    
      string movetext_section=m_mse[i][0].entry;
      m_mse[i].pop_back();
      for(auto c : movetext_section){
	if(c=='{' && !comment && !semicolon_comment){
	  if(!s.empty()){
	    mse.entry=s;
	    mse.type=mse_t::unknown;
	    m_mse[i].push_back(mse);
	    s.clear();
	  }
	  comment=true;
	}else if(c=='}' && comment){
	  if(!s.empty()){
	    mse.entry=s;
	    mse.type=mse_t::comment;
	    m_mse[i].push_back(mse);
	    s.clear();
	  }
	  comment=false;
	}else if(c==';' && !comment && !semicolon_comment){
	  semicolon_comment=true;
	}else if(c=='\n' && semicolon_comment){
	  if(!s.empty()){
	    mse.entry=s;
	    mse.type=mse_t::comment;
	    m_mse[i].push_back(mse);
	    s.clear();
	  }
	  semicolon_comment=false;
	}else{
	  if(comment && c=='\n'){c=' ';}
	  s+=c;
	}
      }
    
      if(!s.empty()){
	mse.entry=s;
	if(comment || semicolon_comment){
	  mse.type=mse_t::comment;
	}else{
	  mse.type=mse_t::unknown;
	}
	m_mse[i].push_back(mse);
	s.clear();
      }
    }
    
  }

  
  //-------------------------- parse moves: ------------------------------------------------------
  //
  {
    for(unsigned i=0;i<m_mse.size();i++){
      bool type_unknown=true;
      while(type_unknown){
	type_unknown=false;
	for(auto j=m_mse[i].begin();j!=m_mse[i].end();j++){
	  if(j->type==mse_t::unknown){
	    type_unknown=true;
	    string movetext=j->entry;
	    auto it=j;
	    it=m_mse[i].erase(j);
	    
	    vector<MoveSectionEntry> mse=extract_moves(movetext);
	    
	    m_mse[i].insert(it,mse.begin(),mse.end());
	    break;
	  }
	}
      }
    }
  }

  //-------------------------- set color and movenumber: -----------------------------------------
  //
  {
    set_color_and_movenumber(0,m_initial_color,m_initial_movenumber);
  }

}

string Game::dump()
{
  string r;
  for(auto t : m_tag_name){
    r+="[";
    r+=t+" \""+m_tag[t]+"\"]\n";
  }
  r+="\n";
  for(unsigned i=0;i<m_mse.size();i++){
    r+="Variation " + to_string(i) + ":\n";
    for(auto t : m_mse[i]){
      if(t.type==mse_t::comment){
	r+="{";
	r+=t.entry;
	r+="} ";
      }else if(t.type==mse_t::subvariation){
	r+=t.entry;
	r+=" ";
      }else{
	r+=t.entry;
	r+=" ";
      }
    }
    r+="\n";
  }

  r+=get_current()+"\n";
  return r;
}

void Game::set_tag(string name,string value)
{
  if(m_current.first>0){return;}
  if(m_tag.count(name)==0){
    m_tag_name.push_back(name);
  }
  m_tag[name]=value;  
  if(name=="FEN"){
    string s1,s2,s3,s4,s5;
    m_initial_fen=value;
    istringstream istr(value);
    istr >> s1 >> s2 >> s3 >> s4 >> s5 >> m_initial_movenumber;
    if(s2=="w"){m_initial_color=white;}else{m_initial_color=black;}
    cp.fen(value);
  }

}

string Game::get_tag(string name)
{
  string r;
  if(m_tag.count(name)>0){r=m_tag[name];}
  return r;
}

Move Game::get_move()
{
  Move m;
  auto v=m_current.first;
  auto i=m_current.second;
  while(i<m_mse[v].size() && m_mse[v][i].type!=mse_t::move){i++;}
  if(i<m_mse[v].size() && m_mse[v][i].type==mse_t::move){m=SanMove(cp,m_mg,m_mse[v][i].entry);}
  return m;
}

bool Game::find_subvariation(Move m)
{
  auto v=m_current.first;
  auto i=m_current.second;
  while(i<m_mse[v].size() && m_mse[v][i].type!=mse_t::move){i++;}
  assert(m_mse[v][i].type==mse_t::move);
  SanMove sm(cp,m_mg,m);
  if(i<m_mse[v].size() && sm.str()==m_mse[v][i].entry){
    m_current.first=v;
    m_current.second=i;
    return true;
  }
  i++;
  while(i<m_mse[v].size() && m_mse[v][i].type!=mse_t::move){
    if(m_mse[v][i].type==mse_t::subvariation){
      string sv=m_mse[v][i].entry;
      for(auto& c: sv){if(c=='(' || c==')')c=' ';}
      istringstream iss(sv);
      unsigned iv;iss>>iv;
      unsigned j{0};
      while(j<m_mse[iv].size() && m_mse[iv][j].type!=mse_t::move){j++;}
      if(m_mse[iv][j].type==mse_t::move){
	if(m_mse[iv][j].entry==sm.str()){
	  m_current.first=iv;
	  m_current.second=j;
	  return true;
	}
      }
    }
    i++;
  }
  return false;
}

void Game::add_move(Move m)
{
  auto v=m_current.first;
  auto i=m_current.second;

  //------------------ at end of current variation? --------------------
  //
  bool at_end{false};
  if(is_last_move()){at_end=true;}
  while(i<m_mse[v].size() && m_mse[v][i].type!=mse_t::move){i++;}
  if(i==m_mse[v].size()){
    at_end=true;
  }else{
    if(is_last_move()){at_end=true;}
  }
  //--------------------------------------------------------------------

  if(i<m_mse[v].size()){i++;}
  
  if(at_end){
    MoveSectionEntry mse;
    mse.type=mse_t::move;
    mse.entry=SanMove(cp,m_mg,m).str();
    m_mse[v].push_back(mse);
    m_current.second=m_mse[v].size();
    cp.make_move(m);
  }else{
    if(find_subvariation(m)){
      m_current.second++;
      cp.make_move(m);
    }else{
      unsigned uv=m_mse.size();
      string suv="("+to_string(uv)+")";
      MoveSectionEntry mse;
      mse.type=mse_t::subvariation;
      mse.entry=suv;
      m_mse[v].insert(m_mse[v].cbegin()+i,mse);
      m_mse.push_back(vector<MoveSectionEntry>());
      mse.type=mse_t::move;
      mse.entry=SanMove(cp,m_mg,m).str();
      m_mse[uv].push_back(mse);
      m_current.first=uv;
      m_current.second=m_mse[uv].size();
      cp.make_move(m);
    }
  }
  
}

void Game::add_comment(string s)
{
  if(s.empty()){return;}
  auto v=m_current.first;
  auto i=m_current.second;
  MoveSectionEntry mse;
  mse.type=mse_t::comment;
  mse.entry=s;

  if(i==m_mse[v].size()){
    if(i>0){
      if(m_mse[v][i-1].type==mse_t::comment){
	m_mse[v][i-1]=mse;
      }else{
	m_mse[v].push_back(mse);
      }
    }else{
      m_mse[v].push_back(mse);
    }
  }else if(i==0){
    if(m_mse[v][0].type==mse_t::comment){
      m_mse[v][0]=mse;
    }else{
      m_mse[v].insert(m_mse[v].cbegin(),mse);
      m_current.second++;
    }
  }else if(m_mse[v][i].type!=mse_t::move){
    while(i>0 && m_mse[v][i].type!=mse_t::move){i--;}
    if(m_mse[v][i].type==mse_t::move){i++;}
    if(m_mse[v][i].type==mse_t::comment){
      m_mse[v][i]=mse;
    }else{
      m_mse[v].insert(m_mse[v].cbegin()+i,mse);
    }
  }else{
    i--;
    while(i>0 && m_mse[v][i].type!=mse_t::move){i--;}
    if(m_mse[v][i].type==mse_t::move){i++;}
    if(m_mse[v][i].type==mse_t::comment){
      m_mse[v][i]=mse;
    }else{
      m_mse[v].insert(m_mse[v].cbegin()+i,mse);
    }
  }
}

string Game::get_comment()
{
  auto v=m_current.first;
  auto i=m_current.second;
  if(m_mse[v].size()==0){return "";}
  if(m_mse[v].size()>0){
    while(i>0 && m_mse[v][i-1].type!=mse_t::move){i--;}
    while(i<m_mse[v].size() && m_mse[v][i].type!=mse_t::comment && m_mse[v][i].type!=mse_t::move){i++;}
    if(i<m_mse[v].size()){
      if(m_mse[v][i].type==mse_t::comment){
	return m_mse[v][i].entry;
      }else{
	return "";
      }
    }
  }
  return "";
}

void Game::make_opening(unsigned int n)
{
  if(m_mse.size()>1){
    m_mse.erase(m_mse.cbegin()+1,m_mse.cend());
  }
  if(m_mse.size()>0){
    vector<MoveSectionEntry> vmse;
    for(auto v : m_mse[0]){
      if(v.type==mse_t::move){
	vmse.push_back(v);
      }
    }
    while(vmse.size()>n){vmse.pop_back();}
    MoveSectionEntry mse;
    mse.type=mse_t::termination;
    mse.entry="*";
    vmse.push_back(mse);
    m_mse[0]=vmse;
  }

  map<string,string> tags;
  tags["Event"]="?";
  tags["Site"]="?";
  tags["Date"]="????.??.??";
  tags["Round"]="1";
  tags["White"]="?";
  tags["Black"]="?";
  tags["Result"]="*";
  m_tag=tags;
  while(m_tag_name.size()>7){m_tag_name.pop_back();}
  m_result="*";
}

string Game::get_result()const
{
  return m_result;
}

void Game::set_result(string r)
{
  m_result=r;
}

string Game::get_title()
{
  string r=m_tag["White"]+" "+m_tag["Black"];
  return r;
}

string Game::get_current()const
{
  ostringstream oss;
  oss << "{" << m_current.first << "," << m_current.second << "}";
  return oss.str();
}

uint64_t Game::get_hash()const
{
  return cp.get_hash();
}

string Game::epd_str()
{
  int posnbr{0};
  string position,comment,white_player,black_player;
  Move m;
  ostringstream oss;
  white_player=get_tag("White");
  black_player=get_tag("Black");
  for(auto& c : white_player){c=c==' '?'_':c;}
  for(auto& c : black_player){c=c==' '?'_':c;}
  
  position=get_first_position();
  m=get_move();
  while(position!=""){
    istringstream iss(position);
    string s1,s2,s3,s4;
    iss >> s1 >> s2 >> s3 >> s4;
    position=s1+" "+s2+" "+s3+" "+s4+" ";
    oss << position << "id \"Round " << get_tag("Round") << " " << white_player << " - " << black_player << " " << ++posnbr << "\"; "
	<< (m!=NullMove()?("sm " + m.algebraic() + "; "):"")
	<< (comment!=""?"c0 \"" + comment + "\"; ":"")
	<< endl;

    position=get_next_position();
    comment=get_comment();
    m=get_move();
  }
  return oss.str();
}

string Game::str()
{
  return basic_str(false);
}

string Game::comment_str()
{
  return basic_str(true);
}

string Game::basic_str(bool comms)
{
  set_color_and_movenumber(0,m_initial_color,m_initial_movenumber);
  string game;
  game=basic_str_tags() + basic_str_movesection({0,0},comms) + " " + m_result + "\n\n";
  
  //----------------- adjust the length of the lines: ------------------------
  int linelength=0;
  for(size_t i=0;i<game.size();i++){
    if(game[i]!='\n'){linelength++;}else{linelength=0;}
    if(linelength==80){
      int j=i;
      linelength=0;
      while(game[j]!=' '){
	linelength++;
	j--;
      }
      game[j]='\n';
    }
  }
  //--------------------------------------------------------------------------

  return game;
}

string Game::basic_str_tags()
{
  string r;
  for(auto t : m_tag_name){
    r+="["+t+" \""+m_tag[t]+"\"]\n";
  }
  r+="\n";
  return r;
}

string Game::basic_str_movesection(pair<unsigned,unsigned> current,bool comms)
{
  auto v=current.first;
  ostringstream oss;
  for(unsigned i=0;i<m_mse[v].size();i++){
    if(m_mse[v][i].type==mse_t::move){
      if(m_mse[v][i].color==white){
	oss << m_mse[v][i].movenumber << ". ";
      }else if(i==0){
	oss << m_mse[v][i].movenumber << "... ";
      }else if(comms && (m_mse[v][i-1].type==mse_t::comment || m_mse[v][i-1].type==mse_t::subvariation)){
	oss << m_mse[v][i].movenumber << "... ";
      }
      oss << m_mse[v][i].entry;
      if(i<m_mse[v].size()-1){oss << " ";}
    }
    if(comms && m_mse[v][i].type==mse_t::comment){
      oss << "{" << m_mse[v][i].entry << "}";
      if(i<m_mse[v].size()-1){oss << " ";}
    }
    if(comms && m_mse[v][i].type==mse_t::subvariation){
      string sv=m_mse[v][i].entry;
      for(auto& c: sv){if(c=='(' || c==')')c=' ';}
      istringstream iss(sv);
      int iv;iss>>iv;
      oss << "(" << basic_str_movesection({iv,0},comms) << ")";
      if(i<m_mse[v].size()-1){oss << " ";}
    }
    if(m_mse[v][i].type==mse_t::nag){
      oss << m_mse[v][i].entry;
      if(i<m_mse[v].size()-1){oss << " ";}
    }
    if(m_mse[v][i].type==mse_t::termination){
      /*
      oss << m_mse[v][i].entry;
      if(i<m_mse[v].size()-1){oss << " ";}
      */
    }
  }
  return oss.str();
}

void Game::set_color_and_movenumber(unsigned variation,int color,int movenumber)
{
  if(variation>=m_mse.size())return;
  int old_color{m_initial_color},old_movenumber{m_initial_movenumber};
  for(unsigned i=0;i<m_mse[variation].size();i++){
    if(m_mse[variation][i].type==mse_t::move){
      m_mse[variation][i].color=color;
      m_mse[variation][i].movenumber=movenumber;
      old_color=color;
      old_movenumber=movenumber;
      color=color==white?black:white;
      if(color==white){movenumber++;}
    }
    if(m_mse[variation][i].type==mse_t::subvariation){
      string sv=m_mse[variation][i].entry;
      for(auto& c: sv){if(c=='(' || c==')')c=' ';}
      istringstream iss(sv);
      int iv;iss>>iv;
      set_color_and_movenumber(iv,old_color,old_movenumber);
    }
  }
}

bool Game::is_first_move()const
{
  bool r{false};
  auto v=m_current.first;
  int i=m_current.second;
  if(m_mse[v][i].type!=mse_t::move){return false;}
  i--;
  while(i>=0 && m_mse[v][i].type!=mse_t::move){
    i--;
  }
  if(i<0){r=true;}
  return r;
}

bool Game::is_last_move()const
{
  bool r{false};
  auto v=m_current.first;
  auto i=m_current.second;
  
  if(i>=m_mse[v].size()){return false;}
  if(m_mse[v][i].type!=mse_t::move){return false;}
  while(i<m_mse[v].size() && m_mse[v][i].type!=mse_t::move){i++;}
  if(i==m_mse[v].size()){r=true;}
  return r;
}


//----------------------------------------------------------------
//
string Game::get_first_position()
{
  m_current={0,0};
  cp.fen(m_initial_fen);
  return cp.fen();
}

string Game::get_last_position()
{
  auto v=m_current.first;
  auto i=m_current.second;
  SanMove sm(cp,m_mg);
  while(i<m_mse[v].size()){
    if(m_mse[v][i].type==mse_t::move){
      sm=m_mse[v][i].entry;
      cp.make_move(sm);
    }
    i++;
  }
  m_current.second=m_mse[v].size();
  return cp.fen();
}

string Game::get_previous_position()
{
  auto v=m_current.first;
  auto& i=m_current.second;

  if(cp.moves_history().size()>0){
    if(i==m_mse[v].size()){
      i--;
      while(i>0 && m_mse[v][i].type!=mse_t::move){i--;}
    }else{
      if(i>0 && m_mse[v][i].type==mse_t::move){i--;}
      while(i>0 && m_mse[v][i].type!=mse_t::move){i--;}
    }
    if(!is_first_move() || v==0){
      cp.unmake_move();
      return cp.fen();
    }
  
    for(unsigned x=0;x<m_mse.size();x++){
      for(unsigned y=0;y<m_mse[x].size();y++){
	if(m_mse[x][y].type==mse_t::subvariation){
	  string sv=m_mse[x][y].entry;
	  for(auto& c: sv){if(c=='(' || c==')')c=' ';}
	  istringstream iss(sv);
	  unsigned iv;iss>>iv;
	  if(iv==m_current.first){	  
	    m_current.first=x;
	    m_current.second=y;
	    return get_previous_position();
	  }
	}
      }
    }
    throw logic_error("Game::get_previous_position():  game is corrupt");  
  }else{
    return "";
  }
}

string Game::get_next_position()
{
  auto v=m_current.first;
  auto& i=m_current.second;
  if(i>=m_mse[v].size()){return "";}
  SanMove sm(cp,m_mg);
  while(i<m_mse[v].size()){
    if(m_mse[v][i].type==mse_t::move){
      sm=m_mse[v][i].entry;
      cp.make_move(sm);
      i++;
      break;
    }
    i++;
    if(i>=m_mse[v].size()){return "";}
  }
  return cp.fen();
}


const ChessPosition& Game::chessposition()const
{
  return cp;
}


//---------------------------------------------------------------------------------------------------
//
bool symbolcharacter(char c)
{
  if((c>='A' && c<='Z') || (c>='a' && c<='z') || (c>='0' && c<='9') || c=='_' || c=='-' || c=='+' || c=='#' || c=='=' || c==':'){
    return true;
  }else{
    return false;
  }
}

vector<MoveSectionEntry> extract_moves(string x)
{
  vector<MoveSectionEntry> r;
  for(unsigned i=0;i<x.size();i++){
    if((x[i]>='a' && x[i]<='h') || x[i]=='N' || x[i]=='B' || x[i]=='R' || x[i]=='Q' || x[i]=='K' || x[i]=='O'){
      MoveSectionEntry s;
      unsigned j;
      for(j=0;(i+j)<x.size();j++){
	if(symbolcharacter(x[i+j])){
	  s.entry+=x[i+j];
	}else{
	  break;
	}
      }
      i+=j;
      s.type=mse_t::move;
      r.push_back(s);
    }
    if(x[i]=='$'){
      MoveSectionEntry s;
      s.entry="$";
      for(unsigned j=1;(i+j)<x.size();j++){
	if(x[i+j]>='0' && x[i+j]<='9'){
	  s.entry+=x[i+j];
	}else{
	  break;
	}
      }
      s.type=mse_t::nag;
      r.push_back(s);
    }
    if(x[i]=='*'){
      MoveSectionEntry s;
      s.entry="*";
      s.type=mse_t::termination;
      r.push_back(s);
    }
    if(x[i]=='1' && x[i+1]=='-'){
      MoveSectionEntry s;
      s.entry="1-0";
      s.type=mse_t::termination;
      r.push_back(s);
    }
    if(x[i]=='0' && x[i+1]=='-'){
      MoveSectionEntry s;
      s.entry="0-1";
      s.type=mse_t::termination;
      r.push_back(s);
    }
    if(i<x.size()-4 && x[i]=='1' && x[i+1]=='/'){
      MoveSectionEntry s;
      s.entry="1/2-1/2";
      s.type=mse_t::termination;
      r.push_back(s);
    }
    if(x[i]=='('){
      MoveSectionEntry s;
      for(unsigned j=1;(i+j)<x.size();j++){
	if(x[i+j]!=')'){
	  s.entry+=x[i+j];
	}else{
	  i+=j;
	  break;
	}
      }
      s.type=mse_t::subvariation;
      r.push_back(s);
    }
  }
  return r;
}
