/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"guiengine.h"
#include<thread>
#include<string>
#include<iostream>
#include<iomanip>
#include<fstream>
#include<sstream>
#include<stdexcept>
using std::setw;
using std::istringstream;
using std::ofstream;
using std::ostringstream;
using std::logic_error;
using std::string;
using std::to_string;
using std::thread;
using std::cout;
using std::endl;
using std::flush;

GuiEngine::GuiEngine()
{
  m_tbs.transpositiontable().set_size(0);
}

GuiEngine::~GuiEngine()
{
}

void GuiEngine::connect(int id,LowLevelEngine& lle)
{
  m_engine_id=id;
  m_pengine=&lle;
  m_pcom_engine=&lle.communicator();
  m_engvar=lle.enginevariables();
}

int GuiEngine::get_engine_id()const
{
  return m_engine_id;
}

void GuiEngine::set_log(bool x)
{
  m_log=x;
  m_pengine->set_log(x);
}

bool GuiEngine::good()const
{
  return m_pengine->good();
}

void GuiEngine::set_logstream(ofstream* x)
{
  m_pofs=x;
}

void GuiEngine::set_white(bool x)
{
  m_engvar.set_white(x);
}

bool GuiEngine::white()const
{
  return m_engvar.white();
}

string GuiEngine::myname()const
{
  return m_engvar.myname();
}

void GuiEngine::set_ponder(bool x)
{
  m_ponder=x;
}

void GuiEngine::reset_clock()
{
  m_engvar.reset_clock();
}

void GuiEngine::start_clock()
{
  m_engvar.start_clock();
}

void GuiEngine::move_and_stop_clock()
{
  m_engvar.move_and_stop_clock();
}

bool GuiEngine::uci()const
{
  return m_engvar.uci();
}

void GuiEngine::set_options(Map x)
{
  m_engvar.set_options(x);
  for(auto v : x){
    string s;
    if(m_engvar.uci()){
      s="setoption name ";
      s+=v.second.name();
      if(v.second.value()!=""){s+=" value "+v.second.value();}
    }else{
      s="option ";
      s+=v.second.name();
      string sv=v.second.value();

      //--------------------- special CECP case: memory   (cores not implemented) -------------
      if(v.second.name()=="cecp_memory"){
	s="memory "+sv;
	m_pcom_engine->to_engine(s);
	continue;
      }
      //----------------------------------------------------------------------------------

      if(sv=="true"){sv="1";}else if(sv=="false"){sv="0";}
      if(sv!=""){s+="="+ sv;}
    }

    m_pcom_engine->to_engine(s);
  }
}

Map GuiEngine::get_options()
{
  return m_engvar.get_options();
}

const EngineVariables& GuiEngine::get_enginevariables()const
{
  return m_engvar;
}

void GuiEngine::set_enginevariables(EngineVariables& x)
{
  m_engvar.set_uci(x.uci());
  m_engvar.set_enginename(x.get_enginename());
  m_engvar.set_filename(x.get_filename());
  m_engvar.set_params(x.get_params());

  auto mop=m_engvar.get_options();
  for(auto opt : x.get_options()){
    if(mop.find(opt.first) != mop.end()){
	mop[opt.first].set_name(opt.second.name());
	mop[opt.first].set_value(opt.second.value());
      }else{
	ostringstream oss;oss << "GuiEngine::set_enginevariables() " << opt.first << " unknown";
	throw logic_error(oss.str());
      }
  }
  
  set_options(mop);
}

void GuiEngine::set_opponent(GuiEngine* opp)
{
  m_opponent=opp;
}

void GuiEngine::position(const ChessPosition& cp)
{
  m_cp=cp;
}

void GuiEngine::newgame()
{
  m_cp.newgame();
  reset_clock();
}

void GuiEngine::set_level(const Level& x)
{
  m_engvar.set_level(x);
}

Duration GuiEngine::get_time()   
{
  return m_engvar.get_time();
}

Duration GuiEngine::get_wtime()   
{
  if(white()){return get_time();}
  else{return m_opponent->get_time();}
}

Duration GuiEngine::get_btime()   
{
  if(!white()){return get_time();}
  else{return m_opponent->get_time();}
}

Duration GuiEngine::get_inc()   
{
  return m_engvar.get_inc();
}

int GuiEngine::get_movestogo()   
{
  return m_engvar.get_movestogo();
}

Move GuiEngine::alg2move(string s)
{
  return m_cp.convert_to_move(s);
}
