/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"installdialog.h"
#include<iostream>
#include<sstream>
using std::cout;
using std::endl;
using std::istringstream;
using std::stoi;

InstallDialog::InstallDialog(const Glib::ustring& title,Gtk::Window& w,EngineVariables& ev) : m_ev(ev)
{
  m_grid.set_row_spacing(10);
  m_grid.set_column_spacing(10);
  set_title(title);
  set_transient_for(w);
  p_content=get_content_area();
  p_content->pack_start(m_grid);


  m_name_entry.set_entry(&m_ev.get_enginename());
  m_params_entry.set_entry(&m_ev.get_params());

  
  int i{0},j{0};


    
  //------------------ name / commandline: -------------------------  
  string s="name: ";
  m_name_label.set_text(s);
  s=m_ev.get_enginename();
  m_name_entry.set_text(s);
  m_grid.attach(m_name_label,0,j/4,1,1);
  m_grid.attach(m_name_entry,1,j/4,3,1);
  j+=4;
  s="parameters: ";
  m_params_label.set_text(s);
  s=m_ev.get_params();
  m_params_entry.set_text(s);
  m_grid.attach(m_params_label,0,j/4,1,1);
  m_grid.attach(m_params_entry,1,j/4,3,1);
  
  
  //------------------ checkbuttons: -------------------------------  
  j=j+4 - j%4;
  for(auto& v : m_ev.get_options()){
    if(v.second.type()=="check"){
      string s=v.second.name();
      if(v.second.value()=="true"){
	s+=":  yes";
      }else{
	s+=":  no";
      }
      m_label[i].set_text(s);      
      m_grid.attach(m_label[i],j%4,j/4,1,1);
      j++;
    }
    i++;
  }
  
  //------------------ spinbuttons / comboboxes: ------------------  
  j=j+4 - j%4;
  i=0;
  for(auto& v : m_ev.get_options()){
    if(v.second.type()=="spin" || v.second.type()=="combo"){
      string s=v.second.name() + ":  " + v.second.value();
      m_label[i].set_text(s);
      m_grid.attach(m_label[i],j%4,j/4,1,1);
      j++;
    }
    i++;
  }
  
  //------------------ strings: ------------------------------------  
  j=j+4 - j%4;
  i=0;
  for(auto& v : m_ev.get_options()){
    if(v.second.type()=="string"){
      string s=v.second.name() + ":  " + v.second.value();
      m_label[i].set_text(s);
      m_grid.attach(m_label[i],0,j/4,4,1);
      j+=4;
    }
    i++;
  }


  
  p_content->show_all_children();  
  add_button("OK",1);
  show();
}


InstallDialog::~InstallDialog()
{
}

