/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include <iostream>
#include "installedlist.h"
#include "pgnfile.h"

InstalledList::InstalledList(Gtk::Window& x)
  : m_pw(x), m_VBox(Gtk::ORIENTATION_VERTICAL)
{
  //Add the TreeView, inside a ScrolledWindow, with the button underneath:
  m_ScrolledWindow.add(m_TreeView);

  //Only show the scrollbars when they are necessary:
  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  m_VBox.pack_start(m_ScrolledWindow);

  //Create the Tree model:
  m_refTreeModel = Gtk::ListStore::create(m_Columns);
  m_TreeView.set_model(m_refTreeModel);

  //Add the TreeView's view columns:
  m_TreeView.append_column("ID", m_Columns.m_col_id);
  m_TreeView.append_column("Engine", m_Columns.m_col_engine);
}


InstalledList::~InstalledList()
{
}

void InstalledList::clear()
{
  m_engine_id=0;
  m_refTreeModel->clear();
}

void InstalledList::add_engine(string engine)
{
  Gtk::TreeModel::Row row = *(m_refTreeModel->append());
  row[m_Columns.m_col_id] = ++m_engine_id;
  row[m_Columns.m_col_engine] = engine;
}

string InstalledList::get_selection(InstalledEngines& ie)
{
  clear();
  for(auto eng : ie){
    add_engine(eng.first);
  }


  Gtk::Dialog dialog("Choose Position",m_pw,true);
  dialog.set_border_width(20);
  dialog.set_default_size(400,500);

  //Add response buttons the the dialog:
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  //Prepare content area:
  Gtk::Box* pcontent=dialog.get_content_area();
  pcontent->pack_start(m_VBox);
  pcontent->show_all_children();

  //Show the dialog and wait for a user response:
  int result = dialog.run();

  //Handle the response:
  switch(result)
    {
    case(Gtk::RESPONSE_OK):
      {
	auto iter=m_TreeView.get_selection()->get_selected();
	if(iter){
	  auto row=*iter;
	  m_selection=row[m_Columns.m_col_engine];
	}
	return m_selection;
      }
    case(Gtk::RESPONSE_CANCEL):
      {
	return "";
      }
    default:
      {
	return "";
      }
    }

}
