/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"installengine.h"
#include<iostream>
#include<iomanip>
#include<fstream>
#include<sstream>
#include<string>
#include<stdexcept>
#include<filesystem>
using namespace std::filesystem;
using std::filesystem::path;
using std::cerr;
using std::endl;
using std::string;
using std::ifstream;
using std::ofstream;
using std::istringstream;
using std::runtime_error;


InstalledEngines::InstalledEngines()
{
  path curp=current_path();
  string dir=curp.string();

  //------------------------ homedir: ------------------------------
  //
  string newdir;
  for(auto& c : dir){newdir+=c=='/'?' ':c;}
  istringstream iss(newdir);
  string home;
  string user;
  iss >> home >> user;
  if(home=="home"){
    m_homedir="/" + home + "/" + user + "/.galjoenchess";
  }else{
    m_homedir=dir + "/galjoenchess";
  }
  //
  //----------------------------------------------------------------

  //------------------------ create .galjoenchess: -----------------
  //
  path hd=path(m_homedir);
  if(!exists(hd)){
    if(!create_directory(hd)){
      throw runtime_error("InstalledEngines::InstalledEngines   create .galjoenchess failed");
    }
  }
  //
  //----------------------------------------------------------------

  m_engines_cfg=m_homedir + "/engines.cfg";
  m_engines_old=m_homedir + "/engines.old";
}


InstalledEngines::~InstalledEngines()
{
}

void InstalledEngines::read_engines_cfg()
{
  EngineVariables engvar;
  string sline,s1,s2,engine_name,option_name,option_value;
  ifstream ifs(m_engines_cfg);
  if(ifs.good()){
    getline(ifs,sline);
    while(!ifs.eof()){
      istringstream iss(sline);
      iss >> s1;
      if(s1=="name"){
	getline(iss,engine_name);
	while(engine_name.size()>0 && engine_name.front()==' '){engine_name.erase(0,1);}
	while(engine_name.size()>0 && engine_name.back()==' '){engine_name.pop_back();}
	engvar.clear();
	m_engines[engine_name]=engvar;
	m_engines[engine_name].set_enginename(engine_name);
      }else if(s1=="filename"){
	iss >> s2;
	m_engines[engine_name].set_filename(s2);
      }else if(s1=="command"){
	getline(iss,s2);
	m_engines[engine_name].set_params(s2);
      }else if(s1=="setoption"){
	iss >> s2;
	if(s2=="name"){
	  string s3;
	  iss >> option_name >> s3;
	  while(!iss.eof() && s3!="value"){
	    option_name += " " + s3;
	    s3.clear();
	    iss >> s3;
	  }
	  if(s3=="value"){
	    option_value.clear();
	    iss >> option_value;
	    Option op;
	    op.set_name(option_name);
	    op.set_value(option_value);
	    m_engines[engine_name].add_option(op);
	  }else{
	    // do nothing
	  }
	}
      }
      getline(ifs,sline);
    }
  }
  ifs.close();
}

void InstalledEngines::write_engines_cfg()
{
  try{
    std::filesystem::rename(path(m_engines_cfg),path(m_engines_old));
  }
  catch(...){}
  ofstream ofs(m_engines_cfg);
  if(!ofs.good()){
    throw runtime_error("InstalledEngines::write_engines_cfg()   open file error");
  }
  ofs << "#                       Installed engines" << endl;
  ofs << "#" << endl;
  for(auto engine : m_engines){
    ofs << endl << "name " << engine.first << endl;
    if(engine.second.get_filename()!=""){
      ofs << "filename " << engine.second.get_filename() << endl;
    }
    if(engine.second.get_params()!=""){
      ofs << "params " << engine.second.get_params() << endl;
    }
    for(auto op : engine.second.get_options()){
      ofs << "setoption name " << op.second.name()
	  << " type " << op.second.type()
	  << " value " << op.second.value()
	  << endl;
    }
  }
  ofs.close();
}

void InstalledEngines::install(EngineVariables& x)
{
  string name=x.get_enginename();
  if(name.empty()){
    name=x.myname();
  }
  m_engines[name]=x;
}
