/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef LOWLEVELENGINE_H
#define LOWLEVELENGINE_H

#include<string>
#include<thread>
#include"../engine/communicator.h"
#include"engio.h"
#include"engvar.h"
using std::string;
using std::thread;
using Map=std::map<string,Option>;



class LowLevelEngine
{
 private:
  EngineVariables m_engvar;
  Communicator m_com_engine;
  int m_engine_id;
  string m_engine_filename;

  bool m_log{false};
  bool m_uci{false};
  bool m_good{false};
  Engio* pengine;
  thread m_t;

  
 public:
  LowLevelEngine();
  LowLevelEngine(const LowLevelEngine&)=delete;
  LowLevelEngine& operator=(const LowLevelEngine&)=delete;
  LowLevelEngine(LowLevelEngine&&)=delete;
  LowLevelEngine& operator=(LowLevelEngine&&)=delete;
  virtual ~LowLevelEngine();

  void load(int,string);
  void reload();
  void set_protocol();
  void kill();
  void set_log(bool);
  
  bool uci()const{return m_uci;}
  bool good()const{return pengine->good();}
  EngineVariables& enginevariables(){return m_engvar;}
  Communicator& communicator(){return m_com_engine;}
};


#endif
