/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"option.h"
#include<sstream>
#include<iostream>
#include<stdexcept>
using std::endl;
using std::runtime_error;
using std::istringstream;



Option::Option(string x)
{
  x+=" "; // avoid eof after reading last token
  istringstream iss(x);
  string token;
  iss >> token;
  if(token!="option"){throw runtime_error("Option:Option  argument not an option");}
  iss >> token;
  while(!iss.eof()){
    if(token=="name"){
      iss >> token;
      while(!iss.eof()){
	m_name+=token+" ";
	iss >> token;
	if(reserved(token)){break;}
      }
    }else if(token=="type"){
      iss >> token;
      m_type=token;
      iss >> token;
    }else if(token=="default"){
      iss >> token;
      while(!iss.eof()){
	m_default+=token+" ";
	iss >> token;
	if(reserved(token)){break;}
      }      
    }else if(token=="min"){
      iss >> token;
      while(!iss.eof()){
	m_min+=token+" ";
	iss >> token;
	if(reserved(token)){break;}
      }      
    }else if(token=="max"){
      iss >> token;
      while(!iss.eof()){
	m_max+=token+" ";
	iss >> token;
	if(reserved(token)){break;}
      }      
    }else if(token=="value"){
      iss >> token;
      while(!iss.eof()){
	m_value+=token+" ";
	iss >> token;
	if(reserved(token)){break;}
      }            
    }else if(token=="var"){
      iss >> token;
      while(!iss.eof()){
	m_var.push_back(token);
	iss >> token;
	if(reserved(token)){break;}
      }      
    }
  }
  
  while(m_name.size()>0 && m_name.back()==' '){m_name.pop_back();}
  while(m_type.size()>0 && m_type.back()==' '){m_type.pop_back();}
  while(m_default.size()>0 && m_default.back()==' '){m_default.pop_back();}
  while(m_min.size()>0 && m_min.back()==' '){m_min.pop_back();}
  while(m_max.size()>0 && m_max.back()==' '){m_max.pop_back();}
  while(m_value.size()>0 && m_value.back()==' '){m_value.pop_back();}

  m_value=m_default;
}

string Option::name()const
{
  return m_name;
}

string Option::type()const
{
  return m_type;
}

string Option::default_value()const
{
  return m_default;
}

string Option::minimum_value()const
{
  return m_min;
}

string Option::maximum_value()const
{
  return m_max;
}

vector<string> Option::var()const
{
  return m_var;
}

string Option::value()const
{
  return m_value;
}

bool Option::changed()const
{
  return m_changed;
}

bool Option::reserved(const string& x)const
{
  return (x=="name" || x=="value" || x=="type" || x=="default" || x=="min" || x=="max" || x=="var" || x=="check" || x=="spin" || x=="combo" || x=="button" || x=="string");
}

void Option::set_name(string x)
{
  m_name=x;
}

void Option::set_type(string x)
{
  m_type=x;
}

void Option::set_default(string x)
{
  m_default=x;
}

void Option::set_min(string x)
{
  m_min=x;
}

void Option::set_max(string x)
{
  m_max=x;
}

void Option::add_var(string x)
{
  m_var.push_back(x);
}

void Option::set_value(string x)
{
  m_value=x;
}

void Option::set_changed(bool x)
{
  m_changed=x;
}
