/*

Copyright (C) 2019 Werner Taelemans, Belgium

This file is part of Galjoen.

Galjoen is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; version 3 of the License.

Galjoen is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Galjoen; see the file COPYING.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include"optionsdialog.h"
#include<iostream>
#include<sstream>
using std::cout;
using std::endl;
using std::istringstream;
using std::stoi;

OptionsDialog::OptionsDialog(const Glib::ustring& title,Gtk::Window& w,vector<Option>& option) : m_option(option)
{
  for(size_t i=0;i<max_options && i<m_option.size();i++){
    m_checkbutton[i].set_opt(&m_option[i]);
    m_button[i].set_opt(&m_option[i]);
    m_spinbutton[i].set_opt(&m_option[i]);
    m_combobox[i].set_opt(&m_option[i]);
    m_entry[i].set_opt(&m_option[i]);
  }
  m_grid.set_row_spacing(10);
  m_grid.set_column_spacing(10);
  set_title(title);
  set_transient_for(w);
  p_content=get_content_area();
  p_content->pack_start(m_grid);

  //------------------ checkbuttons: -------------------------------  
  int j=0;
  for(size_t i=0;i<max_options && i<m_option.size();i++){
    if(m_option[i].type()=="check"){
      m_checkbutton[i].set_label(m_option[i].name());
      m_checkbutton[i].set_active(m_option[i].value()=="true"?true:false);
      m_option[i].set_changed(false);
      m_grid.attach(m_checkbutton[i],j%4,j/4,1,1);
      j++;
    }
  }

  //------------------ buttons: -------------------------------  
  j=j+4 - j%4;
  for(size_t i=0;i<max_options && i<m_option.size();i++){
    if(m_option[i].type()=="button"){
      m_button[i].set_label(m_option[i].name());
      m_option[i].set_changed(false);
      m_grid.attach(m_button[i],j%4,j/4,1,1);
      j++;
    }
  }
  
  //------------------ spinbuttons: -------------------------------  
  j=j+4 - j%4;
  for(size_t i=0;i<max_options && i<m_option.size();i++){
    if(m_option[i].type()=="spin"){
      m_label[i].set_text(m_option[i].name());
      int lmin=stoi(m_option[i].minimum_value());
      int lmax=stoi(m_option[i].maximum_value());
      int lvalue=stoi(m_option[i].value());
      m_spinbutton[i].set_range(lmin,lmax);
      m_spinbutton[i].set_increments(1,50);
      m_spinbutton[i].set_value(lvalue);
      m_option[i].set_changed(false);
      m_grid.attach(m_label[i],j%4,j/4,1,1);j++;
      m_grid.attach(m_spinbutton[i],j%4,j/4,1,1);
      j++;
    }
  }
  
  //------------------ comboboxes: -------------------------------  
  j=j+4 - j%4;
  for(size_t i=0;i<max_options && i<m_option.size();i++){
    if(m_option[i].type()=="combo"){
      m_label[i].set_text(m_option[i].name());
      for(auto& v : m_option[i].var()){
	m_combobox[i].append(v);
      }
      if(m_option[i].value()==""){m_combobox[i].set_active_text(m_option[i].var()[0]);}
      else{m_combobox[i].set_active_text(m_option[i].value());}
      m_option[i].set_changed(false);
      m_grid.attach(m_label[i],j%4,j/4,1,1);j++;
      m_grid.attach(m_combobox[i],j%4,j/4,1,1);
      j++;
    }
  }

  
  //------------------ strings: ------------------------------------  
  j=j+4 - j%4;
  for(size_t i=0;i<max_options && i<m_option.size();i++){
    if(m_option[i].type()=="string"){
      m_label[i].set_text(m_option[i].name());
      //m_entry[i].set_has_frame();
      m_entry[i].set_text(m_option[i].value());
      m_option[i].set_changed(false);
      m_grid.attach(m_label[i],0,j/4,1,1);
      m_grid.attach(m_entry[i],1,j/4,3,1);
      j+=4;
    }
  }


  
  p_content->show_all_children();  
  add_button("OK",1);
  show();
}


OptionsDialog::~OptionsDialog()
{
}

